#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

/** List of errors that can be returned from the SDK. */
typedef NS_ENUM(NSInteger, RecaptchaErrorCode) {
  /** Unknown error occurred during the workflow. */
  RecaptchaErrorCodeUnknown = 0,
  /** reCAPTCHA encountered an internal issue. */
  RecaptchaErrorCodeInternalError = 1,
  /** There was a network error during one of the operations. */
  RecaptchaErrorCodeNetworkError = 2,
  /** There was a network error during one of the operations. */
  RecaptchaErrorCodeTwoFactorInvalidRequestToken = 3,
  /** There was a network error during one of the operations. */
  RecaptchaErrorCodeTwoFactorInvalidPin = 4,
  /** There was a network error during one of the operations. */
  RecaptchaErrorCodeTwoFactorChallengeExpired = 5,
  /** There was a network error during one of the operations. */
  RecaptchaErrorCodeTwoFactorPinMismatch = 6,
  /** There was a network error during one of the operations. */
  RecaptchaErrorCodeTwoFactorSessionExpired = 7,
  /** There was a network error during one of the operations. */
  RecaptchaErrorCodeTwoFactorAttemptsExhausted = 8,
  /** The user cancelled the verification. */
  RecaptchaErrorCodeTwoFactorVerificationCancelled = 9,
  /** Deprecated.*/
  RecaptchaErrorCodeTwoFactorSiteOnboardingIncomplete = 10,
  /** Deprecated.*/
  RecaptchaErrorCodeTwoFactorRecipientNotAllowed = 11,
  /** Deprecated.*/
  RecaptchaErrorCodeTwoFactorRecipientAbuseLimitExhausted = 12,
  /**
   *  2FA Workflow was aborted due to a problem. You can retrieve a reCAPTCHA
   *  token with more information on the `userInfo` property using
   *  `workflowAbortedTokenKey`.
   */
  RecaptchaErrorCodeTwoFactorAborted = 13,
};

/** Error class for reCAPTCHA Events. */
@interface RecaptchaError : NSError

@property(nonatomic) NSString *errorMessage;

/**
 * If the returned error code is of type `Aborted` you can use this key to
 * get the aborted token out of the `userInfo` property. The token is stored
 * inside an instance of `RecaptchaToken`.
 */
@property(class, nonatomic, readonly) NSString *workflowAbortedTokenKey;

/**
 * Builds a new reCAPTCHA Error object.
 */
- (instancetype)initWithDomain:(NSErrorDomain)domain
                          code:(NSInteger)code
                      userInfo:(NSDictionary<NSErrorUserInfoKey, id> *_Nullable)dict
                  errorMessage:(NSString *)message;

- (instancetype)init NS_UNAVAILABLE;

/** For debug purposes, prints a human friendly description of the error. */
- (NSString *)description;

@end

NS_ASSUME_NONNULL_END
