//
//  AuthV4ViewController
//  HIVE_SDK_Sample
//
//  Created by jhjang on 03/12/2018.
//  Copyright © 2018 gcp. All rights reserved.
//

import UIKit
import HIVEService
import HIVECore
import HIVEBase
import HIVEUI

// swiftlint:disable type_body_length

enum AuthV4Api: String, CaseIterable {
    case metaData = "Metadata"
    case setLogFilter = "SetLogFilter"
    case setup = "Setup"
    case signIn = "SignIn"
    case showSignIn = "Show SignIn"
    case signOut = "SignOut"
    case connect = "Connect"
    case disconnect = "Disconnect"
    case isAutoSignIn = "Is Auto SignIn"
    case getPlayerInfo = "Get Player Info"
    case selectConflict = "Select Conflict"
    case getProviderFriendsList = "Get Friends"
    case showConflict = "Show Conflict"
    case showProfile = "Show Profile"
    case checkMaintenance = "Check Maintenance"
    case checkBlacklist = "Check Blacklist"
    case setProviderChangeListener = "Set Provider Change Listener"
	case showInquiry = "show Inquiry"
    case showChatbot = "show Chatbot"
    case showMyInquiry = "show My Inquiry"
	case showTerms = "Show Terms"
    case reset = "Reset"
    case showAdultConfirm = "Show Adult Confirm"
    case showGameCenterPopup = "GameCenter Popup"
    case configurationGetPermissionViewData = "Configuration getPermissionViewData"
    case authPermissionViewData = "AuthV4 getPermissionViewData"
    case providerAppleReportScore = "GameCenter ReportScore"
    case providerAppleReportAchievement = "GameCenter Report Achievement"
    case providerAppleShowLeaderBoard = "GameCenter Show LeaderBoard"
    case providerAppleShowAchievements = "GameCenter Show Achievements"
    case providerAppleLoadAchievements = "GameCenter Load Achievements"
    case providerAppleResetAchievements = "GameCenter Reset Achievements"
    case showDeviceManagement = "Show DeviceManagement"
    case externalAuthAndSignInWithAuthKey = "External Auth & SignIn with authKey"
    case showCommunity = "Show Community"
    case realnameverification = "Real Name Verification"
}

enum AuthV4IDPType {
    case signIn
    case externalAuthAndSignInWithAuthKey
    case connect
    case disconnect
    case friends
}

class AuthV4ViewController: BaseViewController {
    enum externalAuthProvider: String, CaseIterable {
        case twitter = "CUSTOM_TWITTER"
        case naver = "CUSTOM_NAVER"
    }
    var externalProviderUserId = ""
    let apis: [AuthV4Api] = AuthV4Api.allCases
    var providerList: [ProviderType]?
    var actionSheetType: AuthV4IDPType = .signIn
    var conflictPlayer: PlayerInfo?
    
    var actionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "Provider", message: "Select Provider", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        if let providerList = self.providerList {
            for item in providerList {
                let actionStyle: UIAlertAction.Style = item == .Auto ? .destructive : .default
                
                let action = UIAlertAction(title: item.description, style: actionStyle) { (_) in
                    switch self.actionSheetType {
                    case .connect:
                        AuthV4Interface.connect(item) { (result, playerInfo) in
                            let log = """
                            === AuthV4Interface.connect() ===
                            
                            result - \(result)
                            playerInfo - \(String(describing: playerInfo))
                            """
                            
                            self.logDelegate?.addLog(log: log)
                            
                            if result.getErrorCode() == .conflictPlayer {
                                self.conflictPlayer = playerInfo
                            }
                        }
                    case .signIn:
                        AuthV4Interface.signIn(item) { (result, playerInfo) in
                            let log = """
                            === AuthV4Interface.signIn() ===
                            
                            result - \(result)
                            result.needExit() - \(result.needExit())
                            playerInfo - \(String(describing: playerInfo))
                            providerInfoData - \(String(describing: playerInfo?.providerInfoData))
                            """
                            
                            self.logDelegate?.addLog(log: log)
                            // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                        }
                        
                    case .disconnect:
                        AuthV4Interface.disconnect(item, handler: { (result) in
                            let log = """
                            === AuthV4Interface.disconnect() ===
                            
                            result - \(result)
                            """
                            
                            self.logDelegate?.addLog(log: log)
                        })
                    
                    case .friends:
                        AuthV4Interface.getProviderFriendsList(item, handler: { (result, providerType, friendsList) in
                            let log = """
                            === AuthV4Interface.getProviderFriendsList() ===
                            
                            result - \(result)
                            providerType - \(providerType)
                            friendsList - \(String(describing: friendsList))
                            """
                            
                            self.logDelegate?.addLog(log: log)
                        })
                    case .externalAuthAndSignInWithAuthKey:
                        // do nothing
                        break
                    }
                }
                actionSheet.addAction(action)
            }
        }
        
        return actionSheet
    }
    
    var logTypeSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "LogFilter", message: "Select LogType", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        let logTypeList: [LogType] = [ .Verbose, .Debug, .Info, .Warning, .Error, .None]
        
        for item in logTypeList {
            let actionStyle: UIAlertAction.Style = item == .Verbose ? .destructive : .default
            
            var itemName: String {
                switch item {
                case .Verbose:
                    return "Verbose"
                case .Debug:
                    return "Debug"
                case .Info:
                    return "Info"
                case .Warning:
                    return "Warning"
                case .Error:
                    return "Error"
                case .None:
                    return "None"
                @unknown default:
                    return "Debug"
                }
            }
            
            let action = UIAlertAction(title: itemName, style: actionStyle) { (_) in
                LogInterface.setLogFilter(LogFilter.init(coreLog: item, serviceLog: item))
            }
            actionSheet.addAction(action)
        }
        
        return actionSheet
    }

    var externalAuthProviderActionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "Custom Provider", message: "Select Custom Provider", preferredStyle: .alert)
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)

        actionSheet.addTextField { textField in
            textField.placeholder = "external provider user id"
            textField.addTarget(self, action: #selector(self.textFieldDidChange(_:)), for: .editingChanged)
        }

        for provider in externalAuthProvider.allCases {
            let action = UIAlertAction(title: provider.rawValue, style: .default) { (_) in

                guard self.actionSheetType == .externalAuthAndSignInWithAuthKey else {
                    return
                }

                // 게임 서버 인증 통신
                ExternalAuth.shared.requestSignInIdp(zoneType: ConfigurationInterface.getZone(),
                                                     appId: ConfigurationInterface.getAppId(),
                                                     idpId: provider.rawValue,
                                                     idpUserId: self.externalProviderUserId) { response in

                    self.logDelegate?.addLog(log: """
                        === ExternalAuth.requestSignInIdp() ===
                        "[ExternalAuth] response = \(response?.description ?? "")
                        """)

                    guard let response = response, response.isSuccess() else {
                        self.logDelegate?.addLog(log: "[ExternalAuth] response failed.")
                        return
                    }

                    guard
                        !response.authKey.isEmpty
                    else {
                        self.logDelegate?.addLog(log: "[ExternalAuth] authKey is empty.")
                        return
                    }
                    
                    AuthV4Interface.signInWithAuthKey(response.authKey) { (result, playerInfo) in
                        let log = """
                        === AuthV4Interface.signInWithName() ===

                        result - \(result)
                        result.needExit() - \(result.needExit())
                        playerInfo - \(String(describing: playerInfo))
                        """

                        self.logDelegate?.addLog(log: log)
                        // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                    }
                }

            }
            actionSheet.addAction(action)
        }

        return actionSheet
    }
    
    var communityUrlActionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "CommunityUrl", message: "Select CommunityUrl", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        let urlList: [String] = ["withhive.com", "community.withhive.com", "community.withhive.comm", "community.wwithhive.com", "not set"]

        for url in urlList {
            let actionStyle: UIAlertAction.Style = .default
            let title = url
            let action = UIAlertAction(title: title, style: actionStyle) { (_) in
                if url != "not set" {
                    ConfigurationInterface.setHiveCommunityUrl("https://\(url)/hivesdk/login_webview")
                }
                self.present(self.communityViewTypeActionSheet, animated: true, completion: nil)
            }
            actionSheet.addAction(action)
        }
        
        return actionSheet
    }
    
    var communityViewTypeActionSheet: UIAlertController {
        let actionSheet = UIAlertController(title: "ViewType", message: "Select ViewType", preferredStyle: .actionSheet)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        actionSheet.addAction(cancelAction)
        
        let viewTypeList: [SocialViewType] = [.FullScreen, .Frame]

        for viewType in viewTypeList {
            let actionStyle: UIAlertAction.Style = .default
            let title = """
                \(viewType.description)
                """
            let action = UIAlertAction(title: title, style: actionStyle) { (_) in
                SocialV4Interface.showCommunity(.HIVE, viewType: viewType) { result in
                    let log = """
                    === SocialV4Interface.showCommunity() ===
                    
                    result - \(result)
                    """
                    
                    self.logDelegate?.addLog(log: log)
                }
            }
            actionSheet.addAction(action)
        }
        
        return actionSheet
    }
    
    
    @IBOutlet weak var tableView: UITableView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.tableView.contentInset = UIEdgeInsets.init(top: 44, left: 0, bottom: 0, right: 0)
    }
    
    func callAPI(api: AuthV4Api) {
        switch api {
        case .metaData: 
            // let keyList = ["bool", "int", "double", "string", "stringArray", "intArray", "dictionary"]
            let keyList = ["game_server_url", "white_list", "hive_country", "timezone"]
            var resultDict = [String: Any]()
            for testKey in keyList {
                ConfigurationInterface.getMetaData(testKey, forceReload: false) { _, value  in
                    print("key : \(testKey) \n     value : \(value)\n")
                    resultDict[testKey] = value
                }
            }
            
            print("resultDict : \(resultDict)")
        case .setLogFilter:
            self.present(self.logTypeSheet, animated: true, completion: nil)
        case .setup:
            AuthV4Interface.setup { (result, isAutoSignIn, did, providerList) in
                let log = """
                === AuthV4Interface.setup() ===
                
                result - \(result)
                result.needExit() - \(result.needExit())
                isAutoSignIn - \(isAutoSignIn)
                did - \(String(describing: did))
                providerList - \(String(describing: providerList))
                """
                // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                
                self.logDelegate?.addLog(log: log)
                
                self.providerList = providerList?.compactMap { ProviderType(rawValue: $0) }
                
                // Test
                self.providerList?.append(.WeChat)
                self.providerList?.append(.VK)
                self.providerList?.append(.QQ)
                self.providerList?.append(.Line)
                self.providerList?.append(.Guest)
                // WeverseTest
                self.providerList?.append(.Weverse)
                
                if isAutoSignIn {
                    self.providerList?.insert(.Auto, at: 0)
                }
            }
            
        case .signIn:
            self.actionSheetType = .signIn
            self.present(self.actionSheet, animated: true, completion: nil)
            
        case .signOut:
            AuthV4Interface.signOut { (result) in
                let log = """
                === AuthV4Interface.signOut() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
            
        case .connect:
            self.actionSheetType = .connect
            self.present(self.actionSheet, animated: true, completion: nil)
            
        case .disconnect:
            self.actionSheetType = .disconnect
            self.present(self.actionSheet, animated: true, completion: nil)
            
        case .isAutoSignIn:
            let isAutoSignIn = AuthV4Interface.isAutoSignIn()
            
            let log = """
            === AuthV4Interface.isAutoSignIn() ===
            
            isAutoSignIn - \(isAutoSignIn)
            """
            
            self.logDelegate?.addLog(log: log)
        case .getPlayerInfo:
            let playerInfo = AuthV4Interface.getPlayerInfo()
            
            let log = """
            === AuthV4Interface.getPlayerInfo() ===
            
            playerInfo - \(String(describing: playerInfo))
            """
            
            self.logDelegate?.addLog(log: log)
            
        case .getProviderFriendsList:
            self.actionSheetType = .friends
            self.present(self.actionSheet, animated: true, completion: nil)
        case .selectConflict:
            self.showConflict()
        case .showSignIn:
            AuthV4Interface.showSignIn { (result, playerInfo) in
                let log = """
                === AuthV4Interface.showSignIn() ===
                
                result - \(result)
                result.needExit() - \(result.needExit())
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
                // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
            }
        case .showProfile:
            AuthV4Interface.showProfile(AuthV4Interface.getPlayerInfo()?.playerID ?? -1) { (result) in
                let log = """
                === AuthV4Interface.showProfile() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showConflict:
            let currentUserData: [String: Any] = [
                "player_id": AuthV4Interface.getPlayerInfo()?.playerID ?? -1,
                "game_data": [
                    "Gold": "1024",
                    "Gem": "110"
                ]
            ]
            
            let conflictUserData: [String: Any] = [
                "player_id": self.conflictPlayer?.playerID ?? -1,
                "game_data": [
                    "Gold": "2048",
                    "Gem": "220"
                ]
            ]
            
            AuthV4Interface.showConflictSelection(currentUserData, conflictPlayerData: conflictUserData) { (result, playerInfo) in
                let log = """
                === AuthV4Interface.showConflictSelection() ===
                
                result - \(result)
                result.needExit() - \(result.needExit())
                playerInfo - \(String(describing: playerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
                // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
            }
        
        case .reset:
            AuthV4Interface.reset()
            
        case .checkMaintenance:
            AuthV4Interface.checkMaintenance(true) { (result, maintenances) in
                let log = """
                === AuthV4Interface.checkMaintenance() ===
                
                result - \(result)
                result.needExit() - \(result.needExit())
                maintenances - \(String(describing: maintenances))
                """
                
                self.logDelegate?.addLog(log: log)
                // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
            }
            
        case .checkBlacklist:
            AuthV4Interface.checkBlacklist(true) { (result, blacklist) in
                let log = """
                === AuthV4Interface.checkBlacklist() ===
                
                result - \(result)
                result.needExit() - \(result.needExit())
                blacklist - \(String(describing: blacklist))
                """
                
                self.logDelegate?.addLog(log: log)
                // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
            }
        case .setProviderChangeListener:
            AuthV4Interface.setProviderChangedListener { (result, providerInfo) in
                let log = """
                === AuthV4Interface.setProviderChangedListener() ===
                
                result - \(result)
                providerInfo - \(String(describing: providerInfo))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .showInquiry:
			AuthV4Interface.showInquiry { (result) in
				let log = """
				=== AuthV4Interface.showInquiry() ===
				
				result - \(result)
				"""
				
				self.logDelegate?.addLog(log: log)
			}
        case .showChatbot:
            AuthV4Interface.showChatbotInquiry("{\"evt_code\":1000,\"p1\":\"random\"}") { (result) in
                let log = """
                === AuthV4Interface.showChatBotInquiry() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                
            }
        case .showMyInquiry:
            AuthV4Interface.showMyInquiry { (result) in
                let log = """
                === AuthV4Interface.showMyInquiry() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                
            }
        case .showTerms:
			AuthV4Interface.showTerms { (result) in
				let log = """
				=== AuthV4Interface.showTerms() ===
				
				result - \(result)
				"""
				
				self.logDelegate?.addLog(log: log)
			}
        case .showAdultConfirm:
            AuthV4Interface.showAdultConfirm { (result) in
                let log = "result - \(result)"
                self.logDelegate?.addLog(log: log)
            }
        case .showGameCenterPopup:
            AuthV4Interface.showGameCenterLoginCancelDialog { (result) in
                let log = """
                === AuthV4Interface.showGameCenterLoginCancelDialog() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .configurationGetPermissionViewData:
            let result = ConfigurationInterface.getPermissionViewData(.KO)
            self.logDelegate?.addLog(log: result.description)
        case .authPermissionViewData:
            AuthV4Interface.requestPermissionViewData { (_, data) in
                
                self.logDelegate?.addLog(log: data?.description ?? "")
            }
        case .providerAppleReportScore:
            ProviderApple.shared.reportScore("00", leaderboardIdentifier: "grp.com.com2us.hivesdk.10hit") { (result) in
                let log = """
                === ProviderApple.shared.reportScore() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .providerAppleReportAchievement:
            ProviderApple.shared.reportAchievement(
                "100",
                showsCompletionBanner: true,
                achievementIdentifier: "grp.com.com2us.hivesdk.10hit"
            ) { (result) in
                let log = """
                === ProviderApple.shared.reportAchievement() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .providerAppleShowLeaderBoard:
            ProviderApple.shared.showLeaderboard { (result) in
                let log = """
                === ProviderApple.shared.showLeaderboard() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                
            }
        case .providerAppleShowAchievements:
            ProviderApple.shared.showAchievements { (result) in
                let log = """
                === ProviderApple.shared.showAchievements() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                
            }
        case .providerAppleLoadAchievements:
            ProviderApple.shared.loadAchievements { (result, _) in
                let log = """
                === ProviderApple.shared.loadAchievements() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                
            }
        case .providerAppleResetAchievements:
            ProviderApple.shared.resetAchievements { (result) in
                let log = """
                === ProviderApple.shared.resetAchievements() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
                //
            }
        case .showDeviceManagement:
            AuthV4Interface.showDeviceManagement { result in
                let log = """
                === AuthV4Interface.showDeviceManagement() ===
                
                result - \(result)
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .externalAuthAndSignInWithAuthKey:
            self.actionSheetType = .externalAuthAndSignInWithAuthKey
            self.present(self.externalAuthProviderActionSheet, animated: true, completion: nil)
        case .showCommunity:
            self.present(self.communityUrlActionSheet, animated: true, completion: nil)
        case .realnameverification:
            RealNameVerification.shared.verify {
                print("success")
            }
            
        }
    }

    @objc func textFieldDidChange(_ textField: UITextField) {
        switch self.actionSheetType {
        case .externalAuthAndSignInWithAuthKey:
            self.externalProviderUserId = textField.text ?? ""
        default:
            // do nothing
            return
        }
    }
    
    private func showConflict() {
        if self.conflictPlayer != nil {
            let alertViewController = UIAlertController(title: "Conflict", message: "select", preferredStyle: .alert)
            
            let selectAction = UIAlertAction(title: "switch", style: .cancel) { _ in
                AuthV4Interface.selectConflict(self.conflictPlayer!.playerID, handler: { (result, playerInfo) in
                    let log = """
                    === AuthV4Interface.selectConflict() ===
                    
                    result - \(result)
                    result.needExit() - \(result.needExit())
                    playerInfo - \(String(describing: playerInfo))
                    """
                    
                    self.logDelegate?.addLog(log: log)
                    // result.needExit() true일 경우 적절한 앱 종료 이벤트를 처리. exit(0)
                })
            }
            
            let resolveAction = UIAlertAction(title: "resolve", style: .default) { (result) in
                AuthV4Interface.resolveConflict({ (result) in
                    let log = """
                    === AuthV4Interface.resolveConflict() ===
                    
                    result - \(result)
                    """
                    
                    self.logDelegate?.addLog(log: log)
                })                
            }
            
            alertViewController.addAction(selectAction)
            alertViewController.addAction(resolveAction)
            
            self.present(alertViewController, animated: true, completion: nil)
        } else {
            Logger.d("Conflict info is nil.")
        }
    }
}

extension AuthV4ViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        self.apis.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "ApiCell") as? AuthV4TableViewCell else {
            preconditionFailure("Not supported cell type")
        }
        
        cell.configureCell(api: self.apis[indexPath.row])
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.callAPI(api: apis[indexPath.row])
        tableView.deselectRow(at: indexPath, animated: true)
    }
}
