//
//  MainPageViewController.swift
//  HIVE_SDK_Sample
//
//  Created by jhjang on 04/12/2018.
//  Copyright © 2018 gcp. All rights reserved.
//

import UIKit


class MainPageViewController: UIViewController {
	var pages = [UIViewController]()
	var contentsStoryboardIds = [
        "AuthV4ViewController",
        "AuthV4HelperViewController",
        "AuthViewController",
        "PromotionViewController",
        "IAPV4ViewController",
        "PushViewController",
        "PlatformHelperViewController",
        "AnalyticesViewController",
        "DataStoreViewController"
    ]
	
	let storyboardName = "Main"
	var pageContainer: UIPageViewController!
	var currentIndex: Int = 0
	var pendingIndex: Int = 0
	var returnIndex: Int?
	let logViewIndex = 0
	
	@IBOutlet weak var pageControl: UIPageControl!
	@IBOutlet weak var logButton: UIBarButtonItem!
	
	override func viewWillAppear(_ animated: Bool) {
		super.viewWillAppear(animated)
		
		self.navigationController?.interactivePopGestureRecognizer?.isEnabled = false
		self.navigationController?.navigationItem.setHidesBackButton(true, animated: false)
	}
	
	override func viewDidLoad() {
		super.viewDidLoad()
		
		setupPages()
	}
	
	@IBAction func didLogButtonTouched(_ sender: Any) {
		Logger.d("")
		
		if currentIndex == logViewIndex {
			if let index = self.returnIndex {
				pageContainer.setViewControllers([pages[index]], direction: .forward, animated: true) { (_) in
					self.returnIndex = nil
				}
            }
		}
		
		self.returnIndex = self.currentIndex
		let direction: UIPageViewController.NavigationDirection = currentIndex < logViewIndex ? .forward : .reverse
		
		guard let firstPage = pages.first else {
			preconditionFailure("First page must be exist.")
		}
		
		pageContainer.setViewControllers([firstPage], direction: direction, animated: true) { (_) in
			self.currentIndex = 0
		}
	}
	
	private func setupPages() {
		pages.append(self.getViewcontroller(withIdentifier: "LogViewController"))
		
		for storyboardId in contentsStoryboardIds {
            if let vc = self.getViewcontroller(withIdentifier: storyboardId) as? BaseViewController {
                vc.logDelegate = self
                pages.append(vc)
            }
		}
		
		pageContainer = UIPageViewController(transitionStyle: .scroll, navigationOrientation: .horizontal, options: nil)
		pageContainer.delegate = self
		pageContainer.dataSource = self
		
		let firstPage = pages[0]
		
		pageContainer.setViewControllers([firstPage], direction: .forward, animated: true, completion: nil)
        self.title = firstPage.title
		
		view.addSubview(pageContainer.view)
		
		view.bringSubviewToFront(pageControl)
		pageControl.numberOfPages = pages.count
		pageControl.currentPage = 0
	}
	
	private func getViewcontroller(withIdentifier identifier: String) -> UIViewController {
		UIStoryboard(name: self.storyboardName, bundle: nil).instantiateViewController(withIdentifier: identifier)
	}
}

// MARK: LogDelegate
extension MainPageViewController: LogProtocol {
	func addLog(log: String) {
		guard let logVC = pages.first as? LogViewController else {
			preconditionFailure("First view controller must be LogViewController")
		}
        
        DispatchQueue.main.async {
            Logger.d("\(log)")
            logVC.logTextView.insertText("\(log)\n\n")
        }
	}
}

// MARK: UIPageViewControllerDelegate
extension MainPageViewController: UIPageViewControllerDelegate {
	func pageViewController(_ pageViewController: UIPageViewController, willTransitionTo pendingViewControllers: [UIViewController]) {
        if let index = pages.firstIndex(of: pendingViewControllers.first!) {
			pendingIndex = index
		}
	}
	
	func pageViewController(_ pageViewController: UIPageViewController,
                            didFinishAnimating finished: Bool,
                            previousViewControllers: [UIViewController],
                            transitionCompleted completed: Bool) {
		if completed {
			currentIndex = pendingIndex
			pageControl.currentPage = currentIndex
			self.navigationItem.title = pages[currentIndex].title
		}
		
	}

}

// MARK: UIPageViewControllerDatasource
extension MainPageViewController: UIPageViewControllerDataSource {
	func pageViewController(_ pageViewController: UIPageViewController, viewControllerBefore viewController: UIViewController) -> UIViewController? {
		
        guard let currentIndex = self.pages.firstIndex(of: viewController), currentIndex != 0 else {
			return nil
		}
		
		let previousIndex = abs((currentIndex - 1) % pages.count)
		
		return pages[previousIndex]
	}
	
	func pageViewController(_ pageViewController: UIPageViewController, viewControllerAfter viewController: UIViewController) -> UIViewController? {
		
        guard let currentIndex = self.pages.firstIndex(of: viewController), currentIndex != pages.count-1 else {
			return nil
		}
		
		let nextIndex = abs((currentIndex + 1) % pages.count)
		return pages[nextIndex]
	}
}
