//
//  ConfigurationAPIKeyViewController.swift
//  HIVE_SDK_Sample
//
//  Created by daunjoung on 2021/04/07.
//  Copyright © 2021 gcp. All rights reserved.
//

import UIKit
import HIVEService

class ConfigurationAPIKeyViewController: UIViewController {
    @IBOutlet weak var configTableView: UITableView!
    override func viewDidLoad() {
        super.viewDidLoad()

        // Do any additional setup after loading the view.
    }
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        SampleConfigAPIKey.resetDefaultKey()
        configTableView.reloadData()
    }
    @IBAction func onClickReset(_ sender: Any) {
        SampleConfigAPIKey.resetDefaultKey()
        configTableView.reloadData()
    }
    
}


extension ConfigurationAPIKeyViewController: UITableViewDelegate {
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let data = SampleConfigAPIKey.getData(at: indexPath)
        if SampleConfigAPIKey.isEditable(at: indexPath) {
            let alert = UIAlertController(title: data.title, message: "", preferredStyle: .alert)
            
            
            let ok = UIAlertAction(title: "Save", style: .default) { (_) in
                let changedValue = alert.textFields?[0].text ?? ""
                SampleConfigAPIKey.setData(at: indexPath, data: changedValue)
                self.configTableView.reloadData()
                
            }
            
            let cancel = UIAlertAction(title: "Cancel", style: .cancel) { (_) in
                self.configTableView.reloadData()
            }
            
            alert.addAction(ok)
            alert.addAction(cancel)
            
            alert.addTextField { (textField) in
                textField.text = data.value
            }
            DispatchQueue.main.async {
                self.present(alert, animated: true, completion: nil)
            }
        } else {
            let alert = UIAlertController(title: "Error", message: "This key can not edit", preferredStyle: .alert)
            
            let ok = UIAlertAction(title: "OK", style: .default)
            alert.addAction(ok)
            
            DispatchQueue.main.async {
                self.present(alert, animated: true, completion: nil)
            }
            
        }
    }
}

extension ConfigurationAPIKeyViewController: UITableViewDataSource {
    func numberOfSections(in tableView: UITableView) -> Int {
        SampleConfigAPIKey.getSectionCnt()
    }
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        SampleConfigAPIKey.getRowCnt(at: section)
    }
    
    func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        SampleConfigAPIKey.getSectionTitle(at: section)
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = UITableViewCell(style: .subtitle, reuseIdentifier: nil)
        
        let data = SampleConfigAPIKey.getData(at: indexPath)
        cell.textLabel?.text = data.title
        cell.detailTextLabel?.text = data.value
        
        if !SampleConfigAPIKey.isEditable(at: indexPath) {
            cell.textLabel?.textColor = .gray
        }
        
        return cell
    }
}


    
class SampleConfigAPIKey {
    private static let SECTION_GOOGLE = 0
    private static let SECTION_FACEBOOK = 1
    private static let SECTION_QQ = 2
    private static let SECTION_VK = 3
    private static let SECTION_WECHAT = 4
    private static let SECTION_LINE = 5
    private static let SECTION_ADJUST = 6
    private static let SECTION_SINGULAR = 7
    private static let SECTION_APPSFLYER = 8
    private static let SECTION_END = SECTION_APPSFLYER
    
    private static let SAMPLEKEY_GoogleServerClientID = "331526026701-42uadtkeght91f7saspdg92gtdill6mv.apps.googleusercontent.com"
    private static let SAMPLEKEY_WeChatSecretKey = "b5a73b0ee67d546bf876851944eda308"
    private static let SAMPLEKEY_AdjustKey = "8szd9kddtt6o"
    private static let SAMPLEKEY_AdjustSecretID = "1"
    private static let SAMPLEKEY_AdjustInfo1 = "2071917595"
    private static let SAMPLEKEY_AdjustInfo2 = "254356864"
    private static let SAMPLEKEY_AdjustInfo3 = "427896519"
    private static let SAMPLEKEY_AdjustInfo4 = "820983601"
    private static let SAMPLEKEY_SingularKey = "9b7ff73a0fc13c6c9397a9881dba2673"
    private static let SAMPLEKEY_AppsFlyerKey = "G6yEQjHw4CrEN8sEzF9onU"
    
    private static var _googleServerClientID      = "" {
        willSet(newVal) {
            ConfigurationInterface.setConfigurations(configType: .googleServerClientId, value: newVal)
        }
    }
    private static var _wechatSecretKey     = "" {
        willSet(newVal) {
            ConfigurationInterface.setConfigurations(configType: .wechatSecret, value: newVal)
        }
    }
    private static var _adjust              = (key: "",
                                               secretId: "",
                                               info1: "",
                                               info2: "",
                                               info3: "",
                                               info4: "") {
        willSet(newVal) {
            ConfigurationInterface.setConfigurations(configType: .adjustKey, value: newVal.key)
            ConfigurationInterface.setConfigurations(configType: .adjustSecretId, value: newVal.secretId)
            ConfigurationInterface.setConfigurations(configType: .adjustInfo1, value: newVal.info1)
            ConfigurationInterface.setConfigurations(configType: .adjustInfo2, value: newVal.info2)
            ConfigurationInterface.setConfigurations(configType: .adjustInfo3, value: newVal.info3)
            ConfigurationInterface.setConfigurations(configType: .adjustInfo4, value: newVal.info4)
        }
    }
    private static var _singularKey         = "" {
        willSet(newVal) {
            ConfigurationInterface.setConfigurations(configType: .singularKey, value: newVal)
        }
    }
    private static var _appsFlyerKey        = "" {
        willSet(newVal) {
            ConfigurationInterface.setConfigurations(configType: .appsflyerKey, value: newVal)
        }
    }
    
    static func getSectionCnt() -> Int {
        SECTION_END
    }
    static func getSectionTitle(at section: Int) -> String {
        switch section {
        case SECTION_GOOGLE:
            return "Google"
        case SECTION_FACEBOOK:
            return "Facebook"
        case SECTION_QQ:
            return "QQ"
        case SECTION_VK:
            return "VK"
        case SECTION_WECHAT:
            return "WeChat"
        case SECTION_LINE:
            return "Line"
        case SECTION_ADJUST:
            return "Adjust"
        case SECTION_SINGULAR:
            return "Singular"
        case SECTION_APPSFLYER:
            return "AppsFlyer"
        default:
            return ""
        }
    }
    static func getRowCnt(at section: Int) -> Int {
        switch section {
        case SECTION_GOOGLE:
            return 1
        case SECTION_FACEBOOK:
            return 0
        case SECTION_QQ:
            return 0
        case SECTION_VK:
            return 0
        case SECTION_WECHAT:
            return 1
        case SECTION_LINE:
            return 0
        case SECTION_ADJUST:
            return 6
        case SECTION_SINGULAR:
            return 1
        case SECTION_APPSFLYER:
            return 1
        default:
            return 0
        }
    }
    static func getData(at indexPath: IndexPath) -> (title: String, value: String) {
        switch (indexPath.section, indexPath.row) {
        case (SECTION_GOOGLE, 0):
            return (title: "ServerClientID", value: _googleServerClientID)
        case (SECTION_WECHAT, 0):
            return (title: "SecretKey", value: _wechatSecretKey)
        case (SECTION_ADJUST, 0):
            return (title: "Key", value: _adjust.key)
        case (SECTION_ADJUST, 1):
            return (title: "SecretID", value: _adjust.secretId)
        case (SECTION_ADJUST, 2):
            return (title: "Info1", value: _adjust.info1)
        case (SECTION_ADJUST, 3):
            return (title: "Info2", value: _adjust.info2)
        case (SECTION_ADJUST, 4):
            return (title: "Info3", value: _adjust.info3)
        case (SECTION_ADJUST, 5):
            return (title: "Info4", value: _adjust.info4)
        case (SECTION_SINGULAR, 0):
            return (title: "Key", value: _singularKey)
        case (SECTION_APPSFLYER, 0):
            return (title: "Key", value: _appsFlyerKey)
        default:
            return (title: "", value: "")
        }
    }
    
    static func setData(at indexPath: IndexPath, data: String) {
        switch (indexPath.section, indexPath.row) {
        case (SECTION_GOOGLE, 0):
            _googleServerClientID = data
        case (SECTION_WECHAT, 0):
            _wechatSecretKey = data
        case (SECTION_ADJUST, 0):
            _adjust.key = data
        case (SECTION_ADJUST, 1):
            _adjust.secretId = data
        case (SECTION_ADJUST, 2):
            _adjust.info1 = data
        case (SECTION_ADJUST, 3):
            _adjust.info2 = data
        case (SECTION_ADJUST, 4):
            _adjust.info3 = data
        case (SECTION_ADJUST, 5):
            _adjust.info4 = data
        case (SECTION_SINGULAR, 0):
            _singularKey = data
        case (SECTION_APPSFLYER, 0):
            _appsFlyerKey = data
        default:
            print("Failed to set data")
        }
        
    }
    
    
    static func resetDefaultKey() {
        _googleServerClientID   = SAMPLEKEY_GoogleServerClientID
        _wechatSecretKey        = SAMPLEKEY_WeChatSecretKey
        _adjust                 = (key: SAMPLEKEY_AdjustKey,
                                   secretId: SAMPLEKEY_AdjustSecretID,
                                   info1: SAMPLEKEY_AdjustInfo1,
                                   info2: SAMPLEKEY_AdjustInfo2,
                                   info3: SAMPLEKEY_AdjustInfo3,
                                   info4: SAMPLEKEY_AdjustInfo4)
        _singularKey            = SAMPLEKEY_SingularKey
        _appsFlyerKey           = SAMPLEKEY_AppsFlyerKey
        
    }
    
    static func isEditable(at indexPath: IndexPath) -> Bool {
        switch (indexPath.section, indexPath.row) {
        case (SECTION_GOOGLE, 0),
            (SECTION_WECHAT, 0),
            (SECTION_ADJUST, 0),
            (SECTION_ADJUST, 1),
            (SECTION_ADJUST, 2),
            (SECTION_ADJUST, 3),
            (SECTION_ADJUST, 4),
            (SECTION_ADJUST, 5),
            (SECTION_SINGULAR, 0),
            (SECTION_APPSFLYER, 0):
            return true
        default:
            return false
        }
    }
    
}
    
    
    

