//
//  ConfigurationViewController.swift
//  HIVE_SDK_Sample
//
//  Created by jhjang on 04/12/2018.
//  Copyright © 2018 gcp. All rights reserved.
//

import UIKit
import HIVEService
import HIVECore
import AdSupport

enum ConfigurationCellSection: Int, CaseIterable {
    case label = 0
    case textField = 1
    case actionSheet = 2
    case switchCell = 3
}

class ConfigurationViewController: BaseViewController {
	
	@IBOutlet weak var tableView: UITableView!
    var selectedIndexPath: IndexPath?
    
    var advertisingID: String { ASIdentifierManager.shared().advertisingIdentifier.uuidString }
    
    // Label
    var sdkVersion = ("SDK Version", ConfigurationInterface.getHiveSDKVersion())
    lazy var ifa = ("IFA", self.advertisingID)
    
    // TextField
    var appID = ("App ID", ConfigurationInterface.getAppId())
    var hiveCertificationKey = ("hiveCertificationKey", "")
    var serverID = ("Server ID", ConfigurationInterface.getServerId())
    var updateServerID = ("Update Server ID", ConfigurationInterface.getServerId())
    var logUserID = ("Log User ID", "")
    var channel = ("Channel", ConfigurationInterface.getChannel())
    
    // Switch
    var log = ("Log", ConfigurationInterface.getUseLog())
    var ageGateU13 = ("AgeGateU13", ConfigurationInterface.getAgeGateU13())
    var hivePermissionViewOn = ("hivePermissionViewOn", true)
    var hiveTheme = ("isDarkMode", false)
    var useHercules = ("Use Hercules", true)

    // ActionSheet
    var zone = ("Zone", ConfigurationInterface.getZone())
    var language = ("Language", ConfigurationInterface.getGameLanguage())
    var selectedResourceLanguage = "en"
    var getPermissionViewData = "getPermissionViewData"
    var setGameLanguage = "setGameLanguage"
    var updateGameLanguage = "updateGameLanguage"
    var selectedGameLanguage = "en"
    var getHiveOrientation = "hiveOrientation"
    var selectedOrientation = ""
    
    
    var languages = [
        "English": "en",
        "Deutsch": "de",
        "español": "es",
        "français": "fr",
        "Indonesia": "id",
        "italiano": "it",
        "日本語": "ja",
        "한국어": "ko",
        "português": "pt",
        "русский": "ru",
        "ไทย": "th",
        "Türkçe": "tr",
        "Tiếng Việt": "vi",
        "简体中文": "zh-Hans",
        "繁體中文": "zh-Hant",
        "arab": "ar",
        "Not set": "null"
    ]
    
    var zones = ZoneType.allCases
    
    override func viewDidLoad() {
        super.viewDidLoad()
        self.hiveCertificationKey.1 =
        "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9" +
        ".eyJpc3MiOiJIaXZlIiwiaWF0IjoxNjAyMDU2NzI2LCJqdGkiOiIxODczMTExMzIwIn0" +
        ".3soFiHTPlObCoqR5xX9ZeOQTSvnHrHDHWmopP3QfWtY"
    }
	
	override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
		if segue.identifier == "EditSegue" {
            guard let editVC = segue.destination as? ConfigurationEditViewController else { return }
            
            if let indexPath = self.selectedIndexPath {
                
                switch indexPath.row {
                case 0:
                    // app id
                    editVC.labelTitle = self.appID.0
                    editVC.content = self.appID.1
                case 1:
                    // server id
                    editVC.labelTitle = self.serverID.0
                    editVC.content = self.serverID.1
                case 2:
                    // log user id
                    editVC.labelTitle = self.logUserID.0
                    editVC.content = self.logUserID.1
                case 3:
                    // hiveCertificationKey
                    editVC.labelTitle = self.hiveCertificationKey.0
                    editVC.content = self.hiveCertificationKey.1
                case 4:
                    editVC.labelTitle = self.updateServerID.0
                    editVC.content = self.updateServerID.1
                case 5:
                    editVC.labelTitle = self.channel.0
                    editVC.content = self.channel.1
                default:
                    break
                }
            
                editVC.delegate = self
            }
		}
        
        if segue.identifier == "OK" {
            ConfigurationInterface.setHiveCertificationKey(hiveCertificationKey.1)
        }
	}
    
    private func getActionSheet(index: Int) -> UIAlertController {
        UIAlertController(title: "Select Item", message: nil, preferredStyle: .actionSheet)
    }
}

// MARK: Configuration Edit Delegate {
extension ConfigurationViewController: ConfigurationEditDelegate {
    func didTextFieldChanged(value: String) {
        switch self.selectedIndexPath!.row {
        case 0:
            // app id
            self.appID.1 = value
            ConfigurationInterface.setAppId(value)
        case 1:
            // server id
            self.serverID.1 = value
            self.updateServerID.1 = value
            ConfigurationInterface.setServerId(value)
        case 2:
            // log user id
            self.logUserID.1 = value
        case 3:// hiveCertificationKey
            self.hiveCertificationKey.1 = value
            ConfigurationInterface.setHiveCertificationKey(value)
        case 4: // updateServerId
            self.updateServerID.1 = value
            self.serverID.1 = value
            ConfigurationInterface.updateServerId(value)
        case 5: // set Channel
            self.channel.1 = value
            ConfigurationInterface.setChannel(value)
        default:
            break
        }
        self.selectedIndexPath = nil
        
        self.tableView.reloadData()
    }
}

// MARK: UITableView Delegate, Datasource
extension ConfigurationViewController: UITableViewDelegate, UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        ConfigurationCellSection.allCases.count
    }
    
	func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        
        guard
            let sectionType = ConfigurationCellSection(rawValue: section)
        else {
            preconditionFailure("Not supported section type")
        }
        
        switch sectionType {
        case .label:
            // Label cell
            return 2
        case .textField:
            // Textfield Cell
            return 6
        case .actionSheet:
            // Actionsheet Cell
            return 5
        case .switchCell:
            // Switch Cell
            return 5
        }
	}
	
	func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cellIdentifiers = ["LabelCell", "TextFieldCell", "ActionSheetCell", "SwitchCell"]
        
        guard
            let sectionType = ConfigurationCellSection(rawValue: indexPath.section)
            else {
                preconditionFailure("Not supported section type")
        }
        
        switch sectionType {
        case .label:
            guard let labelCell = tableView.dequeueReusableCell(withIdentifier: cellIdentifiers[indexPath.section])
                    as? LabelTableViewCell else { return UITableViewCell() }
            switch indexPath.row {
            case 0:
                labelCell.setupCell(title: self.sdkVersion.0, content: self.sdkVersion.1)
            case 1:
                labelCell.setupCell(title: self.ifa.0, content: self.ifa.1)
            default:
                break
            }
            
            return labelCell
        case .textField:
            guard let textFieldCell = tableView.dequeueReusableCell(withIdentifier: cellIdentifiers[indexPath.section])
                    as? TextFieldTableViewCell else { return UITableViewCell() }
            switch indexPath.row {
            case 0:
                textFieldCell.setupCell(title: appID.0, content: appID.1)
            case 1:
                textFieldCell.setupCell(title: serverID.0, content: serverID.1)
                Configuration.serverID = serverID.1
            case 2:
                textFieldCell.setupCell(title: logUserID.0, content: logUserID.1)
            case 3:// hiveCertificationKey
                textFieldCell.setupCell(title: hiveCertificationKey.0, content: hiveCertificationKey.1)
            case 4:
                textFieldCell.setupCell(title: updateServerID.0, content: updateServerID.1)
            case 5:
                textFieldCell.setupCell(title: channel.0, content: channel.1)
            default:
                break
            }
            
            return textFieldCell
        
        case .switchCell:
            guard let switchCell = tableView.dequeueReusableCell(withIdentifier: cellIdentifiers[indexPath.section])
                    as? SwitchTableViewCell else { return UITableViewCell() }
            
            switch indexPath.row {
            case 0:
                switchCell.setupCell(title: self.log.0, isOn: self.log.1)
            case 1:
                switchCell.setupCell(title: self.ageGateU13.0, isOn: self.ageGateU13.1, tag: 1)
            case 2:
                switchCell.setupCell(title: self.hivePermissionViewOn.0, isOn: self.hivePermissionViewOn.1, tag: 2)
            case 3:
                switchCell.setupCell(title: self.hiveTheme.0, isOn: self.hiveTheme.1, tag: 3)
            case 4:
                switchCell.setupCell(title: self.useHercules.0, isOn: self.useHercules.1, tag: 4)
            default:
                break
            }

            return switchCell        
        case .actionSheet:
            guard let actionSheetCell = tableView.dequeueReusableCell(withIdentifier: cellIdentifiers[indexPath.section])
                    as? ActionSheetTableViewCell else { return UITableViewCell() }
            switch indexPath.row {
            case 0: actionSheetCell.setupCell(title: zone.0, label: self.zone.1.description)
            case 1:
                actionSheetCell.setupCell(title: self.getPermissionViewData, label: self.selectedResourceLanguage)
            case 2:
                actionSheetCell.setupCell(title: self.setGameLanguage, label: self.selectedGameLanguage)
            case 3:
                actionSheetCell.setupCell(title: self.updateGameLanguage, label: self.selectedGameLanguage)
            case 4:
                actionSheetCell.setupCell(title: self.getHiveOrientation, label: self.selectedOrientation)
            default:
                break
            }

            return actionSheetCell
        }
    }

    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        Logger.d("indexPath section - \(indexPath.section), row - \(indexPath.row)")

        if let textFieldSection = ConfigurationCellSection(rawValue: indexPath.section),
            textFieldSection == .textField {
            self.selectedIndexPath = indexPath

            self.performSegue(withIdentifier: "EditSegue", sender: self)
        } else if let actionSheetSection = ConfigurationCellSection(rawValue: indexPath.section), actionSheetSection == .actionSheet {
            
            // Zone
            if indexPath.row == 0 {
                let actionSheet = UIAlertController(title: "setZone", message: "Select one", preferredStyle: .actionSheet)
                for key in zones {
                    actionSheet.addAction(UIAlertAction(title: key.description, style: .default, handler: {_ in
                        
                        Logger.d("Called setZone")
                        ConfigurationInterface.setZone(key)

                        
                        // 로그 출력
                        let alertView = UIAlertController(title: "setZone", message: key.description, preferredStyle: .alert)
                        alertView.addAction(UIAlertAction(title: "OK", style: .default, handler: nil))
                        
                        // 셀 업데이트
                        self.present(alertView, animated: true) {
                            self.zone = ("Zone", key)
                            tableView.reloadData()
                        }
                        
                    }))
                }
                self.present(actionSheet, animated: true, completion: nil)
            }

            // getPermissionViewData
            if indexPath.row == 1 {
                Logger.d("getPermissionViewData")

                let actionSheet = UIAlertController(title: "Resouce Language", message: "Select one", preferredStyle: .actionSheet)
                for key in languages.keys {
                    actionSheet.addAction(UIAlertAction(title: key, style: .default, handler: {_ in

                        // getPermissionViewData API 호출
                        guard let value = self.languages[key] else {
                            return
                        }
                        let permissionViewData = ConfigurationInterface.getPermissionViewData(HIVELanguage(stringValue: value) ?? HIVELanguage.EN)

                        // 로그 출력
                        let alertView = UIAlertController(
                            title: "getPermissionViewData",
                            message: permissionViewData.description,
                            preferredStyle: .alert
                        )
                        alertView.addAction(UIAlertAction(title: "OK", style: .default, handler: nil))

                        // 셀 업데이트
                        self.present(alertView, animated: true) {
                            self.selectedResourceLanguage = value
                            tableView.reloadData()
                        }
                    }))
                }
                self.present(actionSheet, animated: true, completion: nil)
            } else if  indexPath.row == 2 ||  indexPath.row == 3 {
                Logger.d("setGameLanguage")
                
                let actionSheet = UIAlertController(title: "Game Language", message: "Select one", preferredStyle: .actionSheet)
                for key in languages.keys {
                    actionSheet.addAction(UIAlertAction(title: key, style: .default, handler: {_ in
                        
                        guard let value = self.languages[key] else {
                            return
                        }
                        
                        if value != "null" {
                            Logger.d("Called setGameLanguage")
                            if indexPath.row == 2 {
                                ConfigurationInterface.setGameLanguage(value)
                            } else if indexPath.row == 3 {
                                ConfigurationInterface.updateGameLanguage(value)
                            }
                        }
                        
                        // 로그 출력
                        let alertView = UIAlertController(title: "setGameLanguage", message: value, preferredStyle: .alert)
                        alertView.addAction(UIAlertAction(title: "OK", style: .default, handler: nil))
                        
                        // 셀 업데이트
                        self.present(alertView, animated: true) {
                            self.selectedGameLanguage = value
                            tableView.reloadData()
                        }
                        
                    }))
                }
                self.present(actionSheet, animated: true, completion: nil)
                
            } else if indexPath.row == 4 {
                Logger.d("setHiveOrientation")
                
                let actionSheet = UIAlertController(title: "HiveOrientation", message: "select", preferredStyle: .actionSheet)
                for orientation in ["all", "portrait", "landscape", "", "Empty", "ErrorValue"] {
                    actionSheet.addAction(UIAlertAction(title: orientation, style: .default) { (_) in
                        let value = orientation
                        ConfigurationInterface.setHiveOrientation(value)
                        self.selectedOrientation = value
                        tableView.reloadData()
                    })
                }
                
                self.present(actionSheet, animated: true, completion: nil)
            }
        }

        tableView.deselectRow(at: indexPath, animated: true)
    }
}

// MARK: IBActions
extension ConfigurationViewController {
	@IBAction func didResetButtonTouched(_ sender: Any) {
		Logger.d("didResetButtonTouched")
	}
	
	@IBAction func didViewConfigurationButtonTouched(_ sender: Any) {
		Logger.d("didViewConfigurationButtonTouched")
	}
}

