//
//  DataStoreViewController.swift
//  HIVE_SDK_Sample
//
//  Created by sjyong on 2022/02/22.
//  Copyright © 2022 gcp. All rights reserved.
//

import UIKit
import HIVEService

enum DataStoreApi: String, CaseIterable {
    case setWithKeyValue = "set(key,value) "
    case setWithData = "set(data)"
    case getWithKey = "get(key)"
    case getMyData = "getMyData()"
    case getUsersDataWithKey = "getUsersData(key)"
}

class DataStoreViewController: BaseViewController {
    static let PARAM_DATA_EXTRA_SIZE = 3
    let PARAM_DATA_TEXTFIELD_HEIGHT = 40.0

    var paramKey = ""
    var paramValue = ""
    var paramData: [String: String] = [
        "score": "25000",
        "level": "12",
        "ranking": "1234",
        "gold": "1200",
        "diamond": "500",
    ]
    var paramDataString = ""
    var paramDataKeys = [String](repeating: "", count: PARAM_DATA_EXTRA_SIZE)
    var paramDataValues = [String](repeating: "", count: PARAM_DATA_EXTRA_SIZE)

    enum TextFieldType: Int {
        case key = 0
        case value
        case data
        case dataKey_1
        case dataValue_1
        case dataKey_2
        case dataValue_2
        case dataKey_3
        case dataValue_3
    }

    let apiList = DataStoreApi.allCases

    required init?(coder: NSCoder) {
        // fatalError("init(coder:) has not been implemented")
        super.init(coder: coder)
        self.paramDataString = self.paramData.toJSON() ?? ""
    }

    func addAlertViewController(api: DataStoreApi, handler: ((UIAlertAction) -> Void)? ) {
        let ALERT_CONTROLLER_TITLE = "DataStore"
        let ALERT_CONTROLLER_MESSASGE = api.rawValue

        let alert = UIAlertController(title: ALERT_CONTROLLER_TITLE, message: ALERT_CONTROLLER_MESSASGE, preferredStyle: .alert)

        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel) { action in
            handler?(action)
        }
        alert.addAction(cancelAction)

        let okAction = UIAlertAction(title: "OK", style: .default) { action in
            handler?(action)
        }
        alert.addAction(okAction)

        switch api {
        case .setWithKeyValue:
            alert.addTextField { textField in
                textField.placeholder = "key:"
                textField.tag = TextFieldType.key.rawValue
                textField.addTarget(self, action: #selector(self.textFieldDidChange(_:)), for: .editingDidEnd)
            }

            alert.addTextField { textField in
                textField.placeholder = "value:"
                textField.tag = TextFieldType.value.rawValue
                textField.addTarget(self, action: #selector(self.textFieldDidChange(_:)), for: .editingDidEnd)
            }
        case .setWithData:
            alert.addTextField { textField in
                textField.text = self.paramDataString
                let heightConstraint = NSLayoutConstraint(
                    item: textField,
                    attribute: .height,
                    relatedBy: .equal,
                    toItem: nil,
                    attribute: .notAnAttribute,
                    multiplier: 1,
                    constant: self.PARAM_DATA_TEXTFIELD_HEIGHT
                )
                textField.addConstraint(heightConstraint)
                textField.tag = TextFieldType.data.rawValue
                textField.addTarget(self, action: #selector(self.textFieldDidChange(_:)), for: .editingDidEnd)
            }

            for i in 0 ..< DataStoreViewController.PARAM_DATA_EXTRA_SIZE {
                alert.addTextField { textField in
                    textField.placeholder = "data-key"
                    textField.tag = TextFieldType.dataKey_1.rawValue + i*2
                    textField.addTarget(self, action: #selector(self.textFieldDidChange(_:)), for: .editingDidEnd)
                }

                alert.addTextField { textField in
                    textField.placeholder = "data-value"
                    textField.tag = TextFieldType.dataKey_1.rawValue + i*2 + 1
                    textField.addTarget(self, action: #selector(self.textFieldDidChange(_:)), for: .editingDidEnd)
                }
            }
        case .getWithKey, .getUsersDataWithKey:
            alert.addTextField { textField in
                textField.placeholder = "key:"
                textField.tag = TextFieldType.key.rawValue
                textField.addTarget(self, action: #selector(self.textFieldDidChange(_:)), for: .editingDidEnd)
            }
        case .getMyData:
            handler?(okAction)
            return
        }

        self.present(alert, animated: false, completion: nil)
    }

    @objc func textFieldDidChange(_ textField: UITextField) {
        switch TextFieldType(rawValue: textField.tag) {
        case .key:
            self.paramKey = textField.text ?? ""
        case .value:
            self.paramValue = textField.text ?? ""
        case .data:
            self.paramDataString = textField.text ?? ""
        case .dataKey_1:
            self.paramDataKeys[0] = textField.text ?? ""
        case .dataValue_1:
            self.paramDataValues[0] = textField.text ?? ""
        case .dataKey_2:
            self.paramDataKeys[1] = textField.text ?? ""
        case .dataValue_2:
            self.paramDataValues[1] = textField.text ?? ""
        case .dataKey_3:
            self.paramDataKeys[2] = textField.text ?? ""
        case .dataValue_3:
            self.paramDataValues[2] = textField.text ?? ""
        default:
            break
        }
    }

    func callAPI(api: DataStoreApi) {

        self.addAlertViewController(api: api) { action in
            guard action.style == .default else {
                return
            }

            switch api {
            case .setWithKeyValue:
                DataStoreInterface.set(self.paramKey, value: self.paramValue) { result in
                    let log = """
                    === DataStoreInterface.set(key,value) callback ===
                    result = \(result.description)
                    """
                    self.logDelegate?.addLog(log: log)
                }
                self.logDelegate?.addLog(log: "DataStoreInterface.set(key,value) called.")

            case .setWithData:
                DataStoreInterface.set(self.getMergedData()) { result in
                    let log = """
                    === DataStoreInterface.set(data) callback ===
                    result = \(result.description)
                    """
                    self.logDelegate?.addLog(log: log)
                }
                self.logDelegate?.addLog(log: "DataStoreInterface.set(data) called.")

            case .getWithKey:
                DataStoreInterface.get(self.paramKey) { result, data in
                    let log = """
                    === DataStoreInterface.get(key) callback ===
                    result = \(result.description)
                    data = \(data ?? "")
                    """
                    self.logDelegate?.addLog(log: log)
                }
                self.logDelegate?.addLog(log: "DataStoreInterface.get(key) called.")

            case .getMyData:
                DataStoreInterface.getMyData { result, myData in
                    let log = """
                    === DataStoreInterface.getMyData() callback ===
                    result = \(result.description)
                    myData = \(myData?.toJSON() ?? "")
                    """
                    self.logDelegate?.addLog(log: log)
                }
                self.logDelegate?.addLog(log: "DataStoreInterface.getMyData() called.")

            case .getUsersDataWithKey:
                DataStoreInterface.getUsersData(self.paramKey) { result, key, usersData in
                    let log = """
                    === DataStoreInterface.getUsersData(key) callback ===
                    result = \(result.description)
                    key = \(key)
                    data = \(self.dictionaryToString(usersData ?? [Int64: String]()))
                    """
                    self.logDelegate?.addLog(log: log)
                }
                self.logDelegate?.addLog(log: "DataStoreInterface.getUsersData(key) called.")
            }
        }
    }

    func getMergedData() -> [String: String] {
        for i in 0 ..< DataStoreViewController.PARAM_DATA_EXTRA_SIZE {
            let extraKey = paramDataKeys[i]
            let extraValue = paramDataValues[i]

            if !extraKey.isEmpty {
                self.paramData[extraKey] = extraValue
            }
        }
        return self.paramData
    }

    func dictionaryToString(_ dict: [Int64: String]) -> String {
        var resultDict = [String: String]()

        for keyInt64 in dict.keys {
            resultDict["\(keyInt64)"] = dict[keyInt64]
        }

        return resultDict.toJSON() ?? ""
    }
}

extension DataStoreViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        self.apiList.count
    }

    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "ApiCell") as? DataStoreTableViewCell else {
            preconditionFailure("Not supported cell type")
        }

        cell.configureCell(api: self.apiList[indexPath.row])

        return cell
    }

    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.callAPI(api: apiList[indexPath.row])
        tableView.deselectRow(at: indexPath, animated: true)
    }
}
