//
//  PlatformHelperViewController.swift
//  HIVE_SDK_Sample
//
//  Created by sjyong on 13/08/2019.
//  Copyright © 2019 gcp. All rights reserved.
//

import UIKit
import HIVEService

enum PlatformHelperApi: String, CaseIterable {
    case requestUserPermissions = "requestUserPermissions"
}

class PlatformHelperViewController: BaseViewController {
    let apis: [PlatformHelperApi] = PlatformHelperApi.allCases
    @IBOutlet weak var tableView: UITableView!

    var requests: [String] = []

    override func viewDidLoad() {
        super.viewDidLoad()

        self.tableView.contentInset = UIEdgeInsets.init(top: 44, left: 0, bottom: 0, right: 0)
    }

    func callAPI(api: PlatformHelperApi) {
        switch api {
        case .requestUserPermissions:
            PlatformHelperInterface.requestUserPermissions(requests) { (resultAPI, granted, denied) in
                let log =   """
                === PlatformHelperInterface.requestUserPermissions ===
                result - \(resultAPI)
                granted - \(String(describing: granted))
                denied - \(String(describing: denied))
                """
                self.logDelegate?.addLog(log: log)
            }
        }
    }
}

extension PlatformHelperViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        self.apis.count
    }

    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "ApiCell") as? PlatformHelperTableViewCell
        else {
            preconditionFailure("Not supported cell type")
        }

        cell.configureCell(api: self.apis[indexPath.row])

        return cell
    }

    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.callAPI(api: apis[indexPath.row])
        tableView.deselectRow(at: indexPath, animated: true)
    }
}
