//
//  PromotionBannerViewController.swift
//  HIVE_SDK_Sample
//
//  Created by 용석진 on 14/10/2019.
//  Copyright © 2019 gcp. All rights reserved.
//

import UIKit
import HIVEService
import HIVEBase

class PromotionBannerViewController: UIViewController {
    var bannerInfos = [PromotionBanner]()
    
    @IBOutlet var tableViewBanner: UITableView!
    typealias CompleteHandler = (_ result: ResultAPI, _ log: String) -> Void
    var handler: CompleteHandler?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        addShadowToView(view: self.view)
    }
    
    init?(data: [PromotionBanner]?, completeHandler: @escaping CompleteHandler) {
        guard let data = data
        else {
            Logger.w("[PromotionBannerViewController] data nil")
            return nil
        }
        bannerInfos = data
        self.handler = completeHandler
        super.init(nibName: "PromotionBannerViewController", bundle: Bundle.main)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
 
    
    @IBAction func onBtnCloseAction(_ sender: Any) {
        self.dismiss(animated: false) {
            Logger.d("[PromotionBannerViewController] dismiss")
        }
    }

    func addShadowToButton(button: UIButton?, title: String?) {
        button?.layer.shadowColor = UIColor.init(red: 0.0, green: 0.0, blue: 0.0, alpha: 1.0).cgColor
        button?.layer.shadowOpacity = 0.31
        button?.layer.shadowOffset = CGSize.init(width: 0.0, height: 2.0)

        button?.titleLabel?.numberOfLines = 2
        button?.titleLabel?.textAlignment = NSTextAlignment.center

        if let title = title, !title.isEmpty {
            button?.setTitle(title, for: .normal)
        }
    }

    func addShadowToView(view: UIView?) {
        view?.layer.shadowColor = UIColor.init(red: 0.0, green: 0.0, blue: 0.0, alpha: 1.0).cgColor
        view?.layer.shadowOpacity = 0.3

        view?.layer.shadowRadius = 30.0
        view?.layer.shadowOffset = CGSize.init(width: 0.0, height: 30.0)
    }
}

extension PromotionBannerViewController: UITableViewDataSource, UITableViewDelegate {
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cellBundle = Bundle.main.loadNibNamed("PromotionBannerViewCell", owner: tableViewBanner, options: nil),
              let cell = cellBundle[0] as? PromotionBannerViewCell else {
            return UITableViewCell()
        }
        
        let banner = bannerInfos[indexPath.row]
        
        guard
            let imageUrl = banner.imageUrl,
            let url = URL(string: imageUrl)
            else {
                Logger.w("pid:\(banner.pid) url is nil")
                return cell
        }
        
        cell.pidLabel.text = cell.pidLabel.text! + "\(banner.pid)"
        cell.startDateLabel.text = cell.startDateLabel.text! + banner.displayStartDate!
        cell.endDateLabel.text = cell.endDateLabel.text! + banner.displayEndDate!
        
        // 이미지 로드 비동기 처리
        DispatchQueue.global(qos: .default).async {
            do {
                let data = try Data(contentsOf: url)
                let image = UIImage(data: data)
                DispatchQueue.main.async {
                    if let cellLoaded = tableView .cellForRow(at: indexPath) as? PromotionBannerViewCell {
                        cellLoaded.urlImageView.image = image
                    }
                }
            } catch {
                Logger.w("pid:\(banner.pid) data from url is nil")
            }
        }
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        bannerInfos.count
    }
    
    func numberOfSections(in tableView: UITableView) -> Int {
        1
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        UITableView.automaticDimension
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let banner = bannerInfos[indexPath.row]
        let pidToString = "\(banner.pid)"
        
        PromotionInterface.showCustomContents(.direct, contents: pidToString) { (resultAPI, viewResultType) in
            self.handler?(resultAPI, viewResultType.description)
        }
    }
}
