//
//  PushViewController.swift
//  HIVE_SDK_Sample
//
//  Created by jhjang on 01/04/2019.
//  Copyright © 2019 gcp. All rights reserved.
//

import UIKit
import HIVEService

enum PushApi: String, CaseIterable {
    case getRemotePush = "Get Remote Push"
    case setRemotePush = "Set Remote Push"
    case registerLocalPush = "Register Local Push"
    case unregisterLocalPush = "Unregister Local Push"
    case unregisterLocalPushes = "Unregister Local Pushes"
    case unregisterAllLocalPushes = "Unregister All Local Pushes"
    case setForegroundPush = "Set Foreground Push"
    case getForegroundPush = "Get Foreground Push"
    case requestPushPermission = "Request Push Permission"
}

class PushViewController: BaseViewController {
    let apis: [PushApi] = PushApi.allCases
    @IBOutlet weak var tableView: UITableView!
        
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.tableView.contentInset = UIEdgeInsets.init(top: 44, left: 0, bottom: 0, right: 0)
    }
    
    func addTextFieldToAlert(_ alert: UIAlertController, placeHolder: String, tag: Int) {
        alert.addTextField { textField in
            textField.placeholder = placeHolder
            textField.tag = tag
        }
    }
    
    func getTextFieldFromAlert(_ alert: UIAlertController, tag: Int) -> UITextField? {
        let textField = alert.textFields?.filter { $0.tag == tag }.first
        return textField
    }
    
    func getTextFromAlert(_ alert: UIAlertController, defaultText: String, tag: Int) -> String {
        let text =  self.getTextFieldFromAlert(alert, tag: tag)?.text ?? ""
        if !text.isEmpty {
            return text
        }
        return defaultText
    }
    
    var localRegistAlert: UIAlertController {
        let alert = UIAlertController(title: "registerLocalPush", message: "input local push", preferredStyle: .alert)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alert.addAction(cancelAction)
        
        let textFieldList = ["pushId", "title", "message", "after", "groupId"]
        
        for (i, item) in textFieldList.enumerated() {
            self.addTextFieldToAlert(alert, placeHolder: item, tag: i)
        }
        
        let okAction = UIAlertAction(title: "OK", style: .default) { (_) in
            
            let localPushObject = LocalPush()
    
            localPushObject.noticeId = Int(self.getTextFromAlert(alert, defaultText: "0", tag: 0)) ?? 0
            localPushObject.title = self.getTextFromAlert(alert, defaultText: "Teset Title", tag: 1)
            localPushObject.msg = self.getTextFromAlert(alert, defaultText: "Test Message", tag: 2)
            localPushObject.after = Double(self.getTextFromAlert(alert, defaultText: "5", tag: 3)) ?? 5
            localPushObject.groupId = self.getTextFromAlert(alert, defaultText: "", tag: 4)
            
            PushInterface.registerLocalPush(localPushObject) { (result, localPush) in
                let log = """
                === PushInterface.registerLocalPush(\(localPushObject.noticeId)) ===
                
                result - \(result)
                localPush - \(localPush?.description ?? "nothing localPush")
                """
                
                self.logDelegate?.addLog(log: log)
            }
        }
        
        alert.addAction(okAction)
        
        return alert
    }
    
    func localUnregistAlert(isArray: Bool) -> UIAlertController {
        let alert = UIAlertController(title: "unregisterLocalPush", message: "input local push", preferredStyle: .alert)
        
        let cancelAction = UIAlertAction(title: "Cancel", style: .cancel, handler: nil)
        alert.addAction(cancelAction)
        
        alert.addTextField { (textField) in
            textField.placeholder = "pushId"
        }
        
        let okAction = UIAlertAction(title: "OK", style: .default) { (_) in
            let pushIdString: String = alert.textFields?[0].text ?? "0"

            if !isArray {
                PushInterface.unregisterLocalPush(Int(pushIdString) ?? 0)
            } else {
                PushInterface.unregisterLocalPushes([Int(pushIdString) ?? 0])
            }
            
            let log = """
            === PushInterface.unregisterLocalPush(\(pushIdString)) ===
            
            pushId - \(pushIdString)
            """
            
            self.logDelegate?.addLog(log: log)
        }
        
        alert.addAction(okAction)
        
        return alert
    }
    
    func callAPI(api: PushApi) {
        switch api {
        case .getRemotePush:
            _ = PushInterface.getRemotePush { (result, remotePush) in
                let log = """
                === PushInterface.getRemotePush() ===
                
                result - \(result)
                remotePush - \(String(describing: remotePush?.description))
                """
                
                self.logDelegate?.addLog(log: log)
            }
        case .setRemotePush:
            let pushAlert = UIAlertController.init(title: "Push Notice / Night", message: "select one", preferredStyle: .actionSheet)

            let closure = { (action: UIAlertAction!) -> Void in
                let index = pushAlert.actions.firstIndex(of: action)
                let remotePush = RemotePush()

                switch index {
                case 0:
                    remotePush.isAgreeNotice = true
                    remotePush.isAgreeNight = true
                case 1:
                    remotePush.isAgreeNotice = true
                    remotePush.isAgreeNight = false
                case 2:
                    remotePush.isAgreeNotice = false
                    remotePush.isAgreeNight = true
                case 3:
                    remotePush.isAgreeNotice = false
                    remotePush.isAgreeNight = false
                default:
                    remotePush.isAgreeNotice = true
                    remotePush.isAgreeNight = false
                }
                PushInterface.setRemotePush(remotePush) { (result, remotePush) in
                    let log = """
                    === PushInterface.setRemotePush() ===
                    
                    result - \(result)
                    remotePush - \(String(describing: remotePush?.description))
                    """

                    self.logDelegate?.addLog(log: log)
                }
            }

            let case0 = UIAlertAction(title: "ON / ON", style: .default, handler: closure)
            pushAlert.addAction(case0)
            let case1 = UIAlertAction(title: "ON / OFF", style: .default, handler: closure)
            pushAlert.addAction(case1)
            let case2 = UIAlertAction(title: "OFF / ON", style: .default, handler: closure)
            pushAlert.addAction(case2)
            let case3 = UIAlertAction(title: "OFF / OFF", style: .default, handler: closure)
            pushAlert.addAction(case3)

            self.present(pushAlert, animated: true, completion: nil)
        case .setForegroundPush:
            let pushAlert = UIAlertController.init(title: "SetForegroundPush Local / Remote", message: "select one", preferredStyle: .actionSheet)

            let closure = { (action: UIAlertAction!) -> Void in
                let index = pushAlert.actions.firstIndex(of: action)
                let setting = PushSetting()

                switch index {
                case 0:
                    setting.useForegroundLocalPush = true
                    setting.useForegroundRemotePush = true
                case 1:
                    setting.useForegroundLocalPush = true
                    setting.useForegroundRemotePush = false
                case 2:
                    setting.useForegroundLocalPush = false
                    setting.useForegroundRemotePush = true
                case 3:
                    setting.useForegroundLocalPush = false
                    setting.useForegroundRemotePush = false
                default:
                    setting.useForegroundLocalPush = true
                    setting.useForegroundRemotePush = true
                }

                PushInterface.setForegroundPush(setting) { (result, settings) in
                    let log = """
                    === PushInterface.setForegroundPush() ===

                    result - \(result)
                    useForegroundLocalPush - \(String(describing: settings?.useForegroundLocalPush))
                    useForegroundRemotePush - \(String(describing: settings?.useForegroundRemotePush))
                    """

                    self.logDelegate?.addLog(log: log)
                }
            }

            let case0 = UIAlertAction(title: "ON / ON", style: .default, handler: closure)
            pushAlert.addAction(case0)
            let case1 = UIAlertAction(title: "ON / OFF", style: .default, handler: closure)
            pushAlert.addAction(case1)
            let case2 = UIAlertAction(title: "OFF / ON", style: .default, handler: closure)
            pushAlert.addAction(case2)
            let case3 = UIAlertAction(title: "OFF / OFF", style: .default, handler: closure)
            pushAlert.addAction(case3)

            self.present(pushAlert, animated: true, completion: nil)
        
        case .getForegroundPush:
            _ = PushInterface.getForegroundPush { (result, settings) in
                let log = """
                === PushInterface.getForegroundPush() ===
                
                result - \(result)
                useForegroundLocalPush - \(String(describing: settings?.useForegroundLocalPush))
                useForegroundRemotePush - \(String(describing: settings?.useForegroundRemotePush))
                """

                self.logDelegate?.addLog(log: log)
            }
        case .registerLocalPush:
            self.present(self.localRegistAlert, animated: true, completion: nil)
        case .unregisterLocalPush:
            self.present(self.localUnregistAlert(isArray: false), animated: true, completion: nil)
        case .unregisterLocalPushes:
            self.present(self.localUnregistAlert(isArray: true), animated: true, completion: nil)
        case .unregisterAllLocalPushes:
            PushInterface.unregisterAllLocalPushes()
            let log = """
            === PushInterface.unregisterAllLocalPushes() ===
            """
            self.logDelegate?.addLog(log: log)

        case .requestPushPermission:
            PushInterface.requestPushPermission()
        }
    }
}

extension PushViewController: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        self.apis.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "ApiCell") as? PushTableViewCell else {
            preconditionFailure("Not supported cell type")
        }
        
        cell.configureCell(api: self.apis[indexPath.row])
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        self.callAPI(api: apis[indexPath.row])
        tableView.deselectRow(at: indexPath, animated: true)
    }
}

