//
//  MercuryViewController.m
//  HIVE_SDK_Sample
//
//  Created by daunjoung on 2021/06/01.
//  Copyright © 2021 gcp. All rights reserved.
//

#import "MercuryViewController.h"
#import "C2SMercury.h"

@interface MercuryViewController ()
@property (weak, nonatomic) IBOutlet UISwitch *isForcedSwitch;
@end

@implementation MercuryViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
}

- (void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];
    
    CS_MercurySetCallBack(mercuryCB);
    
}

// MARK: -- Actions
- (IBAction)onClickShowExEventLegacy:(id)sender {
    CS_MercuryShowEx("", MERCURY_SHOW_EVENT_LEGACY, "{\"additionalInfo\":\"\"}");
}

- (IBAction)onClickShowExEvent:(id)sender {
    CS_MercuryShowEx("", MERCURY_SHOW_EVENT, "{\"additionalInfo\":\"\"}");
}

- (IBAction)onClickShowExNoticeTop:(id)sender {
    if(_isForcedSwitch.on) {
        CS_MercuryShowEx("", MERCURY_SHOW_FORCED_NOTICE_TOP, "{\"additionalInfo\":\"\"}");
    } else {
        CS_MercuryShowEx("", MERCURY_SHOW_NOTICE_TOP, "{\"additionalInfo\":\"\"}");
    }
}

- (IBAction)onClickShowExNoticeBottom:(id)sender {
    if(_isForcedSwitch.on) {
        CS_MercuryShowEx("", MERCURY_SHOW_FORCED_NOTICE_BOTTOM, "{\"additionalInfo\":\"\"}");
    } else {
        CS_MercuryShowEx("", MERCURY_SHOW_NOTICE_BOTTOM, "{\"additionalInfo\":\"\"}");
    }
}

- (IBAction)onClickShowExNoticeOnly:(id)sender {
    if(_isForcedSwitch.on) {
        CS_MercuryShowEx("", MERCURY_SHOW_FORCED_NOTICE_ONLY, "{\"additionalInfo\":\"\"}");
    } else {
        CS_MercuryShowEx("", MERCURY_SHOW_NOTICE_ONLY, "{\"additionalInfo\":\"\"}");
    }
}

- (IBAction)onClickShowExCustom:(id)sender {
    UIAlertController * alert = [UIAlertController alertControllerWithTitle: @"View ID"
                                                                                     message: @"CustomView ID 를 입력하세요"
                                                                                 preferredStyle:UIAlertControllerStyleAlert];
       [alert addTextFieldWithConfigurationHandler:^(UITextField *textField) {
           textField.placeholder = @"ex) 310000";
           textField.text = @"310000";
       }];
    
       [alert addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
           NSArray * textfields = alert.textFields;
           UITextField * serverId = textfields[0];
           NSLog(@"%@",serverId.text);
           
           
           CS_MercuryShowEx("", serverId.text.intValue, "{\"additionalInfo\":\"\"}");
       }]];
       [self presentViewController:alert animated:YES completion:nil];
}

- (IBAction)onClickGetCustomViewInfo:(id)sender {
    UIAlertController * alert = [UIAlertController alertControllerWithTitle: @"View ID"
                                                                                     message: @"CustomView ID 를 입력하세요"
                                                                                 preferredStyle:UIAlertControllerStyleAlert];
       [alert addTextFieldWithConfigurationHandler:^(UITextField *textField) {
           textField.placeholder = @"ex) 310000";
           textField.text = @"310000";
       }];
    
       [alert addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
           NSArray * textfields = alert.textFields;
           UITextField * serverId = textfields[0];
           NSLog(@"%@",serverId.text);
           
           CS_MercuryGetCustomViewInfo("", serverId.text.intValue, mercuryCBWithData);
       }]];
       [self presentViewController:alert animated:YES completion:nil];
    
}

- (IBAction)onClickShowReview:(id)sender {
    CS_MercuryShowReviewPopup("", "", "");
}

- (IBAction)onClickShowNativeReview:(id)sender {
    CS_MercuryShowNativeReviewPopup("", "", "");
}




#pragma mark - Mercury callbacks
void mercuryCB(MERCURY_RESULT result)
{
    switch(result)
    {
        case MERCURY_OPEN:
            NSLog(@"MERCURY_OPEN");
            break;
        case MERCURY_CLOSE:
            NSLog(@"MERCURY_CLOSE");
            break;
        case MERCURY_FORCED_OPEN:
            NSLog(@"MERCURY_FORCED_OPEN");
            break;
        case MERCURY_FORCED_CLOSE:
            NSLog(@"MERCURY_FORCED_CLOSE");
            break;
        case MERCURY_NOTICE_OPEN:
            NSLog(@"MERCURY_NOTICE_OPEN");
            break;
        case MERCURY_NOTICE_CLOSE:
            NSLog(@"MERCURY_NOTICE_CLOSE");
            break;
        case MERCURY_CUSTOM_BOARD_OPEN:
            NSLog(@"MERCURY_CUSTOM_BOARD_OPEN");
            break;
        case MERCURY_CUSTOM_BOARD_CLOSE:
            NSLog(@"MERCURY_CUSTOM_BOARD_CLOSE");
            break;
        case MERCURY_CUSTOM_VIEW_OPEN:
            NSLog(@"MERCURY_CUSTOM_VIEW_OPEN");
            break;
        case MERCURY_CUSTOM_VIEW_CLOSE:
            NSLog(@"MERCURY_CUSTOM_VIEW_CLOSE");
            break;
        case MERCURY_REVIEW_OPEN:
            NSLog(@"MERCURY_REVIEW_OPEN");
            break;
        case MERCURY_REVIEW_CLOSE:
            NSLog(@"MERCURY_REVIEW_CLOSE");
            break;
        case MERCURY_NETWORK_DISCONNECT:
            NSLog(@"MERCURY_NETWORK_DISCONNECT");
            break;
            
        case MERCURY_START_PLAYBACK:
            NSLog(@"MERCURY_START_PLAYBACK");
            break;
        case MERCURY_FINISH_PLAYBACK:
            NSLog(@"MERCURY_FINISH_PLAYBACK");
            break;
            
        default:
            NSLog(@"Custom ShowType: %d", (int)result);
            break;
            
    }
}

void mercuryCBWithData(MERCURY_CUSTOM_RESULT result, const char* jsonData)
{
    NSLog(@"Mercury callback with data : %s", jsonData);
    
    switch(result)
    {
        case MERCURY_SUCCESS:
        {
            if(jsonData == NULL) return;
            
            // CString 파싱
            NSString *convertStr = [NSString stringWithUTF8String:jsonData];
            NSError *error = nil;
            NSDictionary *dataDict = [NSJSONSerialization JSONObjectWithData:[convertStr dataUsingEncoding:NSUTF8StringEncoding]
                                                                     options:NSJSONReadingAllowFragments error:&error];
            
            if(error)
            {
                NSLog(@"%@", [error localizedDescription]);
                return;
            }
            
            NSLog(@"Parsed dictionary = %@", dataDict);
            
            NSDictionary *customViewInfo = [dataDict objectForKey:@"customviewinfo"];
            
            // 뱃지 정보
            // badgetype = new;
            // expire = 24; (min)
            // showtype = 200001;
            // startdate = 1434070800; (sec : interval since1970)
            // term = 30; (min)
            NSDictionary *badgeInfo = [customViewInfo objectForKey:@"badge"];
            
            NSLog(@"Badge info : %@", badgeInfo);
            
            // URL 정보 얻어오기
            NSString *urlStr = [customViewInfo objectForKey:@"url"];
            NSURL *url = [NSURL URLWithString:urlStr];
            
            NSLog(@"URL info : %@", url.absoluteString);
            
            // cookies 정보 얻어오기
            NSDictionary *cookiesDict = [customViewInfo objectForKey:@"cookies"];
            
            NSLog(@"cookiesDict : %@", cookiesDict);
            
            // Test Webview Loading
//            WebviewTestViewController *webViewController = [[WebviewTestViewController alloc] initWithNibName:@"WebviewTestViewController" bundle:[NSBundle mainBundle]];
//
//            [viewControllerSharedInstance presentViewController:webViewController animated:YES completion:^{
//                NSURL *webURL = [NSURL URLWithString:@"http://test.mercury.com2us.com/test/printCookie"];
//                NSURLRequest *request = [NSURLRequest requestWithURL:webURL];
//
//                webViewController.customURL = url;
//                webViewController.cookiesDict = cookiesDict;
//                [webViewController.webView loadRequest:request];
//            }];
        }
            break;
            
        case MERCURY_FAILED:
            
            break;
            
        default:
            break;
            
    }
}


void mercuryBannerInfoCB(MERCURY_CUSTOM_RESULT result, const char* data)
{
    NSLog(@"MercuryBannerInfo callback is called");
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"BannerInfo Result" message:[NSString stringWithUTF8String:data] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
}
@end
