//
//  ETCViewController.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 11..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "ETCViewController.h"
#import "HiveSocialManager.h"
#import "MainNavigationController.h"

//#if TARGET_IPHONE_SIMULATOR
#define TEST_BUTTON
//#endif // TARGET_IPHONE_SIMULATOR

typedef NS_ENUM(NSInteger, DialogPathSection) {
	DialogPathSectionDefault,
	DialogPathSectionCustom,
	DialogPathSectionMax
};

typedef NS_ENUM(NSInteger, CustomTestType) {
	CustomTestTypeSendMessage,
	CustomTestTypeRequestFriend,
	CustomTestTypeSaveData,
	CustomTestTypeMax
};

static NSString* kHiveSocialDialogPathName = @"path_name";
static NSString* kHivesocialDialogPathDescription	= @"path_description";

@interface ETCViewController ()
<UIAlertViewDelegate, UITableViewDataSource, UITableViewDelegate>
{
	NSMutableArray* _allConstraints;
	
	UITableView* _dialogPathTableView;
	NSArray* _dialogPaths;

	UITableView* _customTestTableView;
	NSArray* _customTest;
	
	UIViewController* _modalViewController;
}
#ifdef TEST_BUTTON
@property (readonly, nonatomic, strong) SPButton* testButton;
#endif // TEST_BUTTON
@end

@implementation ETCViewController

- (void)viewDidLoad {
    [super viewDidLoad];
	
	self.title	= TITLE_ETC;
	
    // Do any additional setup after loading the view.
	_facebookShareButton	= [SPButton buttonWithTitle:@"Facebook 공유" handler:^(SPButton *button) {
		[self _onFacebookShare];
	}];
	_facebookShareButton.translatesAutoresizingMaskIntoConstraints	= NO;
	_facebookShareButton.backgroundColor	= SP_FACEBOOK_BACKGROUND_COLOR;
	[self.contentView addSubview:_facebookShareButton];
	
	_googleplusShareButton	= [SPButton buttonWithTitle:@"Google+ 공유" handler:^(SPButton *button) {
		[self _onGooglePlusShare];
	}];
	_googleplusShareButton.translatesAutoresizingMaskIntoConstraints	= NO;
	_googleplusShareButton.backgroundColor	= SP_GOOGLEPLUS_BACKGROUND_COLOR;
	[self.contentView addSubview:_googleplusShareButton];
	
	_dialogButton	= [SPButton buttonWithTitle:@"Hive Dialog" handler:^(SPButton *button) {
		[self _onHiveDialog];
	}];
	_dialogButton.translatesAutoresizingMaskIntoConstraints	= NO;
	_dialogButton.backgroundColor	= SP_BUTTON_COLOR_TYPE03;
	[self.contentView addSubview:_dialogButton];
	
	_facebookFriends	= [SPButton buttonWithTitle:@"Get user by Facebook friends" handler:^(SPButton *button) {
		[self _onFacebookFriends];
	}];
	_facebookFriends.translatesAutoresizingMaskIntoConstraints	= NO;
	_facebookFriends.backgroundColor	= SP_BUTTON_COLOR_TYPE03;
	[self.contentView addSubview:_facebookFriends];
	
#ifdef TEST_BUTTON
	_testButton	= [SPButton buttonWithTitle:@"Hive Test" handler:^(SPButton *button) {
		[self _onHiveTest];
	}];
	_testButton.translatesAutoresizingMaskIntoConstraints	= NO;
	_testButton.backgroundColor	= SP_BUTTON_COLOR_TYPE03;
	[self.contentView addSubview:_testButton];
#endif // TEST_BUTTON
	
	_dialogPaths	= @[@{kHiveSocialDialogPathName : @"user", kHivesocialDialogPathDescription : @"자신의 프로필 페이지"},
						@{kHiveSocialDialogPathName : @"cafe", kHivesocialDialogPathDescription : @"클럽 바로가기"},
						@{kHiveSocialDialogPathName : @"game", kHivesocialDialogPathDescription : @"게임 목록 페이지"},
						@{kHiveSocialDialogPathName : @"inquiry", kHivesocialDialogPathDescription : @"1:1 문의하기"},
						];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)updateViewConstraints
{
	[super updateViewConstraints];
	
	if (_allConstraints && [_allConstraints count] > 0) {
		[self.contentView removeConstraints:_allConstraints];
		[_allConstraints removeAllObjects];
	}
	
	_allConstraints = [@[] mutableCopy];
	
	NSDictionary* views	= NSDictionaryOfVariableBindings(_facebookShareButton, _googleplusShareButton, _dialogButton,
														 _facebookFriends
#ifdef TEST_BUTTON
														 ,_testButton
#endif // TEST_BUTTON
														 );
	NSDictionary* metrics	= @{@"buttonH" : @(44)};
	NSArray* constraints	= @[@"H:|-[_facebookShareButton]-|",
								@"H:|-[_googleplusShareButton]-|",
								@"H:|-[_dialogButton]-|",
								@"H:|-[_facebookFriends]-|",
#ifdef TEST_BUTTON
								@"H:|-[_testButton]-|",
								@"V:|-[_facebookShareButton(==buttonH)]-[_googleplusShareButton(==buttonH)]-[_dialogButton(==buttonH)]-[_testButton(==buttonH)]-[_facebookFriends(==buttonH)]"
#else
								@"V:|-[_facebookShareButton(==buttonH)]-[_googleplusShareButton(==buttonH)]-[_dialogButton(==buttonH)]-[_facebookFriends(==buttonH)]"
#endif // TEST_BUTTON
								
								];
	
	for (NSString* str in constraints) {
		[_allConstraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	[self.contentView addConstraints:_allConstraints];
}

- (void)_onFacebookShare
{
	NSDictionary* params	= @{
								@"name" : @"name : 게임에서 입력한 name",
								@"link" : @"http://m.withhive.com/",
								@"description" : @"description : 게임에서 입력한 description",
								@"picture" : @"http://image-glb.qpyou.cn/hubweb/contents/%ED%95%98%EC%9D%B4%EB%B8%8C%EC%9A%A9(590x214)_140.png"
								//								@"picture" : @"http://splashpage.mtv.com/wp-content/uploads/2010/03/033010_ironman2.jpg"
								//								@"picture" : @"https://encrypted-tbn2.gstatic.com/images?q=tbn:ANd9GcRsHe2lXaVikStgjAxr0bSngJfE_iqvfXQ72fTYcs1Mm4AWvBWT9vfAWjo"
								};
	
	[HiveSocialManager hiveSocialRequest:@"facebook" actionName:@"social/share_app_activity" params:params completion:^(id result, NSError *error) {
		
	}];
}

- (void)_onGooglePlusShare
{
	NSDictionary* params = @{@"name" : @"Hive Share!!!",
							 @"link" : @"http://m.withhive.com/",
							 @"text" : @"Hello world! Hive Sample!",
							 @"description" : @"Hive Share Description!!",
							 @"picture" : @"http://hubimage.com2us.com/hubweb/contents/hive_logo_256.png"};
	[HiveSocialManager hiveSocialRequest:@"googleplus" actionName:@"social/share_app_activity" params:params completion:^(id result, NSError *error) {
		if (error)
			return;
	}];
}

- (void)_onHiveDialog
{
//	UIAlertView* alertView	= [[UIAlertView alloc] initWithTitle:@"Hive Dialog" message:nil delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"OK", nil];
//	alertView.alertViewStyle	= UIAlertViewStylePlainTextInput;
//	[alertView show];
	
	_modalViewController	= [[UIViewController alloc] init];
	_modalViewController.title		= TITLE_DIALOG;
	
	_modalViewController.navigationItem.rightBarButtonItems	= @[[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(_closeModalViewController)]];

	_dialogPathTableView	= [[UITableView alloc] initWithFrame:_modalViewController.view.frame style:UITableViewStyleGrouped];
	_dialogPathTableView.dataSource	= self;
	_dialogPathTableView.delegate	= self;
	[_modalViewController.view addSubview:_dialogPathTableView];
	
	[self.navigationController presentViewController:[[MainNavigationController alloc] initWithRootViewController:_modalViewController] animated:YES completion:nil];
}

- (void)_onHiveTest
{
	[HiveSocialManager hiveAsyncRequest:@"message/get_invitations" params:nil completion:^(id result, NSError *error) {
		
	}];
	
//	_modalViewController	= [[UIViewController alloc] init];
//	_modalViewController.title	= @"Test";
//	
//	_modalViewController.navigationItem.rightBarButtonItems	= @[[[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(_closeModalViewController)]];
//	
//	_customTestTableView	= [[UITableView alloc] initWithFrame:_modalViewController.view.frame style:UITableViewStyleGrouped];
//	_customTestTableView.dataSource	= self;
//	_customTestTableView.delegate	= self;
//	[_modalViewController.view addSubview:_customTestTableView];
//	
//	[self.navigationController presentViewController:[[MainNavigationController alloc] initWithRootViewController:_modalViewController] animated:YES completion:nil];
}

- (void)_onFacebookFriends
{
	[HiveSocialManager hiveSocialRequest:@"facebook" actionName:@"social/request_user_token" params:@{@"requestPermission" : @[@"friends"]} completion:^(id result, NSError *error) {
		if (error) {
			
		}
		else {
			NSDictionary *dict	= (NSDictionary *)result;
			
			NSString *token	= [dict valueForKey:@"token"];
			NSString *appid	= [dict valueForKey:@"appid"];
			
			if (!token || [token length] == 0
				|| !appid || [appid length] == 0) {
				return;
			}
			else {
				NSDictionary *params = @{@"appid" : appid,
										 @"token" : token};
				
				[HiveSocialManager hiveAsyncRequest:@"friends/facebook" params:params completion:^(id result, NSError *error) {
					if (error) {
						
					}
					else {
						
					}
				}];
			}
		}
	}];
}

- (void)_closeModalViewController
{
	if (!_modalViewController)
		return;
	
	[_modalViewController dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - 
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
	if (alertView.cancelButtonIndex == buttonIndex) {
		
	}
	else {
		NSString* path	= [alertView textFieldAtIndex:0].text;
		if ([[path trim] length] == 0)
			return;
		[HiveSocialManager hiveShowDialog:[path trim] completion:^(id result, NSError *error) {
			
		}];
	}
}

#pragma mark - 
#pragma mark
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
	if (_dialogPathTableView == tableView) {
		return DialogPathSectionMax;
	}
	else if (_customTestTableView == tableView) {
		return 1;
	}
	
	return 0;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	if (_dialogPathTableView == tableView) {
		switch (section) {
			case DialogPathSectionDefault:	return [_dialogPaths count];
			case DialogPathSectionCustom:	return 1;
			default:
				return 0;
		}
	}
	else if (_customTestTableView == tableView) {
		return CustomTestTypeMax;
	}
	
	
	return 0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	static NSString* CellIdentifier = @"Cell";
	
	UITableViewCell* cell;
	
	cell	= [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
	
	if (!cell) {
		cell	= [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
	}
	
	cell.textLabel.text	= @"";
	cell.detailTextLabel.text	= @"";
	
	if (_dialogPathTableView == tableView) {
		if (DialogPathSectionDefault ==  indexPath.section) {
			NSDictionary* dict	= _dialogPaths[indexPath.row];
			
			cell.textLabel.text			= dict[kHiveSocialDialogPathName];
			cell.detailTextLabel.text	= dict[kHivesocialDialogPathDescription];
		}
		else {
			cell.textLabel.text			= CUSTOM_DIALOG_PATH_TITLE;
			cell.detailTextLabel.text	= CUSTOM_DIALOG_PATH_DESCRIPTION;
		}
	}
	else if (_customTestTableView == tableView) {
		switch (indexPath.row) {
			case CustomTestTypeSendMessage:		cell.textLabel.text	= @"Send Message"; break;
			case CustomTestTypeRequestFriend: cell.textLabel.text	= @"Request Friend"; break;
			case CustomTestTypeSaveData:		cell.textLabel.text	= @"savedata/download";	break;
			default:
				break;
		}
	}
	
	return cell;
}

#pragma mark
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
	
	
	if (_dialogPathTableView == tableView) {
		if (DialogPathSectionDefault == indexPath.section) {
			NSString* path	= _dialogPaths[indexPath.row][kHiveSocialDialogPathName];
			
			if ([[path trim] length] == 0)
				return;
			
			[HiveSocialManager hiveShowDialog:[path trim] completion:^(id result, NSError *error) {
				
			}];
		}
		else {
			UIAlertView* alertView	= [[UIAlertView alloc] initWithTitle:@"Hive Dialog" message:nil delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:@"OK", nil];
			alertView.alertViewStyle	= UIAlertViewStylePlainTextInput;
			[alertView show];
		}
	}
	else if (_customTestTableView == tableView) {
		if (CustomTestTypeSendMessage == indexPath.row) {
			[HiveSocialManager hiveAsyncRequest:@"message/write" params:@{@"uid" : @"10", @"text" : @"Hi! Hive Message"} completion:^(id result, NSError *error) {
				
			}];
		}
		else if (CustomTestTypeRequestFriend == indexPath.row) {
			[HiveSocialManager hiveAsyncRequest:@"friends/request" params:@{@"friend_uid" : @"10", @"show_status" : @(YES)} completion:^(id result, NSError *error) {
				
			}];
		}
		else if (CustomTestTypeSaveData == indexPath.row){
			[HiveSocialManager hiveAsyncRequest:@"savedata/download" params:nil completion:^(id result, NSError *error) {
				
			}];
		}
	}
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
