//
//  FriendsViewController.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 11..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "FriendsViewController.h"
#import "HiveSocialManager.h"

@interface FriendsViewController ()
<UITableViewDataSource, UITableViewDelegate, SPSegmentedControllerDelegate>
{
	SPSegmentedController* _seg;
	UITableView* _tableView;
	
	NSMutableArray* _allConstraints;
	
	NSArray* _users;
}
@end

@implementation FriendsViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
	
	self.title	= @"친구";
	
	_users	= @[];
	
	_seg	= [[SPSegmentedController alloc] init];
	_seg.translatesAutoresizingMaskIntoConstraints	= NO;
	_seg.delegate	= self;
	_seg.items		= @[@"HIVE 모든 친구", @"같이 게임하는 친구"];
	_seg.segmentedSelectedIndex	= 0;
	[self.contentView addSubview:_seg];
	
	_tableView	= [[UITableView alloc] initWithFrame:CGRectZero style:UITableViewStylePlain];
	_tableView.dataSource	= self;
	_tableView.delegate		= self;
	_tableView.translatesAutoresizingMaskIntoConstraints	= NO;
	[self.contentView addSubview:_tableView];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)updateViewConstraints
{
	[super updateViewConstraints];
	
	if (_allConstraints && [_allConstraints count] > 0) {
		[self.contentView removeConstraints:_allConstraints];
		[_allConstraints removeAllObjects];
	}
	
	_allConstraints	= [@[] mutableCopy];
	
	NSDictionary* views;
	NSDictionary* metrics;
	NSArray* constraints;
	
	if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
		metrics	= @{@"segH" : @"48"};
	}
	else {
		metrics	= @{@"segH" : @"36"};
	}
	
	views		= NSDictionaryOfVariableBindings(_seg, _tableView);
	constraints	= @[@"H:|[_seg]|", @"H:|[_tableView]|", @"V:|[_seg(==segH)]-2@500-[_tableView]-0@500-|"];
	
	for (NSString* str in constraints) {
		[_allConstraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	[self.contentView addConstraints:_allConstraints];
}

#pragma mark - UITableViewDataSource Methods
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	return [_users count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	static NSString* CellIdentifer	= @"Cell";
	UITableViewCell* cell;
	
	cell	= [tableView dequeueReusableCellWithIdentifier:CellIdentifer];
	
	if (!cell) {
		cell	= [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifer];
	}
	
	SPUser* user	= _users[indexPath.row];
	
	if (user) {
		cell.textLabel.text	= [user screenName];
		
		if ([user.profileImageURL length] > 0) {
			__weak UITableViewCell* wCell	= cell;
			[SPImageCache getImageData:user.profileImageURL completion:^(NSData *imageData) {
				wCell.imageView.image	= [UIImage imageWithData:imageData];
				[wCell setNeedsLayout];
			}];
		}
	}
	
	return cell;
}

#pragma mark - SPSegmentedControllerDelegate Methods
- (void)didChangeSegmentedValue:(SPSegmentedController *)controller selectedIndex:(NSInteger)selectedIndex
{
	if (selectedIndex == 0) {
		[HiveSocialManager hiveAsyncRequest:@"friends" params:nil completion:^(id result, NSError *error) {
			NSArray* friendList	= result[@"friend_list"] ? result[@"friend_list"] : nil;
			
			if (!friendList)
				return;
			
			NSMutableArray* array	= [@[] mutableCopy];
			
			for (NSDictionary* friend in friendList) {
				SPUser* user	= [SPUser parseDictionary:friend];
				[array addObject:user];
			}
			
            self->_users	= (NSArray *)array;
			
            [self->_tableView reloadData];
		}];
	}
	else if (selectedIndex == 1) {
		[HiveSocialManager hiveAsyncRequest:@"friends/game" params:nil completion:^(id result, NSError *error) {
			NSArray* friendList	= result[@"friend_list"] ? result[@"friend_list"] : nil;
			
			if (!friendList)
				return;
			
			NSMutableArray* array	= [@[] mutableCopy];
			
			for (NSDictionary* friend in friendList) {
				SPUser* user	= [SPUser parseDictionary:friend];
				[array addObject:user];
			}
			
            self->_users	= (NSArray *)array;
			
            [self->_tableView reloadData];
		}];
	}
}

@end
