//
//  HiveSocialLog.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 14..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "HiveSocialLog.h"

NSString * const HiveSocialFlowDataDidInsertNotification = @"HiveSocialFlowDataDidInsertNotification";
NSString * const HiveSocialFlowDataDidUpdateNotification = @"HiveSocialFlowDataDidUpdateNotification";

static NSString* kHiveSocialFlowDataModel	= @"HiveSocialFlowDataModel";
static NSString* kHiveSocialFlowData	= @"HiveSocialFlowData";

static NSString* kHiveSocialFlowDataFieldRequest	= @"request";
static NSString* kHiveSocialFlowDataFieldRequestDate	= @"request_date";
static NSString* kHiveSocialFlowDataFieldResponse	= @"response";
static NSString* kHiveSocialFlowDataFieldResponseDate	= @"response_date";

@interface HiveSocialLog ()
{
	NSMutableArray* _newOne;
	NSMutableArray* _inserted;
	NSMutableArray* _updated;
}

@property (nonatomic, strong) NSManagedObjectContext* managedObjectContext;

@end

@implementation HiveSocialLog

+ (instancetype)sharedInstance
{
	static dispatch_once_t _once;
	static id _instance	= nil;
	
	dispatch_once(&_once, ^{
		_instance	= [[self alloc] init];
	});
	
	return _instance;
}

- (instancetype)init
{
	self	= [super init];
	
	if (self) {
	}
	
	return self;
}

- (void)initLog
{
    if (_managedObjectContext == nil) {
        NSManagedObjectModel *model = [NSManagedObjectModel mergedModelFromBundles:[NSBundle allBundles]];
        NSPersistentStoreCoordinator *psc = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:model];
        
        NSURL *url = [[[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject] URLByAppendingPathComponent:@"kHiveSocialFlowDataModel.sqlite"];
        
        NSDictionary *options = @{NSPersistentStoreFileProtectionKey: NSFileProtectionComplete,
                                  NSMigratePersistentStoresAutomaticallyOption:@YES};
        NSError *error = nil;
        NSPersistentStore *store = [psc addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:url options:options error:&error];
        if (!store)
        {
            NSLog(@"Error adding persistent store. Error %@",error);

            NSError *deleteError = nil;
            if ([[NSFileManager defaultManager] removeItemAtURL:url error:&deleteError])
            {
                error = nil;
                store = [psc addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:url options:options error:&error];
            }
            
            if (!store)
            {
                // Also inform the user...
                NSLog(@"Failed to create persistent store. Error %@. Delete error %@",error,deleteError);
                abort();
            }
        }
        
        self.managedObjectContext = [[NSManagedObjectContext alloc] initWithConcurrencyType:NSMainQueueConcurrencyType];
        self.managedObjectContext.persistentStoreCoordinator = psc;
    }
}

+ (NSArray *)sortByDate:(NSArray *)fetchData
{
	NSSortDescriptor* sortByRequestDate	= [[NSSortDescriptor alloc] initWithKey:kHiveSocialFlowDataFieldRequestDate ascending:YES];
	NSSortDescriptor* sortByResponseDate = [[NSSortDescriptor alloc] initWithKey:kHiveSocialFlowDataFieldResponseDate ascending:YES];
	
	NSArray* sortedResult	= [fetchData sortedArrayUsingDescriptors:@[sortByRequestDate, sortByResponseDate]];
	
	return sortedResult;
}

- (NSArray *)getAllFlowData
{
	NSManagedObjectContext* managedObjectContext	= _managedObjectContext;
	
	NSFetchRequest* fetchRequest	= [[NSFetchRequest alloc] initWithEntityName:kHiveSocialFlowData];
	
	NSSortDescriptor* sortByRequestDate	= [[NSSortDescriptor alloc] initWithKey:kHiveSocialFlowDataFieldRequestDate ascending:YES];
//	NSSortDescriptor* sortByResponseDate = [[NSSortDescriptor alloc] initWithKey:kHiveSocialFlowDataFieldResponseDate ascending:YES];
	
	[fetchRequest setSortDescriptors:@[sortByRequestDate, /* sortByResponseDate */]];
	
	NSArray* fetchData	= [managedObjectContext executeFetchRequest:fetchRequest error:nil];
	
	NSMutableArray* allData	= [@[] mutableCopy];
	
	for (NSManagedObject* obj in fetchData) {
		HiveSocialLogData* flowData	= [HiveSocialLogData create];
		flowData.object			= obj;
		flowData.requestString	= [obj valueForKey:kHiveSocialFlowDataFieldRequest];
		flowData.requestDate	= [obj valueForKey:kHiveSocialFlowDataFieldRequestDate];
		flowData.responseString	= [obj valueForKey:kHiveSocialFlowDataFieldResponse];
		flowData.responseDate	= [obj valueForKey:kHiveSocialFlowDataFieldResponseDate];
		
		[allData addObject:flowData];
	}
	
	return (NSArray *)allData;
}

- (void)deleteAllFlowData
{
	NSManagedObjectContext* managedObjectContext	= _managedObjectContext;
	
	for (HiveSocialLogData* logData in [self getAllFlowData]) {
		[managedObjectContext deleteObject:logData.object];
	}
	
	NSError* error	= nil;
	if (![managedObjectContext save:&error]) {
		NSLog(@"CANNOT delete : %@ %@", error, [error localizedDescription]);
	}
}

- (void)addNewOne:(id)object
{
	if (!_newOne) {
		_newOne	= [@[] mutableCopy];
	}
	
	if ([_newOne containsObject:object])
		return;
	
	[_newOne addObject:object];
}

- (NSArray *)getNewOne
{
	if (!_newOne)
		return nil;
	
	NSArray* sorted = [HiveSocialLog sortByDate:_newOne];
	
	NSMutableArray* allData	= [@[] mutableCopy];
	
	for (NSManagedObject* obj in sorted) {
		HiveSocialLogData* flowData	= [HiveSocialLogData create];
		flowData.object			= obj;
		flowData.requestString	= [obj valueForKey:kHiveSocialFlowDataFieldRequest];
		flowData.requestDate	= [obj valueForKey:kHiveSocialFlowDataFieldRequestDate];
		flowData.responseString	= [obj valueForKey:kHiveSocialFlowDataFieldResponse];
		flowData.responseDate	= [obj valueForKey:kHiveSocialFlowDataFieldResponseDate];
		
		[allData addObject:flowData];
	}
	
	[_newOne removeAllObjects];
	
	return allData;
}

- (NSArray *)getInserted
{
	if (!_inserted)
		return nil;
	
	NSArray* sorted = [HiveSocialLog sortByDate:_inserted];
	
	NSMutableArray* allData	= [@[] mutableCopy];
	
	for (NSManagedObject* obj in sorted) {
		HiveSocialLogData* flowData	= [HiveSocialLogData create];
		flowData.object			= obj;
		flowData.requestString	= [obj valueForKey:kHiveSocialFlowDataFieldRequest];
		flowData.requestDate	= [obj valueForKey:kHiveSocialFlowDataFieldRequestDate];
		flowData.responseString	= [obj valueForKey:kHiveSocialFlowDataFieldResponse];
		flowData.responseDate	= [obj valueForKey:kHiveSocialFlowDataFieldResponseDate];
		
		[allData addObject:flowData];
	}
	
	[_inserted removeAllObjects];
	
	return [[allData reverseObjectEnumerator] allObjects];
}

- (void)addInserted:(id)object
{
	if (!_inserted) {
		_inserted	= [@[] mutableCopy];
	}
	
	if ([_inserted containsObject:object])
		return;
	
	[_inserted addObject:object];
}

- (NSArray *)getUpdated
{
	if (!_updated)
		return nil;
	
	NSArray* sorted = [HiveSocialLog sortByDate:_updated];
	
	NSMutableArray* allData	= [@[] mutableCopy];
	
	for (NSManagedObject* obj in sorted) {
		HiveSocialLogData* flowData	= [HiveSocialLogData create];
		flowData.object			= obj;
		flowData.requestString	= [obj valueForKey:kHiveSocialFlowDataFieldRequest];
		flowData.requestDate	= [obj valueForKey:kHiveSocialFlowDataFieldRequestDate];
		flowData.responseString	= [obj valueForKey:kHiveSocialFlowDataFieldResponse];
		flowData.responseDate	= [obj valueForKey:kHiveSocialFlowDataFieldResponseDate];
		
		[allData addObject:flowData];
	}
	
	[_updated removeAllObjects];
	
	return allData;
}

- (void)addUpdated:(id)object
{
	if (!_updated) {
		_updated	= [@[] mutableCopy];
	}
	
	if ([_updated containsObject:object])
		return;
	
	[_updated addObject:object];
}

@end


@implementation HiveSocialLogData

- (NSString *)description
{
	return [NSString stringWithFormat:@"%@, requestString : %@, responseString : %@", self, _requestString, _responseString];
}

+ (instancetype)create
{
	return [HiveSocialLogData createWithFlush:NO];
}

+ (instancetype)createWithFlush:(BOOL)flush
{
	HiveSocialLogData* data	= [[HiveSocialLogData alloc] init];
	data.flush				= flush;
	
	return data;

}

- (void)setRequestString:(NSString *)requestString
{
	_requestString	= requestString;
	
	if (!_flush)
		return;

	NSLog(@"%@", requestString);
	
	BOOL isInsertNotification	= NO;
	
	if (!_object) {
		_object	= [NSEntityDescription insertNewObjectForEntityForName:kHiveSocialFlowData
                                                inManagedObjectContext:[HiveSocialLog sharedInstance].managedObjectContext];
		
		[[HiveSocialLog sharedInstance] addInserted:_object];
		isInsertNotification	= YES;
	}
	else {
		[[HiveSocialLog sharedInstance] addUpdated:_object];
		isInsertNotification	= NO;
	}
	
	[_object setValue:requestString forKey:kHiveSocialFlowDataFieldRequest];
	[_object setValue:[NSDate date] forKey:kHiveSocialFlowDataFieldRequestDate];
//	[_object setValue:[HiveFlowDataManager encodeCookieData] forKey:kHiveSocialFlowDataFieldRequestCookie];
	
	NSError* error = nil;
	if (![[HiveSocialLog sharedInstance].managedObjectContext save:&error]) {
		NSLog(@"CANNOT save %@ %@", error, [error localizedDescription]);
	}
	else {
		if (isInsertNotification) {
			[[NSNotificationCenter defaultCenter] postNotificationName:HiveSocialFlowDataDidInsertNotification object:nil];
		}
		else {
			[[NSNotificationCenter defaultCenter] postNotificationName:HiveSocialFlowDataDidUpdateNotification object:nil];
		}
	}
}

- (void)setResponseString:(NSString *)responseString
{
	_responseString	= responseString;

	if (!_flush)
		return;
	
	BOOL isInsertNotification	= NO;
	
	if (!_object) {
		_object	= [NSEntityDescription insertNewObjectForEntityForName:kHiveSocialFlowData
												inManagedObjectContext:[HiveSocialLog sharedInstance].managedObjectContext];
		
		[[HiveSocialLog sharedInstance] addInserted:_object];
		isInsertNotification	= YES;
	}
	else {
		[[HiveSocialLog sharedInstance] addUpdated:_object];
		isInsertNotification	= NO;
	}
	
	[_object setValue:responseString forKey:kHiveSocialFlowDataFieldResponse];
	[_object setValue:[NSDate date] forKey:kHiveSocialFlowDataFieldResponseDate];
//	[_object setValue:[HiveFlowDataManager encodeCookieData] forKey:kHiveSocialFlowDataFieldResponseCookie];
	
	NSError* error = nil;
	if (![[HiveSocialLog sharedInstance].managedObjectContext save:&error]) {
        NSLog(@"CANNOT save %@ %@", error, [error localizedDescription]);
	}
	else {
		if (isInsertNotification) {
			[[NSNotificationCenter defaultCenter] postNotificationName:HiveSocialFlowDataDidInsertNotification object:nil];
		}
		else {
			[[NSNotificationCenter defaultCenter] postNotificationName:HiveSocialFlowDataDidUpdateNotification object:nil];
		}
	}
}

- (void)requestString:(NSString *)requestString returnValue:(NSInteger)returnValue
{
	self.requestString	= requestString;
	
	if (returnValue != 0)
		self.responseString	= [NSString stringWithFormat:@"ret = %ld", (long)returnValue];
}

@end
