//
//  HiveSocialManager.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 9..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "HiveSocialManager.h"
#import "C2SHub2.h"
#import "C2SHub2Errors.h"
//#import <HiveSDK/C2SHub2.h>
//#import <HiveSDK/C2SHub2Errors.h>
#import "HiveSocialLog.h"

NSString * const SPHiveSocialDidInitializeNotification	= @"SPHiveSocialDidInitializeNotification";
NSString * const SPHiveSocialDidUnInitializeNotification	= @"SPHiveSocialDidUnInitializeNotification";

typedef NS_ENUM(NSInteger, HiveURLOption) {
	HiveURLOptionNONE	= 0,
	
	HiveURLProtocolHTTP		= 0 << 6,
	HiveURLProtocolHTTPS	= 1 << 6,
	
	HiveURLServerProduction	= 0 << 3,
	HiveURLServerTest		= 1 << 3,
	HiveURLServerStaging	= 2 << 3,
	
	HiveURLDomainCom2us		= 0 << 0,
	HiveURLDomainQPYou		= 1 << 0
};

static NSString * HiveSocialSDKDomain	= @"com.com2us.ios.hiveSocialSDK";

void hiveSocialCallback(const char* json, void* param);

static NSString* kSPHiveSocialFieldSessionKey	= @"__hive_social_sessionkey";

@interface HiveSocialObject : NSObject

@property (nonatomic, strong) NSString* responseString;

@property (nonatomic, copy) HiveSocialCompletion completion;

@property (nonatomic, strong) HiveSocialLogData* logData;

+ (HiveSocialObject *)create:(HiveSocialLogData *)logData completion:(HiveSocialCompletion)completion;

- (void)runCompletion;

@end

@implementation HiveSocialObject

+ (HiveSocialObject *)create:(HiveSocialLogData *)logData completion:(HiveSocialCompletion)completion
{
	HiveSocialObject* ins	= [[HiveSocialObject alloc] init];
	ins.logData				= logData;
	ins.completion			= completion;
	
	return ins;
}
- (void)runCompletion
{
	dispatch_async(dispatch_get_main_queue(), ^{
		NSError* error	= nil;
		
        NSDictionary* jsonDict	= [NSJSONSerialization JSONObjectWithData:[self->_responseString dataUsingEncoding:NSUTF8StringEncoding] options:NSJSONReadingMutableContainers error:&error];
		
		if (error) {
            if (self->_completion)
                self->_completion(self->_responseString, error);
			return;
		}
		
		if (![NSJSONSerialization isValidJSONObject:jsonDict]) {
            if (self->_completion)
                self->_completion(self->_responseString, [NSError errorWithDomain:HiveSocialSDKDomain code:-1 userInfo:nil]);
			return;
		}
		
		NSString* type		= jsonDict[@"type"];
		NSInteger errorCode	= jsonDict[@"error_code"] ? [jsonDict[@"error_code"] integerValue] : -9999;
		NSString* errorMsg	= jsonDict[@"error_msg"] ? jsonDict[@"error_msg"] : @"";
		
		if (errorCode != HUB_E_SUCCESS || [errorMsg length] > 0) {
            if (self->_completion)
                self->_completion(nil, [NSError errorWithDomain:HiveSocialSDKDomain code:errorCode userInfo:@{NSLocalizedDescriptionKey : errorMsg}]);
			return;
		}
		
		if ([type isEqualToString:@"auth"] || [type isEqualToString:@"guest/bind"]) {
			[[HiveSocialManager sharedInstance] reset];
			
			[HiveSocialManager sharedInstance].uid			= [SPUtility parseOrNil:jsonDict key:@"uid"];
			[HiveSocialManager sharedInstance].sessionKey	= [SPUtility parseOrNil:jsonDict key:@"sessionkey"];
		}
		else if ([type isEqualToString:@"logout"]) {
			[[HiveSocialManager sharedInstance] reset];
		}
		else if ([type isEqualToString:@"user/me"]) {
			[HiveSocialManager sharedInstance].userId	= [SPUtility parseOrNil:jsonDict key:@"id"];
			[HiveSocialManager sharedInstance].picture	= [SPUtility parseOrNil:jsonDict key:@"picture"];
			[HiveSocialManager sharedInstance].username	= [SPUtility parseOrNil:jsonDict key:@"name"];
			[HiveSocialManager sharedInstance].fbid		= [SPUtility parseOrNil:jsonDict key:@"fbid"];
			[HiveSocialManager sharedInstance].googleplusId	= [SPUtility parseOrNil:jsonDict key:@"googleplus_id"];
		}
		else if ([type isEqualToString:@"guest/create"]) {
			[HiveSocialManager sharedInstance].guestUid	= [SPUtility parseOrNil:jsonDict key:@"guest_uid"];
		}
		else if ([type isEqualToString:@"guest/auth"]) {
			[HiveSocialManager sharedInstance].picture	= @"http://hubimage.com2us.com/hubweb/avatar_img/public/default2.png";
		}
		
		
        if (self->_completion)
            self->_completion(jsonDict, nil);
	});
}

@end

@implementation HiveSocialManager

+ (instancetype)sharedInstance
{
	static dispatch_once_t _once;
	static id _instance	= nil;
	
	dispatch_once(&_once, ^{
		_instance	= [[self alloc] init];
	});
	
	return _instance;
}

- (instancetype)init
{
	self	= [super init];
	
	if (self) {
		_username	= @"";
		_fbid		= @"";
		_googleplusId	= @"";
	}
	
	return self;
}

+ (void)hiveInitializeEx:(HiveSocialCompletion)completion
{
	NSString* server	= [SPSettings loadSavedServer];
	
	HiveSocialServerState serverIdx;
	
	if ([server isEqualToString:@"staging"]) {
		serverIdx	= HiveSocialServerStaging;
	}
	else if ([server isEqualToString:@"sandbox"]) {
		serverIdx	= HiveSocialServerSandbox;
	}
	else if ([server isEqualToString:@"test"]) {
		serverIdx	= HiveSocialServerTest;
	}
	else {
		serverIdx	= HiveSocialServerLive;
	}
    
	int ret = CS_hubInitializeEx([[SPSettings loadSavedAppId] UTF8String], serverIdx);
	
	NSError* error = nil;
	if (ret != HUB_E_SUCCESS) {
		error	= [NSError errorWithDomain:HiveSocialSDKDomain code:ret userInfo:nil];
	}
	
	HiveSocialLogData* logData	= [HiveSocialLogData createWithFlush:YES];
	[logData requestString:[NSString stringWithFormat:@"CS_hubInitializeEx(%@, %d)", [SPSettings loadSavedAppId], serverIdx] returnValue:ret];
	logData.responseString	= [NSString stringWithFormat:@"ret = %ld", (long)ret];
	
//	CS_hubSetOption("hiveApp", "disable_dismiss");
    
	if (completion)
		completion(nil, error);
	
	if (!error) {
		[[NSNotificationCenter defaultCenter] postNotificationName:SPHiveSocialDidInitializeNotification object:nil];
		
		[HiveSocialManager sharedInstance].isInitialized	= YES;
        CS_hubSetAuthDialogClear();
	}
}

+ (void)hiveAuth:(HiveSocialCompletion)completion
{
	HiveSocialLogData* logData	= [HiveSocialLogData createWithFlush:YES];
	
//	int ret = CS_hubAuth(hiveSocialCallback, (__bridge_retained void *)[HiveSocialObject create:logData completion:completion]);
//	[logData requestString:@"CS_hubAuth()" returnValue:ret];
    
    NSDictionary* params = @{@"showGuestLogin" : @true};
    NSData* jsonData = [NSJSONSerialization dataWithJSONObject:params options:0 error:nil];
    NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    
    int ret = CS_hubAuthWithParam([jsonString UTF8String], hiveSocialCallback, (__bridge_retained void *)[HiveSocialObject create:logData completion:completion]);
    [logData requestString:@"CS_hubAuthWithParam()" returnValue:ret];
	
	NSError* error = nil;
	
	if (ret != HUB_E_SUCCESS) {
		NSLog(@"\n%s\nret = %d", __func__, ret);
		error	= [NSError errorWithDomain:HiveSocialSDKDomain code:ret userInfo:nil];
		
		if (completion)
			completion(nil, error);
	}
}

+ (void)hiveLogout:(HiveSocialCompletion)completion
{
	HiveSocialLogData* logData	= [HiveSocialLogData createWithFlush:YES];
	
	int ret	= CS_hubLogout(hiveSocialCallback, (__bridge_retained void *)[HiveSocialObject create:logData completion:completion]);
	[logData requestString:@"CS_hubLogout()" returnValue:ret];
	
	NSError* error = nil;
	
	if (ret != HUB_E_SUCCESS) {
		NSLog(@"\n%s\nret = %d", __func__, ret);
		error	= [NSError errorWithDomain:HiveSocialSDKDomain code:ret userInfo:nil];
		
		if (completion)
			completion(nil, error);
	}
}

+ (void)hiveUserDelete:(HiveSocialCompletion)completion
{
    HiveSocialLogData* logData    = [HiveSocialLogData createWithFlush:YES];
    
    int ret    = CS_hubUserDelete(hiveSocialCallback, (__bridge_retained void *)[HiveSocialObject create:logData completion:completion]);
    [logData requestString:@"CS_hubUserDelete()" returnValue:ret];
    
    NSError* error = nil;
    
    if (ret != HUB_E_SUCCESS) {
        NSLog(@"\n%s\nret = %d", __func__, ret);
        error    = [NSError errorWithDomain:HiveSocialSDKDomain code:ret userInfo:nil];
        
        if (completion)
            completion(nil, error);
    }
}

+ (void)hiveShowDialog:(NSString *)path completion:(HiveSocialCompletion)completion
{
	HiveSocialLogData* logData	= [HiveSocialLogData createWithFlush:YES];
	
	int ret = CS_hubShowDialog([path UTF8String], hiveSocialCallback, (__bridge_retained void *)[HiveSocialObject create:logData completion:completion]);
	[logData requestString:[NSString stringWithFormat:@"CS_hubShowDialog(%@)", path] returnValue:ret];
	
	NSError* error = nil;
	
	if (ret != HUB_E_SUCCESS) {
		NSLog(@"\n%s\nret = %d", __func__, ret);
		error	= [NSError errorWithDomain:HiveSocialSDKDomain code:ret userInfo:nil];
		
		if (completion)
			completion(nil, error);
	}
}

+ (void)hiveAsyncRequest:(NSString *)requestName params:(NSDictionary *)params completion:(HiveSocialCompletion)completion
{
	HiveSocialLogData* logData	= [HiveSocialLogData createWithFlush:YES];
	
	NSString* jsonString = nil;
	
	if (params) {
		NSData* jsonData = [NSJSONSerialization dataWithJSONObject:params options:0 error:nil];
		jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
	}
	
	int ret = CS_hubAsyncRequest([requestName UTF8String], [jsonString UTF8String], hiveSocialCallback, (__bridge_retained void *)[HiveSocialObject create:logData completion:completion]);
	[logData requestString:[NSString stringWithFormat:@"CS_hubAsyncRequest(%@, %@)", requestName, jsonString] returnValue:ret];
	
	NSError* error = nil;
	
	if (ret != HUB_E_SUCCESS) {
		NSLog(@"\n%s\nret = %d", __func__, ret);
		error	= [NSError errorWithDomain:HiveSocialSDKDomain code:ret userInfo:nil];
		
		if (completion)
			completion(nil, error);
	}
}

+ (void)hiveSocialRequest:(NSString *)serviceName actionName:(NSString *)actionName params:(NSDictionary *)params completion:(HiveSocialCompletion)completion
{
	HiveSocialLogData* logData	= [HiveSocialLogData createWithFlush:YES];
	
	NSString* jsonString = nil;
	
	if (params) {
		NSData* jsonData = [NSJSONSerialization dataWithJSONObject:params options:0 error:nil];
		jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
	}
	
	int ret = CS_hubSocialRequest([serviceName UTF8String], [actionName UTF8String], [jsonString UTF8String], hiveSocialCallback, (__bridge_retained void *)[HiveSocialObject create:logData completion:completion]);
	[logData requestString:[NSString stringWithFormat:@"CS_hubSocialRequest(%@, %@, %@)", serviceName, actionName, jsonString] returnValue:ret];
	
	NSError* error = nil;
	
	if (ret != HUB_E_SUCCESS) {
		NSLog(@"\n%s\nret = %d", __func__, ret);
		error	= [NSError errorWithDomain:HiveSocialSDKDomain code:ret userInfo:nil];
		
		if (completion)
			completion(nil, error);
	}
}

+ (void)hiveGuestAcquireUid:(HiveSocialCompletion)completion
{
	HiveSocialLogData* logData	= [HiveSocialLogData createWithFlush:YES];
	
	int ret = CS_hubGuestAcquireUid(hiveSocialCallback, (__bridge_retained void *)[HiveSocialObject create:logData completion:completion]);
	[logData requestString:[NSString stringWithFormat:@"CS_hubGuestAcquireUid()"] returnValue:ret];
	
	NSError* error = nil;
	
	if (ret != HUB_E_SUCCESS) {
		NSLog(@"\n%s\nret = %d", __func__, ret);
		error	= [NSError errorWithDomain:HiveSocialSDKDomain code:ret userInfo:nil];
		
		if (completion)
			completion(nil, error);
	}
}

+ (void)hiveGuestBind:(NSString *)guestUid userId:(NSString *)userId completion:(HiveSocialCompletion)completion
{
	HiveSocialLogData* logData	= [HiveSocialLogData createWithFlush:YES];
	
	int ret = CS_hubGuestBind([guestUid UTF8String], [userId UTF8String], hiveSocialCallback, (__bridge_retained void *)[HiveSocialObject create:logData completion:completion]);
	[logData requestString:[NSString stringWithFormat:@"CS_hubGuestBind(%@ %@)", guestUid, userId] returnValue:ret];
	
	NSError* error = nil;
	
	if (ret != HUB_E_SUCCESS) {
		NSLog(@"\n%s\nret = %d", __func__, ret);
		error	= [NSError errorWithDomain:HiveSocialSDKDomain code:ret userInfo:nil];
		
		if (completion)
			completion(nil, error);
	}
}

+ (void) hiveCSBotRequestWithCompletion:(HiveSocialCompletion)completion
{
	HiveSocialLogData* logData	= [HiveSocialLogData createWithFlush:YES];
	int ret = CS_hubShowDialogForChatbot([@"{\"evt_code\":1000,\"p1\":\"random\"}" UTF8String], hiveSocialCallback, (__bridge_retained void *)[HiveSocialObject create:logData completion:completion]);
	
	[logData requestString:[NSString stringWithFormat:@"CS_hubCSBot"] returnValue:ret];
	
	
}


+ (void)hiveApplicationDidBecomeActive
{
}

+ (BOOL)hiveApplicationOpenURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation
{
    return true;
}

- (void)reset
{
	_guestUid	= @"";
	
	_uid		= @"";
	self.sessionKey	= @"";
	
	_picture	= @"";
	_username	= @"";
	_fbid		= @"";
}

+ (NSString *)getHiveSocialVersion
{
	id class = NSClassFromString(@"Peppermint");
	if ([class respondsToSelector:@selector(getVersion)]) {
		NSString* str = [class performSelector:@selector(getVersion)];
		return [NSString stringWithFormat:@"v%@", str];
	}
	
	return @"Unknown Hive version.";
}

+ (NSString *)getAUVersion
{
	NSString* kActiveUserClass = @"ActiveUser";
	
	id cls = NSClassFromString(kActiveUserClass);
	
	NSString* version = @"Unkwon";
	if ([cls respondsToSelector:NSSelectorFromString(@"getVersion")]) {
		version	= [cls performSelector:NSSelectorFromString(@"getVersion")];
	}
	else if ([cls respondsToSelector:NSSelectorFromString(@"getSDKVersion")]) {
		version	= [cls performSelector:NSSelectorFromString(@"getSDKVersion")];
	}
	
	return [NSString stringWithFormat:@"v%@", version];
}

- (void)setSessionKey:(NSString *)sessionKey
{
	NSUserDefaults* ud = [NSUserDefaults standardUserDefaults];
	[ud setObject:kSPHiveSocialFieldSessionKey forKey:sessionKey];
	[ud synchronize];
}

@end


void hiveSocialCallback(const char* json, void* param)
{
	NSString* jsonString		= [[NSString alloc] initWithUTF8String:json];
	NSDictionary* jsonDict		= [NSJSONSerialization JSONObjectWithData:[jsonString dataUsingEncoding:NSUTF8StringEncoding] options:NSJSONReadingMutableContainers error:nil];
	NSData* jsonData			= [NSJSONSerialization dataWithJSONObject:jsonDict options:NSJSONWritingPrettyPrinted error:nil];
	NSString* adjustJsonString	= [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
	
	HiveSocialObject* obj	= (__bridge_transfer HiveSocialObject *)param;
	
	obj.responseString		= jsonString;
	
	obj.logData.responseString	= adjustJsonString;
	
	[obj runCompletion];
}
