//
//  InviteViewController.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 11..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "InviteViewController.h"
#import "HiveSocialManager.h"

@interface InviteViewController ()
<UITableViewDataSource, UITableViewDelegate, SPSegmentedControllerDelegate>
{
	SPSegmentedController* _seg;
	UITableView* _tableView;
	
	NSMutableArray* _allConstraints;
	
	NSArray* _users;
	NSArray* _fbReceivedInvite;
}
@end

@implementation InviteViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
	self.title	= @"초대";
	
	_users	= @[];
	_fbReceivedInvite	= @[];
	
	_seg	= [[SPSegmentedController alloc] init];
	_seg.translatesAutoresizingMaskIntoConstraints	= NO;
	_seg.delegate	= self;
	_seg.items	= @[@"HIVE", @"FB 초대 Dialog", @"FB 받은초대"];
	_seg.segmentedSelectedIndex	= 0;
	[self.contentView addSubview:_seg];
	
	_tableView	= [[UITableView alloc] initWithFrame:CGRectZero style:UITableViewStylePlain];
	_tableView.translatesAutoresizingMaskIntoConstraints	= NO;
	_tableView.dataSource	= self;
	_tableView.delegate		= self;
	[self.contentView addSubview:_tableView];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)updateViewConstraints
{
	[super updateViewConstraints];
	
	if (_allConstraints && [_allConstraints count] > 0) {
		[self.contentView removeConstraints:_allConstraints];
		[_allConstraints removeAllObjects];
	}
	
	_allConstraints	= [@[] mutableCopy];
	
	NSDictionary* views;
	NSDictionary* metrics;
	NSArray* constraints;
	
	if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
		metrics	= @{@"segH" : @"48"};
	}
	else {
		metrics	= @{@"segH" : @"36"};
	}
	
//	if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"7.0")) {
//		id topLayoutGuide		= [self topLayoutGuide];
//		id bottomLayoutGuide	= [self bottomLayoutGuide];
//		views		= NSDictionaryOfVariableBindings(topLayoutGuide, _seg, _tableView, bottomLayoutGuide);
//		constraints	= @[@"H:|[_seg]|", @"H:|-[_tableView]-|", @"V:|[topLayoutGuide][_seg(==segH)]-[_tableView]-[bottomLayoutGuide]|"];
//	}
//	else {
		views		= NSDictionaryOfVariableBindings(_seg, _tableView);
		constraints	= @[@"H:|[_seg]|", @"H:|[_tableView]|", @"V:|[_seg(==segH)]-2@500-[_tableView]-2@500-|"];
//	}
	
	for (NSString* str in constraints) {
		[_allConstraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	[self.contentView addConstraints:_allConstraints];
}

#pragma mark - UITableViewDataSource Methods
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	if (_seg.segmentedSelectedIndex == 0)
		return [_users count];
	else if (_seg.segmentedSelectedIndex == 1)
		return 1;
	else if (_seg.segmentedSelectedIndex == 2)
		return [_fbReceivedInvite count];
	else
		return 0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	static NSString* CellIdentifer	= @"Cell";
	UITableViewCell* cell;
	
	cell	= [tableView dequeueReusableCellWithIdentifier:CellIdentifer];
	
	if (!cell) {
		cell	= [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifer];
	}
	
	cell.textLabel.text			= @"";
	cell.detailTextLabel.text	= @"";
	cell.imageView.image		= nil;
	
	if (_seg.segmentedSelectedIndex == 0) {
		SPUser* user	= _users[indexPath.row];
		
		if (user) {
			cell.textLabel.text	= [user screenName];
			
			if ([user.profileImageURL length] > 0) {
				__weak UITableViewCell* wCell	 = cell;
				[SPImageCache getImageData:user.profileImageURL completion:^(NSData *imageData) {
					if (!imageData)
						return;
					
					wCell.imageView.image	= [UIImage imageWithData:imageData];
					
					[wCell setNeedsLayout];
				}];
			}
		}
	}
	else if (_seg.segmentedSelectedIndex == 1) {
		cell.textLabel.text	= @"Facebook Invite.";
	}
	else if (_seg.segmentedSelectedIndex == 2) {
		NSDictionary* requestInfo	= _fbReceivedInvite[indexPath.row];
		cell.textLabel.text			= requestInfo[@"from"][@"name"];
		cell.detailTextLabel.text	 = [NSString stringWithFormat:@"%@ %@", requestInfo[@"message"], requestInfo[@"created_time"]];
		
		NSString* fromUser	= requestInfo[@"from"][@"id"];
		
		__weak UITableViewCell* wCell	= cell;
		[HiveSocialManager hiveSocialRequest:@"facebook" actionName:@"social/user_profile_image" params:@{@"user_id" : fromUser} completion:^(id result, NSError *error) {
			if (error)
				return;
			
			[SPImageCache getImageData:result[@"url"] completion:^(NSData *imageData) {
				if (!imageData)
					return;
				wCell.imageView.image	= [UIImage imageWithData:imageData];
				[wCell setNeedsLayout];
			}];
		}];
	}
	
	return cell;
}

#pragma mark - UITableViewDelegate Methods
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
	
	if (_seg.segmentedSelectedIndex == 0) {
		SPUser* user = _users[indexPath.row];
		[HiveSocialManager hiveAsyncRequest:@"message/write" params:@{@"uid" : user.uid, @"text" : @"Invite", @"invite" : @(YES)} completion:^(id result, NSError *error) {

		}];
	}
	else if (_seg.segmentedSelectedIndex == 1) {
		[HiveSocialManager hiveSocialRequest:@"facebook" actionName:@"social/is_authorized" params:nil completion:^(id result, NSError *error) {
			if (error)
				return;
			
			BOOL isAuthorized	= [result[@"is_authorized"] boolValue];
			
			if (isAuthorized) {
				[self _requestInvite];
			}
			else {
				[HiveSocialManager hiveSocialRequest:@"facebook" actionName:@"social/request_user_profile" params:nil completion:^(id result, NSError *error) {
					if (error)
						return;
					[self _requestInvite];
				}];
			}
			
		}];
	}
}

#pragma mark - SPSegmentedControllerDelegate Methods
- (void)didChangeSegmentedValue:(SPSegmentedController *)controller selectedIndex:(NSInteger)selectedIndex
{
	if (selectedIndex == 0) {
		[HiveSocialManager hiveAsyncRequest:@"friends/other" params:nil completion:^(id result, NSError *error) {
			NSArray* friendsList	= result[@"friend_list"] ? result[@"friend_list"] : nil;
			
			if (!friendsList) {
                self->_users	= @[];
                [self->_tableView reloadData];
				return;
			}
			
			NSMutableArray* array	= [@[] mutableCopy];
			
			for (NSDictionary* friend in friendsList) {
				SPUser* user	= [SPUser parseDictionary:friend];
				
				[array addObject:user];
			}
			
            self->_users	= (NSArray *)array;
			
            [self->_tableView reloadData];
		}];
	}
	else if (selectedIndex == 1) {
		[_tableView reloadData];
	}
	else if (selectedIndex == 2) {
		[HiveSocialManager hiveSocialRequest:@"facebook" actionName:@"social/received_invite" params:nil completion:^(id result, NSError *error) {
			if (error) {
                self->_fbReceivedInvite = @[];
                [self->_tableView reloadData];
				return;
			}
			
            self->_fbReceivedInvite	= result[@"data"];

            [self->_tableView reloadData];
		}];
	}
}

- (void)_requestInvite
{
	[SPAlertView showAlertView:@"Facebook Invite" placeholder:@"[Hive] invite you!" at:self completion:^(id alertView, NSInteger buttonIndex) {
		if (buttonIndex == SPAlertViewCancelButtonIndex)
			return;
	
		UITextField* textField;
		
		if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"8.0")) {
			UIAlertController* alertController = (UIAlertController *)alertView;
			textField	= [[alertController textFields] firstObject];
		}
		else {
			UIAlertView* alert	= (UIAlertView *)alertView;
			textField	= [alert textFieldAtIndex:0];
		}
		
		NSString* str;
		
		if ([textField.text length] > 0) {
			str	= textField.text;
		}
		else if ([textField.placeholder length] > 0) {
			str	= textField.placeholder;
		}
		else {
			// invalid
			return;
		}
		
		[HiveSocialManager hiveSocialRequest:@"facebook" actionName:@"social/invite_dialog" params:@{@"message" : str} completion:^(id result, NSError *error) {
			
		}];
	}];
}

@end
