//
//  LogDetailViewController.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 14..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "LogDetailViewController.h"

@interface LogDetailViewController ()
{
	UILabel* _requestLabel;
	UILabel* _responseLabel;
	
	NSMutableArray* _allConstraints;
}
@end

@implementation LogDetailViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
	self.title	= @"Log Detail";
	
	_requestLabel	= [[UILabel alloc] init];
	_requestLabel.translatesAutoresizingMaskIntoConstraints	= NO;
	_requestLabel.text	= @"Request";
	[self.view addSubview:_requestLabel];
	
	_requestView	= [[UITextView alloc] init];
	_requestView.translatesAutoresizingMaskIntoConstraints	= NO;
	_requestView.editable	= NO;
	_requestView.layer.borderColor	= [UIColor blackColor].CGColor;
	_requestView.layer.borderWidth	= 1.0f;
	[self.view addSubview:_requestView];
	
	_responseLabel	= [[UILabel alloc] init];
	_responseLabel.translatesAutoresizingMaskIntoConstraints	= NO;
	_responseLabel.text	= @"Response";
	[self.view addSubview:_responseLabel];
	
	_responseView	= [[UITextView alloc] init];
	_responseView.translatesAutoresizingMaskIntoConstraints	= NO;
	_responseView.editable	= NO;
	_responseView.layer.borderColor	= [UIColor blackColor].CGColor;
	_responseView.layer.borderWidth	= 1.0f;
	[self.view addSubview:_responseView];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateViewConstraints) name:UIDeviceOrientationDidChangeNotification object:nil];
	
	self.navigationItem.rightBarButtonItem	= [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(_onClose:)];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self name:UIDeviceOrientationDidChangeNotification object:nil];
}

- (void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	
	_requestView.text	= _requestString;
	_responseView.text	= _responseString;
	
	[self updateViewConstraints];
}

- (void)updateViewConstraints
{
	[super updateViewConstraints];
	
	if (_allConstraints && [_allConstraints count] > 0) {
		[self.view removeConstraints:_allConstraints];
		[_allConstraints removeAllObjects];
	}
	
	_allConstraints	= [@[] mutableCopy];
	
	NSDictionary* views;
	NSDictionary* metrics = @{@"labelH" : @(24)};
	NSArray* constraints;
	
	if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"7.0")) {
		id topLayoutGuide	= [self topLayoutGuide];
		id bottomLayoutGuide	= [self bottomLayoutGuide];
		views	= NSDictionaryOfVariableBindings(topLayoutGuide, _requestLabel, _requestView, _responseLabel, _responseView, bottomLayoutGuide);
		if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
			constraints	= @[@"H:|-[_requestLabel]-|", @"H:|-[_requestView]-|", @"H:|-[_responseLabel]-|", @"H:|-[_responseView]-|",
							@"V:|[topLayoutGuide][_requestLabel(==labelH)]-[_requestView]-[_responseLabel(==labelH)]-[_responseView(==_requestView)][bottomLayoutGuide]|"
							];
		}
		else {
			constraints	= @[@"H:|-[_requestLabel]-[_responseLabel(==_requestLabel)]-|", @"H:|-[_requestView(==_requestLabel)]-[_responseView(==_requestLabel)]-|",
							@"V:|[topLayoutGuide][_requestLabel(==labelH)]-[_requestView][bottomLayoutGuide]|",
							@"V:|[topLayoutGuide][_responseLabel(==labelH)]-[_responseView][bottomLayoutGuide]|"
							];
		}
	}
	else {
		views	= NSDictionaryOfVariableBindings(_requestLabel, _requestView, _responseLabel, _responseView);
		if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
			constraints	= @[@"H:|-[_requestLabel]-|", @"H:|-[_requestView]-|", @"H:|-[_responseLabel]-|", @"H:|-[_responseView]-|",
							@"V:|[_requestLabel(==labelH)]-[_requestView]-[_responseLabel(==labelH)]-[_responseView(==_requestView)]|"
							];
		}
		else {
			constraints	= @[@"H:|-[_requestLabel]-[_responseLabel(==_requestLabel)]-|", @"H:|-[_requestView(==_requestLabel)]-[_responseView(==_requestLabel)]-|",
							@"V:|[_requestLabel(==labelH)]-[_requestView]|",
							@"V:|[_responseLabel(==labelH)]-[_responseView]|"
							];
		}
	}
	
	for (NSString* str in constraints) {
		[_allConstraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	[self.view addConstraints:_allConstraints];
}

- (void)_onClose:(id)sender
{
	[self.navigationController dismissViewControllerAnimated:YES completion:nil];
}

- (void)setRequestString:(NSString *)requestString
{
	_requestString	= requestString;
}

- (void)setResponseString:(NSString *)responseString
{
	_responseString	= responseString;
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
