//
//  MainViewController.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 9..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "MainViewController.h"
#import "LoginViewController.h"
#import "HiveSocialManager.h"
#import "SPGameInfoViewController.h"
#import "C2SActiveUser.h"

typedef NS_ENUM(NSInteger, SPActionSheetTag) {
	SPActionSheetTagProtocol	= 1001,
	SPActionSheetTagServer		= 1002,
	SPActionSheetTagDomain		= 1003,
	SPActionSheetTagCountry		= 1004,
	SPActionSheetTagLanguage	= 1005,
	SPActionSheetTagInquiry		= 1006,
	
};

typedef NS_ENUM(NSInteger, SPAlertViewTag){
	SPAlertViewTagGameIndex	= 2001,
	SPAlertViewTagAppId		= 2002,
};

@interface MainViewController ()
<UIActionSheetDelegate, UIAlertViewDelegate, SPGameInfoViewDelegate>
{
	SPLabel* _serverTitle;
	SPButton* _serverButton;
	
	SPLabel* _resultServerTitle;

	SPLabel* _countryTitle;
	SPButton* _countryButton;
	
	SPLabel* _inquiryTitle;
	SPButton* _inquiryButton;
	
	SPLabel* _languageTitle;
	SPButton* _languageButton;
	
	SPLabel* _appIdTitle;
	SPButton* _appIdButton;
	
	SPButton* _startButton;
	
	SPLabel* _appInfoLabel;
	
	
	NSMutableArray* _allConstraints;
    
}

@end

@implementation MainViewController


- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
	UIImageView* imageView	= [[UIImageView alloc] initWithImage:[UIImage imageWithContentsOfFile:SP_IMAGE_HOME_ICON]];
	
	self.navigationItem.titleView	= imageView;
	
	//
	_serverTitle	= [SPLabel label:TITLE_SERVER border:YES];
	[self.view addSubview:_serverTitle];

	_serverButton	= [SPButton buttonWithTitle:[SPSettings loadSavedServer] handler:^(SPButton *button) {
		[self _showActionSheet:SPActionSheetTagServer];
	}];
	_serverButton.translatesAutoresizingMaskIntoConstraints	= NO;
	[self.view addSubview:_serverButton];
	
	//
	_resultServerTitle	= [SPLabel label:nil border:YES];
	[self.view addSubview:_resultServerTitle];
	
	//
	_countryTitle	= [SPLabel label:TITLE_COUNTRY border:YES];
	[self.view addSubview:_countryTitle];
	
	_countryButton	= [SPButton buttonWithTitle:TITLE_COUNTRY handler:^(SPButton *button) {
		[self _showActionSheet:SPActionSheetTagCountry];
	}];
	_countryButton.translatesAutoresizingMaskIntoConstraints	= NO;
	[self.view addSubview:_countryButton];
	
	//
	_inquiryTitle	= [SPLabel label:TITLE_INQUIRY border:YES];
	[self.view addSubview:_inquiryTitle];
	
	_inquiryButton	= [SPButton buttonWithTitle:TITLE_INQUIRY handler:^(SPButton *button) {
		[self _showActionSheet:SPActionSheetTagInquiry];
	}];
	_inquiryButton.translatesAutoresizingMaskIntoConstraints	= NO;
	[self.view addSubview:_inquiryButton];
	
	//
	_languageTitle	= [SPLabel label:TITLE_LANGUAGE border:YES];
	[self.view addSubview:_languageTitle];
	
	_languageButton	= [SPButton buttonWithTitle:TITLE_LANGUAGE handler:^(SPButton *button) {
		[self _showActionSheet:SPActionSheetTagLanguage];
	}];
	_languageButton.translatesAutoresizingMaskIntoConstraints	= NO;
	[self.view addSubview:_languageButton];
	
	_appIdTitle	= [SPLabel label:TITLE_APPID border:YES];
	[self.view addSubview:_appIdTitle];
	
	_appIdButton	= [SPButton buttonWithTitle:TITLE_APPID handler:^(SPButton *button) {
		UIAlertView* alertView	= [[UIAlertView alloc] initWithTitle:@"AppId" message:nil delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
		alertView.alertViewStyle	= UIAlertViewStylePlainTextInput;
        [[alertView textFieldAtIndex:0] setText:[SPSettings loadSavedAppId]];
		alertView.tag			= SPAlertViewTagAppId;
		[alertView show];
	}];
	_appIdButton.translatesAutoresizingMaskIntoConstraints	= NO;
	[self.view addSubview:_appIdButton];
	
	////
	_startButton	= [SPButton buttonWithTitle:TITLE_START handler:^(SPButton *button) {
		[self _onStartButton];
	}];
	_startButton.translatesAutoresizingMaskIntoConstraints	= NO;
	_startButton.backgroundColor	= SP_BUTTON_COLOR_TYPE03;
	[self.view addSubview:_startButton];
	
	_appInfoLabel	= [SPLabel label:[NSString stringWithFormat:@"HiveSocial Sample %@ made by %@", SP_APP_VERSION, SP_APP_AUTHOR] border:NO];
	[self.view addSubview:_appInfoLabel];
	////
	
	[self _updateSubViews];
	
	[self _update];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

//- (void)viewWillAppear:(BOOL)animated
//{
//	[super viewWillAppear:animated];
//	
//	if (SYSTEM_VERSION_LESS_THAN(@"8.0")) {
//		[self _updateSubViews];
//	}
//}
//
//- (void)viewDidLayoutSubviews
//{
//	if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"8.0")) {
//		[self _updateSubViews];
//	}
//}

- (void)orientationChanged
{
	[self _updateSubViews];
}

- (void)_updateSubViews
{
    
	if (_allConstraints && [_allConstraints count] > 0) {
		[self.view removeConstraints:_allConstraints];
		[_allConstraints removeAllObjects];
	}
	
	_allConstraints	= [@[] mutableCopy];
	
	NSMutableDictionary* views;
	NSMutableArray* constraints;
	
	views	= [NSDictionaryOfVariableBindings(_serverTitle,
											 _serverButton,
											 _resultServerTitle,
											 _countryTitle, _countryButton,
											 _inquiryTitle, _inquiryButton,
											 _languageTitle, _languageButton,
											 _appIdTitle, _appIdButton,
											 _startButton,
											 _appInfoLabel) mutableCopy];
	
	if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"7.0")) {
		id topLayoutGuide		= [self topLayoutGuide];
		id bottomLayoutGuide	= [self bottomLayoutGuide];
		[views addEntriesFromDictionary:NSDictionaryOfVariableBindings(topLayoutGuide, bottomLayoutGuide)];
		constraints	= [@[@"V:|[topLayoutGuide]-[_serverTitle(==titleH)]",
						 @"V:[_startButton(==buttonH)]-[_appInfoLabel]-[bottomLayoutGuide]|",
						 ] mutableCopy];
	}
	else {
		constraints	= [@[@"V:|[_serverTitle(==titleH)]",
						 @"V:[_startButton(==buttonH)]-[_appInfoLabel]|"] mutableCopy];
	}
	
	NSDictionary* metrics;
	
	if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
		metrics = @{@"titleH" : @(32), @"buttonH" : @(32), @"optionW" : @(100), @"searchW" : @(60)};
	}
	else {
		metrics = @{@"titleH" : @(28), @"buttonH" : @(28), @"optionW" : @(80), @"searchW" : @(60)};
	}
	
	if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
		[constraints addObjectsFromArray:@[@"H:|-[_serverTitle]-|",
										   @"H:|-[_serverButton]-|",
										   @"V:[_serverTitle]-[_serverButton(==buttonH)]",
										   @"H:|-[_resultServerTitle]-|",
										   @"V:[_serverButton]-[_resultServerTitle(==titleH)]",
										   
										   //									   @"V:[_startButton(==buttonH)]-[_appInfoLabel]|",
										   @"H:|-24-[_startButton(==200@500)]-24-|",
										   @"H:|-0@500-[_appInfoLabel]-0@500-|"
										   ]];
		
		[constraints addObjectsFromArray:@[@"V:[_resultServerTitle]-[_countryTitle(==titleH)]",
										   @"V:[_resultServerTitle]-[_countryButton(==buttonH)]",
										   @"H:|-[_countryTitle(==optionW)]-[_countryButton]-|",
										   @"V:[_countryTitle]-[_inquiryTitle(==titleH)]",
										   @"V:[_countryTitle]-[_inquiryButton(==buttonH)]",
										   @"H:|-[_inquiryTitle(==optionW)]-[_inquiryButton]-|",
										   @"V:[_inquiryTitle]-[_languageTitle(==titleH)]",
										   @"V:[_inquiryTitle]-[_languageButton(==buttonH)]",
										   @"H:|-[_languageTitle(==optionW)]-[_languageButton]-|",
										   @"V:[_languageTitle]-[_appIdTitle(==titleH)]",
										   @"V:[_languageTitle]-[_appIdButton(==buttonH)]",
										   @"H:|-[_appIdTitle(==optionW)]-[_appIdButton]-|",
										   ]];
	}
	else {
		
		if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"7.0")) {
			[constraints addObjectsFromArray:@[
											   @"H:|-[_serverTitle(==optionW)]-[_serverButton]-|",
											   @"V:|[topLayoutGuide]-[_serverButton(==_serverTitle)]",
											   @"V:[_serverButton]-[_resultServerTitle(==_serverTitle)]",
			 ]];
		}
		else {
			[constraints addObjectsFromArray:@[
											   @"H:|-[_serverTitle(==optionW)]-[_serverButton]-[_resultServerTitle(==_serverButton)]-|",
											   @"V:|[_serverButton(==_serverTitle)]",
											   @"V:|[_serverButton]-[_resultServerTitle(==_serverTitle)]",
											   ]];
		}
		[constraints addObjectsFromArray:@[
										   //									   @"V:[_startButton(==buttonH)]-[_appInfoLabel]|",
										   @"H:|-24-[_startButton(==200@500)]-24-|",
										   @"H:|-0@500-[_appInfoLabel]-0@500-|"
										   ]];
		
		[constraints addObjectsFromArray:@[@"H:|-[_resultServerTitle]-|",
										   @"V:[_resultServerTitle]-[_countryTitle(==titleH)]",
										   @"V:[_resultServerTitle]-[_countryButton(==buttonH)]",
										   @"V:[_resultServerTitle]-[_inquiryTitle(==titleH)]",
										   @"V:[_resultServerTitle]-[_inquiryButton(==buttonH)]",
										   @"V:[_resultServerTitle]-[_languageTitle(==titleH)]",
										   @"V:[_resultServerTitle]-[_languageButton(==buttonH)]",
										   @"H:|-[_countryTitle(==optionW)]-[_countryButton]-[_inquiryTitle(==optionW)]-[_inquiryButton(==_countryButton)]-[_languageTitle(==optionW)]-[_languageButton(==_countryButton)]-|",
										   
										   @"V:[_countryTitle]-[_appIdTitle(==titleH)]",
										   @"V:[_countryTitle]-[_appIdButton(==buttonH)]",
										   @"H:|-[_appIdTitle(==optionW)]-[_appIdButton]|",
										   ]];
	}
	
	for (NSString* str in constraints) {
		[_allConstraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
//		[self.view addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	[self.view addConstraints:_allConstraints];
}

- (void)_update
{
	[_serverButton setText:[SPSettings loadSavedServer]];
	[_appIdButton setText:[SPSettings loadSavedAppId]];
	[_countryButton setText:[SPSettings loadSavedCountry]];
	[_languageButton setText:[SPSettings loadSavedLanguage]];
	[_inquiryButton setText:[SPSettings loadSavedInquiryLanguage]];
	
	[self _updateResultServerTitle];
    
}

- (void)_updateResultServerTitle
{
	_resultServerTitle.textLabel.text	= [SPSettings getSPSettingFullPath];
}

- (void)_showActionSheet:(NSInteger)tag
{
	NSString* title;
	NSArray* items;
	
	switch (tag) {
		case SPActionSheetTagProtocol:
			title	= TITLE_PROTOCOL;
			items	= [SPSettings getAllProtocols];
			break;
		case SPActionSheetTagServer:
			title	= TITLE_SERVER;
			items	= [SPSettings getAllServers];
			break;
		case SPActionSheetTagDomain:
			title	= TITLE_DOMAIN;
			items	= [SPSettings getAllDomains];
			break;
		case SPActionSheetTagCountry:
			title	= TITLE_COUNTRY;
			items	= [SPSettings getAllCountries];
			break;
		case SPActionSheetTagLanguage:
			title	= TITLE_LANGUAGE;
			items	= [SPSettings getAllLanguages];
			break;
		case SPActionSheetTagInquiry:
		{
			title	= TITLE_INQUIRY;
			NSMutableArray* array	= [@[] mutableCopy];
			for (NSDictionary* dict in [SPSettings getAllInquiryLanguage]) {
				[array addObject:dict[@"languageName"]];
			}
			items	= array;
		}
			break;
		default:
			return;
	}
	
	UIActionSheet* actionSheet;
	
	if (SYSTEM_VERSION_LESS_THAN(@"8.0") && UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
		actionSheet	= [[UIActionSheet alloc] initWithTitle:title delegate:self cancelButtonTitle:nil destructiveButtonTitle:nil otherButtonTitles:nil];
	}
	else {
		actionSheet	= [[UIActionSheet alloc] initWithTitle:title delegate:self cancelButtonTitle:TITLE_CANCEL destructiveButtonTitle:nil otherButtonTitles:nil];
	}
	
	actionSheet.tag			= tag;
	for (NSString* item in items) {
		[actionSheet addButtonWithTitle:item];
	}
	[actionSheet showInView:self.view];
}

- (void)_onStartButton
{
	NSString* savedAppId	= [SPSettings loadSavedAppId];
	
	if ([savedAppId length] == 0) {
		[SPAlertView showAlertView:@"Warning" message:@"저장된 AppId가 없습니다.\nGame Index입력 후 '검색' 시도 후 시작바랍니다." at:self];
		return;
	}
	
    [HiveSocialManager hiveInitializeEx:^(id result, NSError *error) {
        LoginViewController* vc	= [[LoginViewController alloc] init];
        self.navigationController.viewControllers	= @[vc];
    }];
    
}

#pragma mark - UIActionSheetDelegate
- (void)actionSheet:(UIActionSheet *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex
{
	NSInteger tag	= actionSheet.tag;
	NSString* title	= [actionSheet buttonTitleAtIndex:buttonIndex];
	
	if (tag == SPActionSheetTagProtocol) {
		[SPSettings saveProtocol:title];
	}
	else if (tag == SPActionSheetTagServer) {
		[SPSettings saveServer:title];
	}
	else if (tag == SPActionSheetTagDomain) {
		[SPSettings saveDomain:title];
	}
	else if (tag == SPActionSheetTagCountry) {
		[SPSettings saveCountry:title];
	}
	else if (tag == SPActionSheetTagLanguage) {
	
		[SPSettings saveLanguage:title];
	}
	else if (tag == SPActionSheetTagInquiry) {
		[SPSettings saveInquiryLanguage:title];
	}
	
	[self _update];
}

#pragma mark - UIAlertViewDelegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
	UITextField* textField	= [alertView textFieldAtIndex:0];
	
	if ([[textField.text trim] length] == 0)
		return;
	
	if (alertView.tag == SPAlertViewTagGameIndex) {
		[SPSettings saveGameIndex:textField.text];
	}
	else if (alertView.tag == SPAlertViewTagAppId) {
		[SPSettings saveAppId:textField.text];
	}
	
	[self _update];
}

#pragma mark -
- (void)didSelectGameTitle:(NSString *)title AppId:(NSString *)appid
{
	if (!title || [title isKindOfClass:[NSNull class]])
		title = @"";
	
	[SPSettings saveAppId:appid];
	
	[SPSettings saveGameTitle:title];
	
	[self _update];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
