//
//  SPAlertView.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 16..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "SPAlertView.h"

@interface SPAlertView ()
<UIAlertViewDelegate>
@end

@implementation SPAlertView

+ (void)showAlertView:(NSString *)title message:(NSString *)message at:(UIViewController *)viewController
{
	[self showAlertView:title message:message buttons:nil at:viewController completion:nil];
}

+ (void)showAlertView:(NSString *)title message:(NSString *)message buttons:(NSArray *)buttonTitles at:(UIViewController *)viewController completion:(SPAlertViewCompletion)completion
{
	if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"8.0")) {
		UIAlertController* alertController	= [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];

		__block SPAlertViewCompletion bCompletion	= completion;
		
		UIAlertAction* cancel	= [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
			[alertController dismissViewControllerAnimated:YES completion:^{
				
			}];
			
			if (bCompletion)
				bCompletion(alertController, SPAlertViewCancelButtonIndex);
		}];
		
		[alertController addAction:cancel];
		
		for (NSString* buttonTitle in buttonTitles) {
			[alertController addAction:[UIAlertAction actionWithTitle:buttonTitle style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
				[alertController dismissViewControllerAnimated:YES completion:^{
					
				}];
				
				if (bCompletion)
					bCompletion(alertController, [buttonTitles indexOfObject:buttonTitle]);
			}]];
		}
		
		[viewController presentViewController:alertController animated:YES completion:nil];
	}
	else {

		static dispatch_once_t _once;
		static SPAlertView* _instance = nil;
		
		dispatch_once(&_once, ^{
			_instance	= [[SPAlertView alloc] init];
		});
		
		_instance.completion	= completion;
		
		UIAlertView* alertView	= [[UIAlertView alloc] initWithTitle:title message:message delegate:_instance cancelButtonTitle:@"Cancel" otherButtonTitles:nil];
		for (NSString* buttonTitle in buttonTitles) {
			[alertView addButtonWithTitle:buttonTitle];
		}
		
		[alertView show];
	}
}

+ (void)showAlertView:(NSString *)title placeholder:(NSString *)placeholder at:(UIViewController *)viewController completion:(SPAlertViewCompletion)completion
{
	if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"8.0")) {
		UIAlertController* alertController	= [UIAlertController alertControllerWithTitle:title message:nil preferredStyle:UIAlertControllerStyleAlert];
		
		__block SPAlertViewCompletion bCompletion	= completion;
		
		UIAlertAction* cancel	= [UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
			[alertController dismissViewControllerAnimated:YES completion:^{
				
			}];
			
			if (bCompletion)
				bCompletion(alertController, SPAlertViewCancelButtonIndex);
		}];
		[alertController addAction:cancel];
		
		UIAlertAction* okButton	= [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
			[alertController dismissViewControllerAnimated:YES completion:^{
				
			}];
			
			if (bCompletion)
				bCompletion(alertController, 0);
		}];
		[alertController addAction:okButton];
		
		[alertController addTextFieldWithConfigurationHandler:^(UITextField *textField) {
			textField.placeholder	= placeholder;
		}];
		
		
		[viewController presentViewController:alertController animated:YES completion:nil];
	}
	else {
		
		static dispatch_once_t _once;
		static SPAlertView* _instance = nil;
		
		dispatch_once(&_once, ^{
			_instance	= [[SPAlertView alloc] init];
		});
		
		_instance.completion	= completion;
		
		UIAlertView* alertView		= [[UIAlertView alloc] initWithTitle:title message:nil delegate:_instance cancelButtonTitle:@"Cancel" otherButtonTitles:@"OK", nil];
		alertView.alertViewStyle	= UIAlertViewStylePlainTextInput;
		((UITextField *)[alertView textFieldAtIndex:0]).placeholder	= placeholder;
		
		[alertView show];
	}
}

#pragma mark - UIAlertViewDelegate Methods
- (void)alertViewCancel:(UIAlertView *)alertView
{
	if (_completion)
		_completion(alertView, SPAlertViewCancelButtonIndex);
}

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
	if ([alertView numberOfButtons] > 1) {
		if (_completion)
			_completion(alertView, --buttonIndex);
	}
}

@end
