//
//  SPButton.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 9..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "SPButton.h"

#define USE_SPBUTTON_ANIMATION	SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"8.0")

@implementation SPButton

+ (SPButton *)button:(SPButtonHandler)handler
{
	SPButton* btn	= [[SPButton alloc] init];
	btn.handler		= handler;
	
	return btn;
}

+ (SPButton *)buttonWithTitle:(NSString *)title handler:(SPButtonHandler)handler
{
	return [self buttonWithTitle:title imagePath:nil handler:handler];
}

+ (SPButton *)buttonWithImagePath:(NSString *)imagePath handler:(SPButtonHandler)handler
{
	return [self buttonWithTitle:nil imagePath:imagePath handler:handler];
}

+ (SPButton *)buttonWithTitle:(NSString *)title imagePath:(NSString *)imagePath handler:(SPButtonHandler)handler
{
	SPButton* btn	= [[SPButton alloc] init];
	btn.handler		= handler;

	if (!USE_SPBUTTON_ANIMATION) {
		if (title) {
			[btn.button setTitle:title forState:UIControlStateNormal];
			[btn.button setTitleColor:[UIColor blackColor] forState:UIControlStateNormal];
			btn.button.titleLabel.font	= [UIFont boldSystemFontOfSize:14];;
		}
		
		if (imagePath) {
			[btn.button setImage:[UIImage imageWithContentsOfFile:imagePath] forState:UIControlStateNormal];
			[btn.button setContentVerticalAlignment:UIControlContentVerticalAlignmentBottom];
			btn.button.imageView.contentMode	= UIViewContentModeScaleAspectFit;
			btn.button.contentMode	= UIViewContentModeScaleAspectFit;
//			btn.button.imageEdgeInsets	= UIEdgeInsetsMake(2, 2, 10, 2);
		}
	}
	else {
		if (title) {
			btn.textLabel.text	= title;
			btn.textLabel.font	= [UIFont boldSystemFontOfSize:14];
		}
		
		if (imagePath) {
			btn.imageView.image	= [UIImage imageWithContentsOfFile:imagePath];
		}
	}
	
	btn.backgroundColor	= SP_BUTTON_COLOR_TYPE01;
	
	return btn;
}

- (instancetype)init
{
	self	= [super init];
	
	if (self) {
		if (!USE_SPBUTTON_ANIMATION) {
			_button			= [UIButton buttonWithType:UIButtonTypeCustom];
			[_button addTarget:self action:@selector(_onAction:) forControlEvents:UIControlEventTouchUpInside];
			_button.backgroundColor	= [UIColor clearColor];
			_button.translatesAutoresizingMaskIntoConstraints	= NO;
			_button.titleLabel.lineBreakMode	= NSLineBreakByWordWrapping;
			_button.titleLabel.numberOfLines	= 0;
			_button.titleLabel.textAlignment	= NSTextAlignmentCenter;
			_button.titleLabel.minimumScaleFactor	= .8;
			_button.titleLabel.adjustsFontSizeToFitWidth	= YES;
			[self addSubview:_button];
		}
		else {
			
			_textLabel		= [[UILabel alloc] init];
			_textLabel.translatesAutoresizingMaskIntoConstraints	= NO;
			_textLabel.textAlignment	= NSTextAlignmentCenter;
			_textLabel.backgroundColor	= [UIColor clearColor];
			_textLabel.textColor		= [UIColor blackColor];
			_textLabel.minimumScaleFactor	= .8;
			_textLabel.adjustsFontSizeToFitWidth	= YES;
			[self addSubview:_textLabel];
			
			_imageView		= [[UIImageView alloc] init];
			_imageView.translatesAutoresizingMaskIntoConstraints	= NO;
			_imageView.contentMode		= UIViewContentModeScaleAspectFit;
			_imageView.backgroundColor	= [UIColor clearColor];
			[self addSubview:_imageView];
		}
		
		self.layer.shadowColor	= [UIColor blackColor].CGColor;
		self.layer.shadowOffset	= CGSizeMake(0, 1);
		self.layer.shadowOpacity	= 0.7;
	}
	
	return self;
}

- (void)layoutSubviews
{
	NSDictionary* views;
	NSDictionary* metrics	= @{};
	NSArray* constraints;

	if (!USE_SPBUTTON_ANIMATION) {
		metrics		= @{};
		views		= NSDictionaryOfVariableBindings(_button);
		constraints	= @[@"H:|[_button]|", @"V:|[_button]|"];
	}
	else {
		_imageView.hidden	= YES;
		_textLabel.hidden	= YES;
		
		if (_imageView.image && [_textLabel.text length] > 0) {
			_imageView.hidden	= NO;
			_textLabel.hidden	= NO;
			metrics		= @{@"textH" : @(14)};
			views		= NSDictionaryOfVariableBindings(_imageView, _textLabel);
			constraints	= @[@"H:|[_imageView]|", @"H:|[_textLabel]|", @"V:|[_imageView][_textLabel(==textH)]|"];
		}
		else if (_imageView.image) {
			_imageView.hidden	= NO;
			views		= NSDictionaryOfVariableBindings(_imageView);
			constraints	= @[@"H:|[_imageView]|", @"V:|[_imageView]|"];
		}
		else if ([_textLabel.text length] > 0) {
			_textLabel.hidden	= NO;
			views		= NSDictionaryOfVariableBindings(_textLabel);
			constraints	= @[@"H:|[_textLabel]|", @"V:|[_textLabel]|"];
		}
	}
	
	for (NSString* str in constraints) {
		[self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str
																	 options:0
																	 metrics:metrics
																	   views:views]];
	}
	
	if (SYSTEM_VERSION_LESS_THAN(@"8.0")) {
		[super layoutSubviews];
	}
}

- (void)setEnabled:(BOOL)enabled
{
	_enabled	= enabled;
	
	self.alpha	= enabled ? 1.0 : 0.2;
}

- (void)_onAction:(id)sender
{
	if (_handler)
		_handler(self);
}

//- (void)updateConstraints
//{
//	[super updateConstraints];
//	
//	NSDictionary* views;
//	NSDictionary* metrics	= @{};
//	NSArray* constraints;
//	
//	_imageView.hidden	= YES;
//	_textLabel.hidden	= YES;
//	
//	if (_imageView.image && [_textLabel.text length] > 0) {
//		_imageView.hidden	= NO;
//		_textLabel.hidden	= NO;
//		metrics		= @{@"textH" : @(CGRectGetHeight(self.frame) * 0.2)};
//		views		= NSDictionaryOfVariableBindings(_imageView, _textLabel);
//		constraints	= @[@"H:|[_imageView]|", @"H:|[_textLabel]|", @"V:|[_imageView][_textLabel(==textH)]|"];
//	}
//	else if (_imageView.image) {
//		_imageView.hidden	= NO;
//		views		= NSDictionaryOfVariableBindings(_imageView);
//		constraints	= @[@"H:|[_imageView]|", @"V:|[_imageView]|"];
//	}
//	else if ([_textLabel.text length] > 0) {
//		_textLabel.hidden	= NO;
//		views		= NSDictionaryOfVariableBindings(_textLabel);
//		constraints	= @[@"H:|[_textLabel]|", @"V:|[_textLabel]|"];
//	}
//	
//	for (NSString* str in constraints) {
//		[self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str
//																	 options:0
//																	 metrics:metrics
//																	   views:views]];
//	}
//}

- (void)setText:(NSString *)text
{
	if (!USE_SPBUTTON_ANIMATION) {
		[_button setTitle:text forState:UIControlStateNormal];
	}
	else {
		_textLabel.text		= text;
	}
}

- (NSString *)title
{
	if (!USE_SPBUTTON_ANIMATION) {
		return _button.titleLabel.text;
	}
	else {
		return _textLabel.text;
	}
}

#pragma mark - Touch Events
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event
{
	_isTouched	= YES;
	
	if (USE_SPBUTTON_ANIMATION) {
		[UIView animateWithDuration:0.1 animations:^{
			self.transform	= CGAffineTransformMakeScale(0.9, 0.9);
		}];
	}
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event
{
	if (!_isTouched)
		return;
	
	UITouch* touch		= [touches anyObject];
	CGPoint location	= [touch locationInView:self];
	
	if ([self pointInside:location withEvent:event]) {
		
	}
	else {
		_isTouched	= NO;
		
//		[[UIApplication sharedApplication] beginIgnoringInteractionEvents];
		
		[self touchesEnded:touches withEvent:event];
		
		if (USE_SPBUTTON_ANIMATION) {
			[UIView animateWithDuration:0.1 animations:^{
				self.transform	= CGAffineTransformMakeScale(1.0, 1.0);
			}];
		}
	}
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event
{
	if (_isTouched) {
		if (USE_SPBUTTON_ANIMATION) {
			[UIView animateWithDuration:0.1 animations:^{
				self.transform	= CGAffineTransformMakeScale(1.1, 1.1);
			} completion:^(BOOL finished) {
				[UIView animateWithDuration:0.1 animations:^{
					self.transform	= CGAffineTransformMakeScale(1.0, 1.0);
				} completion:^(BOOL finished) {
					if ([[UIApplication sharedApplication] isIgnoringInteractionEvents])
						[[UIApplication sharedApplication] endIgnoringInteractionEvents];
					
                    if (self->_handler)
                        self->_handler(self);
				}];
			}];
		}
		else {
			if ([[UIApplication sharedApplication] isIgnoringInteractionEvents])
				[[UIApplication sharedApplication] endIgnoringInteractionEvents];
			
			if (_handler)
				_handler(self);

		}
	}
	else {
		if ([[UIApplication sharedApplication] isIgnoringInteractionEvents])
			[[UIApplication sharedApplication] endIgnoringInteractionEvents];
	}
	
	_isTouched	= NO;
}

@end
