//
//  SPGameInfoViewController.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 10. 14..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "SPGameInfoViewController.h"

static NSString* CollectionCellIdentifier = @"CollectionCell";

@interface SPGameInfoCollectionCell : UICollectionViewCell
{
	UILabel* _keyLabel;
	UILabel* _valueLabel;
	
	NSMutableArray* _constraints;
}
@property (nonatomic, strong) NSString* key;

@property (nonatomic, strong) id value;

@end


@implementation SPGameInfoCollectionCell

- (instancetype)initWithFrame:(CGRect)frame
{
	self	= [super initWithFrame:frame];
	
	if (self) {
		_keyLabel	= [[UILabel alloc] init];
		_keyLabel.translatesAutoresizingMaskIntoConstraints	= NO;
		_keyLabel.textAlignment	= NSTextAlignmentCenter;
		_keyLabel.font	= [UIFont boldSystemFontOfSize:12];
		_keyLabel.backgroundColor	= [UIColor blackColor];
		_keyLabel.textColor			= [UIColor whiteColor];
		[self.contentView addSubview:_keyLabel];
		
		_valueLabel	= [[UILabel alloc] init];
		_valueLabel.translatesAutoresizingMaskIntoConstraints	= NO;
		_valueLabel.textAlignment	= NSTextAlignmentCenter;
		_valueLabel.font	= [UIFont systemFontOfSize:10];
		_valueLabel.backgroundColor	= [UIColor whiteColor];
		_valueLabel.textColor			= [UIColor blackColor];
		_valueLabel.numberOfLines		= 0;
		_valueLabel.lineBreakMode		= NSLineBreakByWordWrapping;
		_valueLabel.adjustsFontSizeToFitWidth	= YES;
		_valueLabel.minimumScaleFactor			= .8f;
		[self.contentView addSubview:_valueLabel];
		
		self.contentView.layer.borderColor	= [UIColor lightGrayColor].CGColor;
		self.contentView.layer.borderWidth	= 1.0f;
		self.contentView.layer.cornerRadius	= 5.0f;
	}
	
	return self;
}

- (void)layoutSubviews
{
	if (_constraints) {
		[self.contentView removeConstraints:_constraints];
		[_constraints removeAllObjects];
	}
	
	_constraints	= [@[] mutableCopy];
	
	CGFloat height	= CGRectGetHeight(self.contentView.frame);
	
	NSDictionary* views	= NSDictionaryOfVariableBindings(_keyLabel, _valueLabel);
	NSDictionary* metrics	= @{@"keyH" : @(height * 0.3)};
	NSArray* constraints	= @[@"H:|-[_keyLabel]-|",
								@"H:|-[_valueLabel]-|",
								@"V:|-[_keyLabel(==keyH)]-[_valueLabel]-|"];
	
	for (NSString* str in constraints) {
		[_constraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
//		[self.contentView addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	[self.contentView addConstraints:_constraints];
	
//	if (SYSTEM_VERSION_LESS_THAN(@"8.0")) {
		[super layoutSubviews];
//	}
}

- (void)setKey:(NSString *)key
{
	_key = key;
	
	_keyLabel.text	= key;
}

- (void)setValue:(id)value
{
	_value	= value;
	
	if ([value isKindOfClass:[NSString class]])
		_valueLabel.text	= (NSString *)value;
	else
		_valueLabel.text	= @"";
}

@end

@interface SPGameInfoViewController ()
<UICollectionViewDataSource, UICollectionViewDelegate, UICollectionViewDelegateFlowLayout>
{
	UICollectionView* _collectionView;
	SPButton* _cancelButton;
	SPButton* _useButton;
	
	NSMutableArray* _essensialKeys;
}
@end

@implementation SPGameInfoViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
	
	if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"8.0")) {
		self.modalPresentationStyle	= UIModalPresentationOverCurrentContext;
	}
	else {
		self.modalPresentationStyle	= UIModalPresentationCurrentContext;
	}
	
	self.view.backgroundColor	= [UIColor colorWithWhite:0.1 alpha:0.6];
	
	UICollectionViewFlowLayout* layout = [[UICollectionViewFlowLayout alloc] init];
	layout.sectionInset					= UIEdgeInsetsMake(2, 2, 2, 2);
	
	_collectionView	= [[UICollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:layout];
	_collectionView.dataSource	= self;
	_collectionView.delegate	= self;
	_collectionView.backgroundColor	= [UIColor whiteColor];
	_collectionView.translatesAutoresizingMaskIntoConstraints	= NO;
	
	[_collectionView registerClass:[SPGameInfoCollectionCell class] forCellWithReuseIdentifier:CollectionCellIdentifier];
	
	[self.view addSubview:_collectionView];
	
	_cancelButton	= [SPButton buttonWithTitle:@"Cancel" handler:^(SPButton *button) {
		[self dismissViewControllerAnimated:NO completion:nil];
	}];
	_cancelButton.translatesAutoresizingMaskIntoConstraints	= NO;
	[self.view addSubview:_cancelButton];
	
	_useButton		= [SPButton buttonWithTitle:@"Use" handler:^(SPButton *button) {
		[self dismissViewControllerAnimated:NO completion:^{
            if (self->_delegate && [self->_delegate respondsToSelector:@selector(didSelectGameTitle:AppId:)]) {
                [self->_delegate didSelectGameTitle:self->_resultDict[@"game_name"] AppId:_resultDict[@"appid"]];
			}
		}];
	}];
	_useButton.translatesAutoresizingMaskIntoConstraints	= NO;
	[self.view addSubview:_useButton];
	
	NSDictionary* views;
	NSArray* constraints;
	NSDictionary* metrics	= @{@"Width" : @(240), @"Height" : @(240), @"buttonH" : @(44)};
	
	views	= NSDictionaryOfVariableBindings(_collectionView, _cancelButton, _useButton);
	constraints	= @[@"H:[_collectionView(==Width)]",
					@"V:[_collectionView(==Height)]",
					@"H:|-[_cancelButton]-[_useButton(==_cancelButton)]-|",
					@"V:[_collectionView]-[_cancelButton(==buttonH)]",
					@"V:[_collectionView]-[_useButton(==_cancelButton)]"
					];
	
//	if ([self respondsToSelector:@selector(topLayoutGuide)]) {
//		id topLayoutGuide		= [self topLayoutGuide];
//		id bottomLayoutGuide	= [self bottomLayoutGuide];
//		views	= NSDictionaryOfVariableBindings(topLayoutGuide, _collectionView, bottomLayoutGuide);
//		constraints	= @[@"H:|-[_collectionView]-|", @"V:|[topLayoutGuide]-[_collectionView]-[bottomLayoutGuide]|"];
//	}
//	else {
//		views	= NSDictionaryOfVariableBindings(_collectionView);
//		constraints	= @[@"H:|-[_collectionView]-|", @"V:|-[_collectionView]-|"];
//	}
	
	for (NSString* str in constraints) {
		[self.view addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	[self.view addConstraints:@[[NSLayoutConstraint constraintWithItem:_collectionView attribute:NSLayoutAttributeCenterX relatedBy:NSLayoutRelationEqual
																toItem:self.view attribute:NSLayoutAttributeCenterX multiplier:1.0 constant:0],
								[NSLayoutConstraint constraintWithItem:_collectionView attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual
																toItem:self.view attribute:NSLayoutAttributeCenterY multiplier:1.0 constant:0],
								]];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)setDictionary:(NSDictionary *)dictionary
{
	_dictionary	= dictionary;
	
	_resultDict	= dictionary[@"result"];
	
	_essensialKeys	= [@[@"appid", @"game_name"] mutableCopy];
	
	for (NSString* k in [_resultDict allKeys]) {
		if ([_essensialKeys containsObject:k])
			continue;
		[_essensialKeys addObject:k];
	}
}

#pragma mark -
#pragma mark
- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section
{
	return [_essensialKeys count];
}

// The cell that is returned must be retrieved from a call to -dequeueReusableCellWithReuseIdentifier:forIndexPath:
- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
	SPGameInfoCollectionCell* cell;
	
	cell	= [collectionView dequeueReusableCellWithReuseIdentifier:CollectionCellIdentifier forIndexPath:indexPath];
	
//	[collectionView.collectionViewLayout invalidateLayout];
	
	NSString* key	= _essensialKeys[indexPath.row];
	
	cell.key	= key;
	cell.value	= _resultDict[key];
	
	return cell;
}

#pragma mark
- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath
{
	if (indexPath.row == 0 || indexPath.row == 1) {
		return CGSizeMake(200, 100);
	}
	
	return CGSizeMake(200, 100);
}

@end
