//
//  SPSegmentedController.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 11..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "SPSegmentedController.h"

@implementation SPSegmentedController

- (instancetype)init
{
	return [self initWithItems:nil];
}

- (instancetype)initWithItems:(NSArray *)items
{
	self	= [super init];
	
	if (self) {
		[self setItems:items];
		_segmentedSelectedIndex	= -1;
	}
	
	return self;
}

- (void)layoutSubviews
{
	[self removeConstraints:self.constraints];
	
	NSInteger numOfButton	= [_items count];
	
	if (numOfButton == 0) {
		
	}
	else if (numOfButton == 1) {
		
	}
	else {
		for (SPButton* button in _buttons) {
			NSInteger index = [_buttons indexOfObject:button];
			
			button.backgroundColor	= (index == _segmentedSelectedIndex) ? SP_BUTTON_COLOR_TYPE03 : SP_BUTTON_COLOR_TYPE01;
			
			id leftLayoutGuide;
			NSDictionary* views;
			NSDictionary* metrics	= @{@"padding" : @(4)};
			NSMutableArray* constraints	= [@[@"V:|-padding-[button]-padding-|"] mutableCopy];
			
			if ([_buttons firstObject] == button) {
				views		= NSDictionaryOfVariableBindings(button);
				[constraints addObject:@"H:|-[button]"];
			}
			else {
				leftLayoutGuide	= [_buttons objectAtIndex:index - 1];
				
				views		= NSDictionaryOfVariableBindings(leftLayoutGuide, button);
				
				NSString* str	= @"H:[leftLayoutGuide]-[button(==leftLayoutGuide)]";
				
				if ([_buttons lastObject] == button) {
					str	= [str stringByAppendingString:@"-|"];
				}
				
				[constraints addObject:str];
			}
			
			for (NSString* str in constraints) {
				[self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
			}
		}
	}
	
	if (SYSTEM_VERSION_LESS_THAN(@"8.0")) {
		[super layoutSubviews];
	}
}

- (void)setItems:(NSArray *)items
{
	_items	= items;
	
	NSMutableArray* buttons	= [@[] mutableCopy];
	
	for (NSString* item in items) {
		SPButton* btn	= [SPButton buttonWithTitle:item handler:^(SPButton *button) {
			[self _onButton:button];
		}];
		btn.translatesAutoresizingMaskIntoConstraints	= NO;
		[self addSubview:btn];
		
		[buttons addObject:btn];
	}
	
	_buttons	= buttons;
}

- (void)setSegmentedSelectedIndex:(NSInteger)segmentedSelectedIndex
{
	SPButton* btn = [_buttons objectAtIndex:segmentedSelectedIndex];
	
	if (!btn)
		return;
	
	[self _onButton:btn];
}

- (void)_onButton:(id)sender
{
	SPButton* btn	= (SPButton *)sender;
	
	NSInteger index	= [_buttons indexOfObject:btn];
	
	if (index == NSNotFound)
		return;
	
	if (index != _segmentedSelectedIndex) {
		_segmentedSelectedIndex	= index;
		
		if (_delegate && [_delegate respondsToSelector:@selector(didChangeSegmentedValue:selectedIndex:)]) {
			[_delegate didChangeSegmentedValue:self selectedIndex:index];
		}
		
		[self setNeedsLayout];
	}
}

@end
