//
//  SPSettings.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 9..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "SPSettings.h"
#import "C2SActiveUser.h"

#import <HIVECore/HIVECore-Swift.h>
#import <ProviderAdapter/ProviderAdapter-Swift.h>
#import <HIVEUI/HIVEUI-Swift.h>
#import <HIVEBase/HIVEBase-Swift.h>
#import <HIVEIAPV4/HIVEIAPV4-Swift.h>
#import <HIVEService/HIVEService-Swift.h>

static NSString* kSPSettingsInfo	= @"__sp_settings_info";
static NSString* kSPSettingsProtocol	= @"__sp_selected_protocol";
static NSString* kSPSettingsServer		= @"__sp_selected_server";
static NSString* kSPSettingsDomain		= @"__sp_selected_domain";
static NSString* kSPSettingsCountry		= @"__sp_selected_country";
static NSString* kSPSettingsLanguage	= @"__sp_selected_language";
static NSString* kSPSettingsGameIndex	= @"__sp_selected_gameindex";
static NSString* kSPSettingsAppId		= @"__sp_selected_appid";
static NSString* kSPSettingsGameTitle	= @"__sp_selected_gametitle";
static NSString* kSPSettingsInquiryLanguage	= @"__sp_selected_inquirylanguage";

@implementation SPSettings

+ (NSDictionary *)defaultSPSettings
{
	return @{kSPSettingsProtocol	: [[self getAllProtocols] firstObject],
			 kSPSettingsServer		: [[self getAllServers] firstObject],
			 kSPSettingsDomain		: [[self getAllDomains] firstObject],
			 kSPSettingsCountry		: [self convertToCountryCode:[[self getAllCountries] firstObject]],
			 kSPSettingsInquiryLanguage : [[self getAllInquiryLanguage] firstObject][@"languageCode"],
			 kSPSettingsLanguage	: [self convertToLanguageCode:[[self getAllLanguages] firstObject]],
			 kSPSettingsAppId		: @"com.com2us.hivesdk.normal.freefull.apple.global.ios.universal",
			 kSPSettingsGameTitle	: @"통합모듈 샘플앱"
			 };
}

//
//	Protocol
//-----------------------------------
+ (NSArray *)getAllProtocols
{
	return @[@"http://", @"https://"];
}

+ (NSString *)loadSavedProtocol
{
	return [self getSPSettingValue:kSPSettingsProtocol];
}

+ (void)saveProtocol:(NSString *)protocol
{
	[self setSPSetting:kSPSettingsProtocol value:protocol];
}

//
//	Server
//-----------------------------------
+ (NSArray *)getAllServers
{
    return @[[self loadSavedServer]];
    
    //return @[@"release", @"test", @"staging", @"sandbox"];
}

+ (NSString *)loadSavedServer
{
    switch([HIVEConfiguration getZone]) {
        case HIVEZoneTypeDev :
        case HIVEZoneTypeTest : return @"test"; break;
        case HIVEZoneTypeSandbox : return @"sandbox"; break;
        case HIVEZoneTypeReal : return @"release"; break;
    }
    
	//return [self getSPSettingValue:kSPSettingsServer];
}

+ (void)saveServer:(NSString *)server
{
	//[self setSPSetting:kSPSettingsServer value:server];
}

//
//	Domain
//-----------------------------------
+ (NSArray *)getAllDomains
{
	return @[@"qpyou.cn", @"com2us.com"];
}

+ (NSString *)loadSavedDomain
{
	return [self getSPSettingValue:kSPSettingsDomain];
}

+ (void)saveDomain:(NSString *)domain
{
	[self setSPSetting:kSPSettingsDomain value:domain];
}

//
//	Country
//-----------------------------------
+ (NSArray *)getAllCountries
{
	return @[@"한국", @"일본", @"미국", @"중국", @"러시아", @"캐나다", @"대만", @"독일"];
}

+ (NSString *)convertToCountryCode:(NSString *)country
{
	if ([country isEqualToString:@"한국"])
		return @"KR";
	else if ([country isEqualToString:@"일본"])
		return @"JP";
	else if ([country isEqualToString:@"미국"])
		return @"US";
	else if ([country isEqualToString:@"중국"])
		return @"CN";
	else if ([country isEqualToString:@"러시아"])
		return @"RU";
	else if ([country isEqualToString:@"캐나다"])
		return @"CA";
	else if ([country isEqualToString:@"대만"])
		return @"TW";
	else if ([country isEqualToString:@"독일"])
		return @"DE";
	
	return nil;
}

+ (NSString *)convertToCountryName:(NSString *)countryCode
{
	if ([countryCode isEqualToString:@"KR"])
		return @"한국";
	else if ([countryCode isEqualToString:@"JP"])
		return @"일본";
	else if ([countryCode isEqualToString:@"US"])
		return @"미국";
	else if ([countryCode isEqualToString:@"CN"])
		return @"중국";
	else if ([countryCode isEqualToString:@"RU"])
		return @"러시아";
	else if ([countryCode isEqualToString:@"CA"])
		return @"캐나다";
	else if ([countryCode isEqualToString:@"TW"])
		return @"대만";
	else if ([countryCode isEqualToString:@"DE"])
		return @"독일";
	
	return nil;
}

+ (NSString *)loadSavedCountry
{
	NSString* countryCode	= [self getSPSettingValue:kSPSettingsCountry];
	
	[self updateCountryCodeCookie:countryCode];
	
	return [self convertToCountryName:countryCode];
}

+ (void)saveCountry:(NSString *)country
{
	NSString* countryCode	= [self convertToCountryCode:country];
	
	if (!countryCode)
		return;
	
	[self setSPSetting:kSPSettingsCountry value:countryCode];
	
	[self updateCountryCodeCookie:countryCode];
}

+ (void)updateCountryCodeCookie:(NSString *)countryCode
{
	NSHTTPCookieStorage* cookieStorage	= [NSHTTPCookieStorage sharedHTTPCookieStorage];
	NSMutableArray* cookies	= [[cookieStorage cookies] mutableCopy];
	
	[cookies addObject:[NSHTTPCookie cookieWithProperties:@{NSHTTPCookieName: @"hive_config_nationality",
															NSHTTPCookieValue: countryCode,
															NSHTTPCookieOriginURL: @"https://.qpyou.cn/",
															NSHTTPCookiePath: @"/"}]];
	
	[cookies addObject:[NSHTTPCookie cookieWithProperties:@{NSHTTPCookieName: @"hive_config_nationality",
															NSHTTPCookieValue: countryCode,
															NSHTTPCookieOriginURL: @"https://.com2us.com/",
															NSHTTPCookiePath: @"/"}]];
	
	[cookieStorage setCookies:cookies forURL:[NSURL URLWithString:@"https://.qpyou.cn/"] mainDocumentURL:nil];
	[cookieStorage setCookies:cookies forURL:[NSURL URLWithString:@"https://.com2us.com/"] mainDocumentURL:nil];
}

//
//	Language
//-----------------------------------
+ (NSArray *)getAllLanguages
{
	return @[@"한국어", @"영어", @"일본어", @"중국어-간체", @"중국어-번체", @"독일어", @"러시아어"];
}

+ (NSString *)convertToLanguage:(NSString *)languageCode
{
	if ([languageCode isEqualToString:@"ko"])
	return @"한국어";
	else if ([languageCode isEqualToString:@"en"])
		return @"영어";
	else if ([languageCode isEqualToString:@"ja"])
		return @"일본어";
	else if ([languageCode isEqualToString:@"zh"])
		return @"중국어-간체";
	else if ([languageCode isEqualToString:@"zh"])
		return @"중국어-번체";
	else if ([languageCode isEqualToString:@"de"])
		return @"독일어";
	else if ([languageCode isEqualToString:@"ru"])
		return @"러시아어";
	
	return nil;
}

+ (NSString *)convertToLanguageCode:(NSString *)language
{
	if ([language isEqualToString:@"한국어"])
		return @"ko";
	else if ([language isEqualToString:@"영어"])
		return @"en";
	else if ([language isEqualToString:@"일본어"])
		return @"ja";
	else if ([language isEqualToString:@"중국어-간체"])
		return @"zh";
	else if ([language isEqualToString:@"중국어-번체"])
		return @"zh";
	else if ([language isEqualToString:@"독일어"])
		return @"de";
	else if ([language isEqualToString:@"러시아어"])
		return @"ru";
	
	return nil;
}

+ (NSString *)loadSavedLanguage
{
	NSString* language	= [self getSPSettingValue:kSPSettingsLanguage];
	
	[self updateLanguageCookie:language];
	
	return [self convertToLanguage:language];
}

+ (void)saveLanguage:(NSString *)language
{
	NSString* languageCode		= [self convertToLanguageCode:language];
	
	if (!languageCode)
		return;
	
	[self setSPSetting:kSPSettingsLanguage value:languageCode];
	
	[self updateLanguageCookie:languageCode];
    CS_ActiveUserSetGameLanguage(languageCode.UTF8String);

}

+ (void)updateLanguageCookie:(NSString *)languageCode
{
	NSHTTPCookieStorage* cookieStorage	= [NSHTTPCookieStorage sharedHTTPCookieStorage];
	NSMutableArray* cookies	= [[cookieStorage cookies] mutableCopy];
	
	[cookies addObject:[NSHTTPCookie cookieWithProperties:@{NSHTTPCookieName: @"hive_config_language",
															NSHTTPCookieValue: languageCode,
															NSHTTPCookieOriginURL: @"https://.qpyou.cn/",
															NSHTTPCookiePath: @"/"}]];
	
	[cookies addObject:[NSHTTPCookie cookieWithProperties:@{NSHTTPCookieName: @"hive_config_language",
															NSHTTPCookieValue: languageCode,
															NSHTTPCookieOriginURL: @"https://.com2us.com/",
															NSHTTPCookiePath: @"/"}]];
	
	[cookieStorage setCookies:cookies forURL:[NSURL URLWithString:@"https://.qpyou.cn/"] mainDocumentURL:nil];
	[cookieStorage setCookies:cookies forURL:[NSURL URLWithString:@"https://.com2us.com/"] mainDocumentURL:nil];
}

#pragma mark 
+ (NSString *)loadSavedGameIndex
{
	NSString* gameIndex	= [self getSPSettingValue:kSPSettingsGameIndex];
	
	return gameIndex;
}

+ (void)saveGameIndex:(NSString *)gameIndex
{
	[self setSPSetting:kSPSettingsGameIndex value:gameIndex];
}

#pragma mark
+ (NSString *)loadSavedAppId
{
	NSString* appid	= [self getSPSettingValue:kSPSettingsAppId];
	
	return appid;
}

+ (void)saveAppId:(NSString *)appid
{
	[self setSPSetting:kSPSettingsAppId value:appid];
}

#pragma mark
+ (NSString *)loadSavedGameTitle
{
	NSString* gameTitle	= [self getSPSettingValue:kSPSettingsGameTitle];
	
	if (gameTitle == nil || [gameTitle isKindOfClass:[NSNull class]] || [gameTitle length] == 0)
		gameTitle = TITLE_NO_GAME_TITLE;
	
	return gameTitle;
}

+ (void)saveGameTitle:(NSString *)gameTitle
{
	if (gameTitle == nil || [gameTitle isKindOfClass:[NSNull class]])
		gameTitle	= @"";
	else if ([gameTitle isEqualToString:TITLE_NO_GAME_TITLE])
		gameTitle	= @"";
	
	[self setSPSetting:kSPSettingsGameTitle value:gameTitle];
}

#pragma mark
+ (NSArray *)getAllInquiryLanguage
{
	return @[@{@"languageName" : @"한국어", @"languageCode" : @"ko_KR"},
			 @{@"languageName" : @"영어", @"languageCode" : @"en_US"},
			 @{@"languageName" : @"일본어", @"languageCode" : @"ja_JP"},
			 @{@"languageName" : @"중국어 간체", @"languageCode" : @"zh_CN"},
			 @{@"languageName" : @"중국어 번체", @"languageCode" : @"zh_TW"},
			 @{@"languageName" : @"독일어", @"languageCode" : @"de_DE"},
			 @{@"languageName" : @"러시아어", @"languageCode" : @"ru_RU"},
			 @{@"languageName" : @"불어", @"languageCode" : @"fr_FR"},
			 @{@"languageName" : @"스페인어", @"languageCode" : @"es_ES"},
			 @{@"languageName" : @"포르투칼어", @"languageCode" : @"pt_PT"},
			 @{@"languageName" : @"인도네시아어", @"languageCode" : @"id_ID"},
			 @{@"languageName" : @"말레이시아어", @"languageCode" : @"ms_MY"},
			 @{@"languageName" : @"태국어", @"languageCode" : @"th_TH"},
			 @{@"languageName" : @"베트남어", @"languageCode" : @"vi_VN"},
			 @{@"languageName" : @"이탈리아어", @"languageCode" : @"it_IT"},
			 @{@"languageName" : @"터키어", @"languageCode" : @"tr_TR"},
			 ];
}

+ (NSString *)loadSavedInquiryLanguage
{
	NSString* inquiryCode	= [self getSPSettingValue:kSPSettingsInquiryLanguage];
	
	if ([inquiryCode length] == 0)
		inquiryCode	= [[self getAllInquiryLanguage] firstObject][@"languageCode"];
	
	NSString* languageName	= [[self getAllInquiryLanguage] firstObject][@"languageName"];
	for (NSDictionary* dict in [self getAllInquiryLanguage]) {
		if (![dict[@"languageCode"] isEqualToString:inquiryCode])
			continue;
		languageName	= dict[@"languageName"];
	}
	
	[self updateInquiryLanguageCookie:inquiryCode];
	
	return languageName;
}

+ (void)saveInquiryLanguage:(NSString *)languageName
{
	NSString* languageCode	= [[self getAllInquiryLanguage] firstObject][@"languageCode"];
	for (NSDictionary* dict in [self getAllInquiryLanguage]) {
		if (![dict[@"languageName"] isEqualToString:languageName])
			continue;
		
		languageCode	= dict[@"languageCode"];
	}
	
	[self updateInquiryLanguageCookie:languageCode];
	
	[self setSPSetting:kSPSettingsInquiryLanguage value:languageCode];
}

+ (void)updateInquiryLanguageCookie:(NSString *)languageCode
{
	NSHTTPCookieStorage* cookieStorage	= [NSHTTPCookieStorage sharedHTTPCookieStorage];
	NSMutableArray* cookies	= [[cookieStorage cookies] mutableCopy];
	
	[cookies addObject:[NSHTTPCookie cookieWithProperties:@{NSHTTPCookieName: @"inquiry_language",
															NSHTTPCookieValue: languageCode,
															NSHTTPCookieOriginURL: @"https://.qpyou.cn/",
															NSHTTPCookiePath: @"/"}]];
	
	[cookies addObject:[NSHTTPCookie cookieWithProperties:@{NSHTTPCookieName: @"inquiry_language",
															NSHTTPCookieValue: languageCode,
															NSHTTPCookieOriginURL: @"https://.com2us.com/",
															NSHTTPCookiePath: @"/"}]];
	
	[cookieStorage setCookies:cookies forURL:[NSURL URLWithString:@"https://.qpyou.cn/"] mainDocumentURL:nil];
	[cookieStorage setCookies:cookies forURL:[NSURL URLWithString:@"https://.com2us.com/"] mainDocumentURL:nil];
}

//
//	Get / Set
//-----------------------------------
#pragma mark
+ (NSString *)getSPSettingValue:(NSString *)key
{
	NSUserDefaults* ud	= [NSUserDefaults standardUserDefaults];
	
	NSDictionary* info	= [ud objectForKey:kSPSettingsInfo];
	
	if (!info) {
		info	= [self defaultSPSettings];
		[ud setObject:info forKey:kSPSettingsInfo];
		[ud synchronize];
	}
	
	return info[key];
}

+ (void)setSPSetting:(NSString *)key value:(NSString *)value
{
	NSUserDefaults* ud	= [NSUserDefaults standardUserDefaults];
	
	NSMutableDictionary* info	= [[ud objectForKey:kSPSettingsInfo] mutableCopy];
	
	if (!info) {
		info	= [[self defaultSPSettings] mutableCopy];
	}
	
	info[key] = value;
	
	[ud setObject:(NSDictionary *)info forKey:kSPSettingsInfo];
	[ud synchronize];
}

+ (NSString *)getSPSettingFullPath
{
	NSString* path;
	if ([[self loadSavedServer] isEqualToString:@"test"] && [[self loadSavedDomain] isEqualToString:@"com2us.com"]) {
		path	= [NSString stringWithFormat:@"%@.api.%@", [self loadSavedServer], [self loadSavedDomain]];
	}
	else if ([[self loadSavedServer] isEqualToString:@"release"]) {
		path	= [NSString stringWithFormat:@"api.%@", [self loadSavedDomain]];
	}
	else {
		path = [NSString stringWithFormat:@"%@-api.%@", [self loadSavedServer], [self loadSavedDomain]];
	}
	
	return [NSString stringWithFormat:@"%@%@", [self loadSavedProtocol], path];
}


@end
