//
//  SPLogViewController.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 14..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "SPSplitViewController.h"
#import "SPLogViewController.h"
#import "MainNavigationController.h"

@interface SPSplitViewController ()
<SPLogViewDelegate>
{
	NSMutableArray* _allConstraints;
	
	NSMutableArray* _logConstraints;
	
	SPLogViewController* _logViewController;
}
@end

@implementation SPSplitViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
	
	_contentView	= [[UIView alloc] init];
	_contentView.translatesAutoresizingMaskIntoConstraints	= NO;
	[self.view addSubview:_contentView];
	
	_logView		= [[UIView alloc] init];
	_logView.translatesAutoresizingMaskIntoConstraints	= NO;
	_logView.layer.borderColor		= SP_BUTTON_COLOR_TYPE01.CGColor;
	_logView.layer.borderWidth		= 2.0f;
	[self.view addSubview:_logView];
	
	_logViewController	= [[SPLogViewController alloc] initWithStyle:UITableViewStylePlain];
	_logViewController.view.translatesAutoresizingMaskIntoConstraints	= NO;
	_logViewController.delegate	= self;
	[_logView addSubview:_logViewController.view];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)orientationChanged
{
	[self.view setNeedsUpdateConstraints];
}

- (void)viewDidAppear:(BOOL)animated
{
	[super viewDidAppear:animated];
	
	[_logViewController viewDidAppear];
}

- (void)updateViewConstraints
{
	[super updateViewConstraints];
	
	if (_allConstraints && [_allConstraints count] > 0) {
		[self.view removeConstraints:_allConstraints];
		[_allConstraints removeAllObjects];
	}
	_allConstraints	= [@[] mutableCopy];
	
	if (_logConstraints && [_logConstraints count] > 0) {
		[_logView removeConstraints:_logConstraints];
		[_logConstraints removeAllObjects];
	}
	_logConstraints	= [@[] mutableCopy];
	
	NSDictionary* views;
	NSDictionary* metrics	= @{@"logViewH" : @(CGRectGetHeight(self.view.frame) * 0.3),
								@"logViewW" : @(CGRectGetWidth(self.view.frame) * 0.4)};
	NSArray* constraints;
	
	if (SYSTEM_VERSION_GREATER_THAN_OR_EQUAL_TO(@"7.0")) {
		id topLayoutGuide		= [self topLayoutGuide];
		id bottomLayoutGuide	= [self bottomLayoutGuide];
		
		views	= NSDictionaryOfVariableBindings(topLayoutGuide, _contentView, _logView, bottomLayoutGuide);
		if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
			constraints	= @[@"H:|[_contentView]|",
							@"H:|[_logView]|",
							@"V:|[topLayoutGuide][_contentView][_logView(==logViewH)][bottomLayoutGuide]|"];
		}
		else {
			constraints	= @[@"H:|[_contentView][_logView(==logViewW)]|",
							@"V:|[topLayoutGuide][_contentView][bottomLayoutGuide]|",
							 @"V:|[topLayoutGuide][_logView][bottomLayoutGuide]|"];
		}
	}
	else {
		views	= NSDictionaryOfVariableBindings(_contentView, _logView);
		if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
			constraints	= @[@"H:|[_contentView]|",
							@"H:|[_logView]|",
							@"V:|[_contentView][_logView(==logViewH)]|"];
		}
		else {
			constraints	= @[@"H:|[_contentView][_logView(==logViewW)]|",
							@"V:|[_contentView]|",
							@"V:|[_logView]|"];
		}
	}
	
	for (NSString* str in constraints) {
		[_allConstraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	[self.view addConstraints:_allConstraints];
	
	UIView* theLogView = _logViewController.view;
	for (NSString* str in @[@"H:|[theLogView]|", @"V:|[theLogView]|"]) {
		[_logConstraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:nil views:NSDictionaryOfVariableBindings(theLogView)]];
	}
	
	[_logView addConstraints:_logConstraints];
}

#pragma mark -
- (void)didSelectLog:(id)controller
{
	[self.navigationController presentViewController:[[MainNavigationController alloc] initWithRootViewController:controller] animated:YES completion:nil];
}

@end
