//
//  WithUserInfoViewController.m
//  HiveSampleV2
//
//  Created by LucidArkeo on 2015. 9. 9..
//  Copyright (c) 2015년 com2us. All rights reserved.
//

#import "WithUserInfoViewController.h"
#import "HiveSocialManager.h"
#import "LoginViewController.h"
#import "FriendsViewController.h"
#import "InviteViewController.h"
#import "ETCViewController.h"

#pragma mark - UserView
@implementation UserView

@end

#pragma mark - UserInfoView
@interface UserInfoView ()
{
	UIView* _borderView;
}
@end

@implementation UserInfoView

- (instancetype)init
{
	self	= [super init];
	
	if (self) {
		_profileImageView	= [[UIImageView alloc] init];
		_profileImageView.translatesAutoresizingMaskIntoConstraints	= NO;
		_profileImageView.layer.borderColor	= SP_LABEL_BORDER_COLOR.CGColor;
		_profileImageView.layer.borderWidth	= 2.0f;
		_profileImageView.contentMode	= UIViewContentModeScaleAspectFit;
		[self addSubview:_profileImageView];
		
		_borderView	= [[UIView alloc] init];
		_borderView.translatesAutoresizingMaskIntoConstraints	= NO;
		_borderView.backgroundColor	= [UIColor clearColor];
		_borderView.layer.borderWidth	= 2.0f;
		_borderView.layer.borderColor	= SP_LABEL_BORDER_COLOR.CGColor;
		[self addSubview:_borderView];
		
		_usernameLabel	= [[UILabel alloc] init];
		_usernameLabel.translatesAutoresizingMaskIntoConstraints	= NO;
		_usernameLabel.font	= [UIFont fontWithName:SP_DEFAULT_FONT_NAME size:10];
		[_borderView addSubview:_usernameLabel];
		
		_facebookButton		= [SPButton buttonWithImagePath:SP_IMAGE_PATH(@"facebook-32.png") handler:^(SPButton *button) {
			if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_facebookButton atUserView:self];
			}
		}];
		_facebookButton.translatesAutoresizingMaskIntoConstraints	= NO;
		[_borderView addSubview:_facebookButton];
		
		_googleplusButton	= [SPButton buttonWithImagePath:SP_IMAGE_PATH(@"google-plus-32.png") handler:^(SPButton *button) {
			if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_googleplusButton atUserView:self];
			}
		}];
		_googleplusButton.translatesAutoresizingMaskIntoConstraints	= NO;
		[_borderView addSubview:_googleplusButton];
	}
	
	return self;
}

- (void)layoutSubviews
{
	[self removeConstraints:self.constraints];
	
	CGFloat padding			= 2;
	NSDictionary* mainViews	= NSDictionaryOfVariableBindings(_profileImageView, _borderView);
	NSDictionary* mainMetrics	= @{@"padding" : @(padding), @"imageH" : @(CGRectGetHeight(self.frame) - (padding * 2))};
	NSArray* mainConstraints	= @[@"V:|[_profileImageView]|",
									@"V:|[_borderView]|",
									@"H:|[_profileImageView(==imageH)]-[_borderView]|",
									];
	
	for (NSString* str in mainConstraints) {
		[self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:mainMetrics views:mainViews]];
	}
	
	NSDictionary* views	= NSDictionaryOfVariableBindings(_usernameLabel, _facebookButton, _googleplusButton);
	NSDictionary* metrics	= @{@"halfH" : @(CGRectGetHeight(self.frame) / 2), @"padding" : @(4)};
	NSArray* constraints	= @[@"H:|-padding-[_usernameLabel]|",
								@"V:|[_usernameLabel(==halfH)]",
								@"H:[_facebookButton(==halfH)]-[_googleplusButton(==halfH)]-|",
								@"V:[_facebookButton(==halfH)]-padding-|",
								@"V:[_googleplusButton(==halfH)]-padding-|",
								];
	
	for (NSString* str in constraints) {
		[_borderView addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	if (SYSTEM_VERSION_LESS_THAN(@"8.0")) {
		[super layoutSubviews];
	}
}

//- (void)updateConstraints
//{
//	[super updateConstraints];
//
//	if (CGRectEqualToRect(self.frame, CGRectZero))
//		return;
//
//	CGFloat padding			= 2;
//	NSDictionary* mainViews	= NSDictionaryOfVariableBindings(_profileImageView, _borderView);
//	NSDictionary* mainMetrics	= @{@"padding" : @(padding), @"imageH" : @(CGRectGetHeight(self.frame) - (padding * 2))};
//	NSArray* mainConstraints	= @[@"V:|-padding-[_profileImageView]-padding-|",
//									@"V:|[_borderView]|",
//									@"H:|[_profileImageView(==imageH)]-[_borderView]|",
//									];
//
//	for (NSString* str in mainConstraints) {
//		[self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:mainMetrics views:mainViews]];
//	}
//
//	NSDictionary* views	= NSDictionaryOfVariableBindings(_usernameLabel, _facebookButton, _googleplusButton);
//	NSDictionary* metrics	= @{@"halfH" : @(CGRectGetHeight(self.frame) / 2), @"padding" : @(4)};
//	NSArray* constraints	= @[@"H:|-[_usernameLabel]-|",
//								@"V:|[_usernameLabel(==halfH)]",
//								@"H:[_facebookButton(==halfH)]-[_googleplusButton(==halfH)]-|",
//								@"V:[_facebookButton(==halfH)]-padding-|",
//								@"V:[_googleplusButton(==halfH)]-padding-|",
//								];
//
//	for (NSString* str in constraints) {
//		[_borderView addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
//	}
//}

@end


#pragma mark - UserActionView
@implementation UserActionView

- (instancetype)init
{
	self	= [super init];
	
	if (self) {
		_friendsButton	= [SPButton buttonWithTitle:@"친구" imagePath:SP_IMAGE_FRIENDS_ICON handler:^(SPButton *button) {
			if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_friendsButton atUserView:self];
			}
		}];
		_friendsButton.translatesAutoresizingMaskIntoConstraints	= NO;
		[self addSubview:_friendsButton];
		
		_inviteButton	= [SPButton buttonWithTitle:@"초대" imagePath:SP_IMAGE_INVITE_ICON handler:^(SPButton *button) {
			if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_inviteButton atUserView:self];
			}
		}];
		_inviteButton.translatesAutoresizingMaskIntoConstraints	= NO;
		[self addSubview:_inviteButton];
		
		_etcButton	= [SPButton buttonWithTitle:@"기타" imagePath:SP_IMAGE_ETC_ICON handler:^(SPButton *button) {
			if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_etcButton atUserView:self];
			}
		}];
		_etcButton.translatesAutoresizingMaskIntoConstraints	= NO;
		[self addSubview:_etcButton];
		
		_faqButton	= [SPButton buttonWithTitle:@"문의" imagePath:SP_IMAGE_FAQ_ICON handler:^(SPButton *button) {
			if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_faqButton atUserView:self];
			}
		}];
		_faqButton.translatesAutoresizingMaskIntoConstraints	= NO;
		[self addSubview:_faqButton];
		
		_logoutButton	= [SPButton buttonWithTitle:@"로그아웃" imagePath:SP_IMAGE_LOGOUT_ICON handler:^(SPButton *button) {
			if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_logoutButton atUserView:self];
			}
		}];
		_logoutButton.translatesAutoresizingMaskIntoConstraints	= NO;
		[self addSubview:_logoutButton];
        
        _csBotButton = [SPButton buttonWithTitle:@"CS Bot" imagePath:SP_IMAGE_FAQ_ICON handler:^(SPButton *button) {
            if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_csBotButton atUserView:self];
            }
        }];
        _csBotButton.translatesAutoresizingMaskIntoConstraints = NO;
        [self addSubview:_csBotButton];
        
        _userDeleteButton    = [SPButton buttonWithTitle:@"계정 삭제" imagePath:SP_IMAGE_LOGOUT_ICON handler:^(SPButton *button) {
            if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_userDeleteButton atUserView:self];
            }
        }];
        _userDeleteButton.translatesAutoresizingMaskIntoConstraints    = NO;
        [self addSubview:_userDeleteButton];
	}
	
	return self;
}

- (void)layoutSubviews
{
	[self removeConstraints:self.constraints];
	
	NSDictionary* views	= NSDictionaryOfVariableBindings(_friendsButton, _inviteButton, _etcButton, _faqButton, _logoutButton, _csBotButton, _userDeleteButton);
	NSDictionary* metrics	= @{@"buttonH" : @(64), @"gap" : @(4)};
	
	NSArray* constraints;
	
//	if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
		constraints	= @[@"V:|[_friendsButton(==buttonH)]-|",
						@"V:|[_inviteButton(==buttonH)]-|",
						@"V:|[_etcButton(==buttonH)]-|",
						@"V:|[_faqButton(==buttonH)]-|",
						@"V:|[_logoutButton(==buttonH)]-|",
                        @"V:|[_csBotButton(==buttonH)]-|",
						@"H:|-gap-[_friendsButton]-gap-[_inviteButton(==_friendsButton)]-gap-[_etcButton(==_friendsButton)]-gap-[_faqButton(==_friendsButton)]-gap-[_logoutButton(==_friendsButton)]-gap-[_csBotButton(==_friendsButton)]-gap-[_userDeleteButton(==_friendsButton)]-gap-|"];
//	}
//	else {
//		constraints	= @[@"H:|[_friendsButton]-[_inviteButton(==_friendsButton)]|",
//						@"H:|[_etcButton(==_friendsButton)]-[_faqButton(==_friendsButton)]|",
//						@"H:|[_logoutButton(==_friendsButton)]",
//						@"V:|[_friendsButton]-[_etcButton(==_friendsButton)]-[_logoutButton(==_friendsButton)]|",
//						@"V:|[_inviteButton(==_friendsButton)]-[_faqButton(==_friendsButton)]"];
//	}
	
	for (NSString* str in constraints) {
		[self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	if (SYSTEM_VERSION_LESS_THAN(@"8.0")) {
		[super layoutSubviews];
	}
}

//- (void)updateConstraints
//{
//	[super updateConstraints];
//
//	NSDictionary* views	= NSDictionaryOfVariableBindings(_friendsButton, _inviteButton, _etcButton, _faqButton, _logoutButton);
//	NSDictionary* metrics	= @{@"buttonH" : @(44)};
//
//	NSArray* constraints;
//
//	constraints	= @[@"V:[_friendsButton(==buttonH)]-|",
//					@"V:[_inviteButton(==buttonH)]-|",
//					@"V:[_etcButton(==buttonH)]-|",
//					@"V:[_faqButton(==buttonH)]-|",
//					@"V:[_logoutButton(==buttonH)]-|",
//					@"H:|-[_friendsButton]-[_inviteButton(==_friendsButton)]-[_etcButton(==_friendsButton)]-[_faqButton(==_friendsButton)]-[_logoutButton(==_friendsButton)]-|"];
//
//	for (NSString* str in constraints) {
//		[self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
//	}
//}

@end

#pragma mark - UserGuestActionView
@implementation UserGuestActionView

- (instancetype)init
{
	self	= [super init];
	
	if (self) {
		_convertAccountButton	= [SPButton buttonWithTitle:@"정식 전환" imagePath:SP_IMAGE_CHANGE_USER_ICON handler:^(SPButton *button) {
			if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_convertAccountButton atUserView:self];
			}
		}];
		_convertAccountButton.translatesAutoresizingMaskIntoConstraints	= NO;
		[self addSubview:_convertAccountButton];
		
		_faqButton	= [SPButton buttonWithTitle:@"문의" imagePath:SP_IMAGE_FAQ_ICON handler:^(SPButton *button) {
			if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_faqButton atUserView:self];
			}
		}];
		_faqButton.translatesAutoresizingMaskIntoConstraints	= NO;
		[self addSubview:_faqButton];
		
		_loginButton	= [SPButton buttonWithTitle:@"로그인" imagePath:SP_IMAGE_LOGIN_ICON handler:^(SPButton *button) {
			if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_loginButton atUserView:self];
			}
		}];
		_loginButton.translatesAutoresizingMaskIntoConstraints	= NO;
		[self addSubview:_loginButton];
        
        _csBotButton = [SPButton buttonWithTitle:@"CS Bot" imagePath:SP_IMAGE_FAQ_ICON handler:^(SPButton *button) {
            if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_csBotButton atUserView:self];
            }
        }];
        _csBotButton.translatesAutoresizingMaskIntoConstraints = NO;
        [self addSubview:_csBotButton];
        
        _userDeleteButton    = [SPButton buttonWithTitle:@"계정 삭제" imagePath:SP_IMAGE_LOGOUT_ICON handler:^(SPButton *button) {
            if (self.delegate && [self.delegate respondsToSelector:@selector(didSelectButton:atUserView:)]) {
                [self.delegate didSelectButton:self->_userDeleteButton atUserView:self];
            }
        }];
        _userDeleteButton.translatesAutoresizingMaskIntoConstraints    = NO;
        [self addSubview:_userDeleteButton];
	}
	
	return self;
}

- (void)layoutSubviews
{
	[self removeConstraints:self.constraints];
	
	NSDictionary* views	= NSDictionaryOfVariableBindings(_convertAccountButton,  _faqButton, _loginButton, _csBotButton, _userDeleteButton);
	NSDictionary* metrics	= @{@"buttonH" : @(64), @"gap" : @(4),};
	
	NSArray* constraints;
	
	//	if (UIInterfaceOrientationIsPortrait([UIApplication sharedApplication].statusBarOrientation)) {
	constraints	= @[@"V:|[_convertAccountButton(==buttonH)]-|",
					@"V:|[_faqButton(==buttonH)]-|",
					@"V:|[_loginButton(==buttonH)]-|",
                    @"V:|[_csBotButton(==buttonH)]-|",
                    @"V:|[_userDeleteButton(==buttonH)]-|",
					@"H:|-gap-[_convertAccountButton]-gap-[_faqButton(==_convertAccountButton)]-gap-[_loginButton(==_convertAccountButton)]-gap-[_csBotButton(==_convertAccountButton)]-gap-[_userDeleteButton(==_convertAccountButton)]-gap-|",
					];
	//	}
	//	else {
	//		constraints	= @[@"H:|[_friendsButton]-[_inviteButton(==_friendsButton)]|",
	//						@"H:|[_etcButton(==_friendsButton)]-[_faqButton(==_friendsButton)]|",
	//						@"H:|[_logoutButton(==_friendsButton)]",
	//						@"V:|[_friendsButton]-[_etcButton(==_friendsButton)]-[_logoutButton(==_friendsButton)]|",
	//						@"V:|[_inviteButton(==_friendsButton)]-[_faqButton(==_friendsButton)]"];
	//	}
	
	for (NSString* str in constraints) {
		[self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	if (SYSTEM_VERSION_LESS_THAN(@"8.0")) {
		[super layoutSubviews];
	}
}

@end

#pragma mark - WithUserInfoViewController
@interface WithUserInfoViewController ()
<UserViewDelegate>
{
	NSMutableArray* _allConstraints;
}
@end

@implementation WithUserInfoViewController

- (instancetype)initWithType:(WithUserInfoViewType)type
{
	self	= [super init];
	
	if (self) {
		_type	= type;
	}
	
	return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
	_userInfoView	= [[UserInfoView alloc] init];
	_userInfoView.translatesAutoresizingMaskIntoConstraints	= NO;
	_userInfoView.delegate	= self;
	[self.contentView addSubview:_userInfoView];
	
	_userActionView	= [[UserActionView alloc] init];
	_userActionView.translatesAutoresizingMaskIntoConstraints	= NO;
	_userActionView.delegate	= self;
	[self.contentView addSubview:_userActionView];
	
	_userGuestActionView	= [[UserGuestActionView alloc] init];
	_userGuestActionView.translatesAutoresizingMaskIntoConstraints	= NO;
	_userGuestActionView.delegate	= self;
	[self.contentView addSubview:_userGuestActionView];
	
	[self orientationChanged];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

//- (void)orientationChanged
//{
//	[super orientationChanged];
//	
//	if (_allConstraints && [_allConstraints count] > 0) {
//		[self.view removeConstraints:_allConstraints];
//		[_allConstraints removeAllObjects];
//	}
//	
//	_allConstraints	= [@[] mutableCopy];
//	
//	id topLayoutGuide		= self.serverPathLabel;
//	id bottomLayoutGuide	= self.logView;
//	
//	NSDictionary* views;
//	NSDictionary* metrics;
//	NSArray* constraints;
//	
//	views	= NSDictionaryOfVariableBindings(topLayoutGuide, _userInfoView, _userActionView, bottomLayoutGuide);
//	metrics	= @{@"UserInfoViewH" : @(44)};
//	
//	if (UIInterfaceOrientationIsPortrait([[UIApplication sharedApplication] statusBarOrientation])) {
//		constraints	= @[@"H:|-[_userInfoView]-|",
//						@"V:[topLayoutGuide]-[_userInfoView(==UserInfoViewH)]-[_userActionView]-[bottomLayoutGuide]",
//						@"H:|-[_userActionView]-|",
//						];
//	}
//	else {
//		constraints	= @[@"H:|-[_userInfoView]-[_userActionView(==_userInfoView)]-|",
//						@"V:[topLayoutGuide]-[_userInfoView(==UserInfoViewH)]",
//						@"V:[topLayoutGuide]-[_userActionView]-[bottomLayoutGuide]"
//						];
//		
//	}
//	
//	for (NSString* str in constraints) {
//		[_allConstraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
//	}
//	
//	[self.view addConstraints:_allConstraints];
//	
//	[self.view layoutIfNeeded];
//}

- (void)orientationChanged
{
	[self.view setNeedsUpdateConstraints];
}

- (void)updateViewConstraints
{
	[super updateViewConstraints];
	
	if (_allConstraints && [_allConstraints count] > 0) {
		[self.contentView removeConstraints:_allConstraints];
		[_allConstraints removeAllObjects];
	}
	
	_allConstraints	= [@[] mutableCopy];
	
	id topLayoutGuide;
	topLayoutGuide		= self.serverPathLabel;
	
	NSDictionary* views;
	NSDictionary* metrics	= @{@"UserInfoViewH" : @(48)};
	NSArray* constraints;
	
	_userActionView.hidden	= YES;
	_userGuestActionView.hidden	= YES;
	
	if (_type == WithUserInfoViewTypeRegular) {
		_userActionView.hidden	= NO;
		views	= NSDictionaryOfVariableBindings(topLayoutGuide, _userInfoView, _userActionView);
		constraints	= @[@"H:|-[_userInfoView]-|",
						@"H:|-[_userActionView]-|",
						@"V:[topLayoutGuide]-2@500-[_userInfoView(==UserInfoViewH)]-2@500-[_userActionView]-0@500-|",
						];
	}
	else {
		_userGuestActionView.hidden	= NO;
		views	= NSDictionaryOfVariableBindings(topLayoutGuide, _userInfoView, _userGuestActionView);
		constraints	= @[@"H:|-[_userInfoView]-|",
						@"H:|-[_userGuestActionView]-|",
						@"V:[topLayoutGuide]-2@500-[_userInfoView(==UserInfoViewH)]-2@500-[_userGuestActionView]-0@500-|",
						];
	}

	for (NSString* str in constraints) {
		[_allConstraints addObjectsFromArray:[NSLayoutConstraint constraintsWithVisualFormat:str options:0 metrics:metrics views:views]];
	}
	
	[self.contentView addConstraints:_allConstraints];
}

#pragma mark - UserViewDelegate Methods
- (void)didSelectButton:(SPButton *)button atUserView:(UserView *)userView
{
	if (userView == _userInfoView) {
		
	}
	else if (userView == _userActionView) {
		if (_userActionView.friendsButton == button) {
			FriendsViewController* vc	= [[FriendsViewController alloc] init];
			[self.navigationController pushViewController:vc animated:YES];
		}
		else if (_userActionView.inviteButton == button) {
			InviteViewController* vc	= [[InviteViewController alloc] init];
			[self.navigationController pushViewController:vc animated:YES];
		}
		else if (_userActionView.etcButton == button) {
			ETCViewController* vc		= [[ETCViewController alloc] init];
			[self.navigationController pushViewController:vc animated:YES];
		}
		else if (_userActionView.faqButton == button) {
			[HiveSocialManager hiveShowDialog:@"inquiry" completion:^(id result, NSError *error) {
				
			}];
		}
		else if (_userActionView.logoutButton == button) {
			[HiveSocialManager hiveLogout:^(id result, NSError *error) {
				LoginViewController* vc	= [[LoginViewController alloc] init];
				self.navigationController.viewControllers	= @[vc];
			}];
		}
		else if (_userActionView.csBotButton == button) {
			[HiveSocialManager hiveCSBotRequestWithCompletion:^(id result, NSError *error) {
				
			}];
		}
        else if (_userActionView.userDeleteButton == button) {
            [HiveSocialManager hiveUserDelete:^(id result, NSError *error) {
                LoginViewController* vc    = [[LoginViewController alloc] init];
                self.navigationController.viewControllers    = @[vc];
            }];
        }
	}
}

@end
