//
//  NSC2SCommonData.h
//  C2SInApp
//
//  Created by Kim Inhong on 12. 2. 24..
//  Copyright (c) 2012년 Com2uS. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

#define COMMONMODULEVER				@"1.3.4"

typedef NS_ENUM(NSInteger, ModuleType) {
    ModuleTypeNone,
    ModuleTypeActiveUser,
    ModuleTypePush,
    ModuleTypeInApp,
    ModuleTypeMercury,
    ModuleTypeOfferwall,
    ModuleTypeHive,
    ModuleTypeHiveIAP,
    ModuleTypeHiveAdultAuthorization,
    ModuleTypeLoginCenter,
    ModuleTypeHiveUserAcquisition,
    ModuleTypeHiveUserEngagement,
    
    ModuleTypeActiveUser_WebView,
    ModuleTypePush_WebView,
    ModuleTypeInApp_WebView,
    ModuleTypeMercury_WebView,
    ModuleTypeOfferwall_WebView,
    ModuleTypeHive_WebView,
    ModuleTypeHiveIAP_WebView,
    ModuleTypeHiveAdultAuthorization_WebView,
    ModuleTypeLoginCenter_WebView,
    ModuleTypeHiveUserAcquisition_WebView,
    ModuleTypeHiveUserEngagement_WebView
};

@protocol C2SInterworkDelegate <NSObject>
@optional
/*
 @abstract When the url scheme of Com2us is loaded by server, this delegate will be called.
 @discuss
 @param
 */
- (void)C2SInterworkDidLoad:(NSURL*)url at:(ModuleType)from;
@end

typedef NS_ENUM(NSInteger, ServerState)  {
    None = -1,
    LiveServer = 0,
    StagingServer,
    SandboxServer
};


@protocol C2SCommonDataDelegate <NSObject>
@optional
/*!
 @abstract Called once when application was lunched.
 @discussion
 @param SeverState that user selected.
 */
- (void)serverStateDidSet:(ServerState)serverState;
@end



@interface C2SCommonData : NSObject

@property (weak, nonatomic) id<C2SInterworkDelegate> interworkDelegate;
@property (weak, nonatomic) id<C2SCommonDataDelegate> delegate;

+ (C2SCommonData*)sharedInstance;

+ (NSDictionary *)appendC2SCommonDictionary:(NSDictionary*) inputData;
+ (NSDictionary *)getC2SCommonDictionary;
+ (NSString *)getCurrentLanguage;
+ (NSString *)getAppID;
+ (NSString *)getMacAddress;
+ (NSString *)getCommonModuleVersion;
+ (NSString*)getMobileCountryCode;
+ (NSString*)getMobileNetworkCode;

+ (BOOL)processHiveInterwork:(NSURL*)url From:(ModuleType)from;
+ (BOOL)setServerState:(ServerState)serverState;
+ (ServerState)getServerState;

+ (BOOL)setAppID:(NSString*)customAppID;

// TimeOUT
+ (BOOL)setTimeOutInterval:(ModuleType)from result:(bool)result;
+ (NSTimeInterval)getTimeOutInterval:(ModuleType)from;
+ (int)checkTimeOutRetry:(ModuleType)from;
+ (NSString*)getErrorPage;

// 서버 전달 추가 식별자
+ (NSDictionary *)getAddServerDictionary;
+ (void)setHiveCountryAndTimeZone:(NSString*)hiveCountry timezone:(NSDictionary*)timezone;

// GameLanguage
+ (void)setGameLanguage:(NSString*)lan;
+ (NSString*)getGameLanguage;

// serverId
+ (void)setServerId:(NSString*)server;
+ (NSString*)getServerId;



@end

