//
//  UIViewController+ActionListView.m
//  HiveSDKSampleXcode
//
//  Created by BK on 2016. 3. 21..
//  Copyright © 2016년 BK. All rights reserved.
//

#import "HiveIAPShopView.h"

@interface HiveIAPShopView ()
{
    NSMutableArray* pidList;
    APIAction *actionP;
    Parameter* param;
    BOOL isInappV4;
}
@end

@implementation HiveIAPShopView

-(void)viewDidLoad
{
    [super viewDidLoad];
    _logLabel.text = [GET_LOG_CONTROLLER getResultLog];

    [self initDataForItemInfo];
    
    _viewTitle.title = @"HiveIAP Shop";
    
    _loginState.text = [GET_ACTION_CONTROLLER getLoginLabel];
    _serverState.text = [GET_ACTION_CONTROLLER getServerLabel];
    
    pidTableView.delegate = self;
    pidTableView.dataSource = self;
    
}

- (void)initDataForItemInfo {
    isInappV4 = false;
    NSString* pidKey = @"pid";
    NSString* typeString = @"type";
    if([GET_ACTION_CONTROLLER getCurrentAction].num == ACTION_NUM_HIVEIAP_INFO_SHOPINFO || [GET_ACTION_CONTROLLER getCurrentAction].num == ACTION_NUM_HIVEIAP_PURCHASE)
        pidList = [GET_ACTION_CONTROLLER getTempDic][@"products"];
    else if([GET_ACTION_CONTROLLER getCurrentAction].num == ACTION_NUM_INAPPV4_ITEMINFO || [GET_ACTION_CONTROLLER getCurrentAction].num == ACTION_NUM_INAPPV4_PURCHASE){
        pidList = [GET_ACTION_CONTROLLER getTempDic][@"productList"];
        isInappV4 = true;
        pidKey = @"marketPid";
        typeString = [GET_ACTION_CONTROLLER getTempDic][@"type"];
    }
    
    // pid 파라미터 설정해주기
    NSMutableArray* pidArray = [[NSMutableArray alloc]init];
    for(NSDictionary* pidD in pidList)
        [pidArray addObject:pidD[pidKey]];
    
    NSArray* actionList = [GET_ACTION_CONTROLLER getCurrentGroup][KEY_ACTIONLIST];
    for (APIAction* action in actionList) {
        if(action.num == ACTION_NUM_HIVEIAP_PURCHASE || action.num == ACTION_NUM_INAPPV4_PURCHASE){
            actionP = action;
            for(Parameter* para in action.paramList){
                if([para.name isEqualToString:PARAM_INAPPV4_PID]){
                    param = para;
                    param.value = pidArray[0];
                    param.selectList = pidArray;
                }else if([para.name isEqualToString:PARAM_VALUE_TYPE]){
                    para.value = typeString;
                }
            }
        }else if(action.num == ACTION_NUM_INAPPV4_RESTORE){
            for(Parameter* para in action.paramList){
                if([para.name isEqualToString:PARAM_VALUE_TYPE]){
                    para.value = typeString;
                    break;
                }
            }
        }
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)viewWillAppear:(BOOL)animated
{
    [pidTableView reloadData];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return [pidList count];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 60;
}

//- (NSString*)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section{
//    return [mainMenu objectAtIndex:section];
//}


- (UITableViewCell*)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    HiveIAPShopViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"PidCell"];
    
    NSMutableDictionary *dict = [pidList objectAtIndex:indexPath.row];
    
    [cell setName:dict];
    
    return cell;
}

- (void)tableView:(UITableView*)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    //누르면 실행!!
    //상품 구매
    if(!isInappV4)
        param.value = [pidList objectAtIndex:indexPath.row][@"pid"];
    else
        param.value = [pidList objectAtIndex:indexPath.row][@"marketPid"];
    
    [GET_ACTION_CONTROLLER setCurrentAction:actionP];
    NSString* dictName = [[NSString alloc] initWithFormat:@"%@의 구매", [pidList objectAtIndex:indexPath.row][@"title"]];
    [GET_LOG_CONTROLLER setResultLog:dictName];
    [GET_ACTION_CONTROLLER callModuleApi];
}


- (IBAction)goBack:(id)sender {
        [GET_LOG_CONTROLLER clearLog:0];
        [self performSegueWithIdentifier:@"actionScene" sender:self];
}

- (IBAction)logMore:(id)sender {
        [GET_LOG_CONTROLLER setCurrentDepth:DEPTH_HIVEIAP];
        [self performSegueWithIdentifier:@"logView" sender:self];
}



@end




@implementation HiveIAPShopViewCell

- (void)setSelected:(BOOL)selected animated:(BOOL)animated
{
    [super setSelected:selected animated:animated];
}

-(void)setName:(NSMutableDictionary*)pidData
{
    _pidData = pidData;
    
    _pidName.text = pidData[@"title"];
    _pidCost.text = pidData[@"displayPrice"];
    _pid.text = pidData[@"pid"];
    _marketPid.text = pidData[@"marketPid"];
    
    //1.11.2 update
    NSString* startUTC = [self convertUTCToLocalTime:pidData[@"salesStartUTC"]];
    NSString* expireUTC = [self convertUTCToLocalTime:pidData[@"salesExpireUTC"]];
    
    _pidSalesTime.text = [[NSString alloc] initWithFormat:@"%@ ~ %@", startUTC, expireUTC];
    
    if([pidData[@"productType"] isEqualToString:@"series"])
        _pidSalesTime.textColor = [UIColor redColor];
    
    
    NSData *data = [NSData dataWithContentsOfURL:[NSURL URLWithString:pidData[@"productImageUrl"]]];
    _pidImg.image = [UIImage imageWithData:data];
    
}

-(NSString*)convertUTCToLocalTime:(NSNumber*)time
{
    NSString* convertTime;
    
    if(time==nil)
        return @"";
    
    if([time isEqual: @-1])
        return @"-1";
    
    NSTimeInterval interval = [time doubleValue]/1000;
    
    NSDate* ts_utc = [NSDate dateWithTimeIntervalSince1970:interval];
    NSDateFormatter* df_local = [[NSDateFormatter alloc]init];
    [df_local setTimeZone:[NSTimeZone timeZoneWithName:@"KOR"]];
    [df_local setDateFormat:@"yyyy.MM.dd HH:mm:ss"];
    
    convertTime = [df_local stringFromDate:ts_utc];
    

    return convertTime;
}

/*
- (IBAction)pidPurchaseButton:(id)sender {
    //상품 구매
    NSArray* actionList = [GET_ACTION_CONTROLLER getCurrentGroup][KEY_ACTIONLIST];
    APIAction *dict = [actionList objectAtIndex:5];
    Parameter* param =  dict.paramList[0];
    param.value = _pid.text;
    
    [GET_ACTION_CONTROLLER setCurrentAction:dict];
    NSString* dictName = [[NSString alloc] initWithFormat:@"%@의 구매", _pidName.text];
    [GET_LOG_CONTROLLER setResultLog:dictName];
    [GET_ACTION_CONTROLLER callModuleApi];
}
*/


@end
