//
//  AuthV4Tests.swift
//  HIVE_SDK_SampleUITests
//
//  Created by jhjang on 03/12/2018.
//  Copyright © 2018 gcp. All rights reserved.
//

import XCTest
@testable import HIVEBase
@testable import HIVEService

class AuthV4Tests: XCTestCase {
    
    var configurationInstance: HIVEService.Configuration!
    var instance: AuthV4!
    
    override func setUp() {
        // Put setup code here. This method is called before the invocation of each test method in the class.

        // In UI tests it is usually best to stop immediately when a failure occurs.
        continueAfterFailure = false

        // UI tests must launch the application that they test. Doing this in setup will make sure it happens for each test method.
        XCUIApplication().launch()

        // In UI tests it’s important to set the initial state - such as interface orientation - required for your tests before they run. The setUp method is a good place to do this.
        
        Logger.setLogEnable(true)
        self.configurationInstance = Configuration.shared(XMLParsable: XMLParseMock())
        self.instance = AuthV4.shared
    }

    override func tearDown() {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
    }
    
    func testSetup() {
        let sema = DispatchSemaphore(value: 0)
        instance.requestGetAgreement { response in
            guard let getAgreementResponse = response else {
                XCTAssert(false, "Get Agreement Response Empty.")
                sema.signal()
                return
            }
            
            XCTAssertTrue(getAgreementResponse.isSuccess())
            Logger.d("Agreement Response:  \(getAgreementResponse.contentDictionary)")
            let expectation = self.expectation(description: "AuthV4Setup")
            
            self.instance.setup(handler: { (result, isAutoSignIn, did, providerList) in
                Logger.d("""
                    result - \(result)
                    isAutoSignIn - \(isAutoSignIn)
                    did - \(String(describing: did))
                    providerList - \(String(describing: providerList))
                    """)
                XCTAssertTrue(result.isSuccess())
                
                expectation.fulfill()
            })
            
            self.waitForExpectations(timeout: 5, handler: nil)
            XCTAssertTrue(self.instance.authService.getAuthState() == .v4Initialized)
            XCTAssertNotNil(HIVEBase.Property.shared.getValue(key: .authV4Did))
            sema.signal()
        }
        sema.wait(timeout: DispatchTime.distantFuture)
    }

    func testSignInApple() {
        self.testSetup()
        
        let expectation = self.expectation(description: "AuthV4SignInApple")
        
        instance.signIn(providerType: .Apple) { (result, playerInfo) in
            Logger.d("""
                result - \(result)
                playerInfo - \(String(describing: playerInfo))
                """)
            XCTAssertTrue(result.isSuccess())
            expectation.fulfill()
        }
        
        waitForExpectations(timeout: 5, handler: nil)
        let playerInfo = self.instance.authService.playerInfo
        XCTAssertNotNil(playerInfo.playerID)
        XCTAssertTrue(playerInfo.playerID > 0)
    }
}

class XMLParseMock: XmlParsable {
    var xmlDefType: String?
    var xmlDictionary: [String: Any]?
    var xmlBundle: Bundle
    var xmlFileName: String
    
    init() {
        xmlBundle = Bundle(for: type(of: self))
        xmlFileName = "hive_config.xml"
        
        Logger.d("bundle identifier: \(String(describing: xmlBundle.bundleIdentifier))")
    }
}
