//
//  C2SHub2Unity.h
//  hive-ios-sdk
//
//  Created by Archaeo Lucid on 2014. 12. 29..
//  Copyright (c) 2014년 com2us. All rights reserved.
//

#ifdef __cplusplus
extern "C" {
#endif

typedef enum HiveSocialServerState
{
	HiveSocialServerLive	= 0,
	HiveSocialServerSandbox,
	HiveSocialServerTest,
	HiveSocialServerStaging
}HiveSocialServerState;
	
/**
 @abstract
 Hub(Hive) module을 초기화한다.
 @param appId Hub(Hive)등록시 받은 application id.
 @param gameIndex Hub(Hive)등록시 받은 application의 game index
 @param isTestServer 테스트 혹은 상용 server로 접속하기 위한 flag.
 @return Hub error code.
*/
// int CS_unity_hubInitialize(
// 	const char *appId,
// 	const char *gameIndex,
// 	bool isTestServer
// ) __deprecated_msg("Implement CS_unity_hubInitializeEx() instead");

/**
 @abstract
 Hub(Hive) module을 초기화한다.
 @param appId Hub(Hive)등록시 받은 application id.
 @param serverState SERVER_STATE_TYPE(release, staging, sandbox)
 @return Hub error code.
 */
int CS_unity_hubInitializeEx(
   const char *appId,
   HiveSocialServerState serverState
);
	
/**
 @abstract
 Hub(Hive)모듈을 해제한다.
 @return Hub error code.
 */
// int CS_unity_hubUninitialize(void) __deprecated_msg("Not Support");
	
	
/**
 @abstract
 Hub(Hive)로 사용자 인증을 요청한다.
 @param callbackObject callbackFunction
 @param callbackFunction
 @return Hub error code.
 @discussion
 server로부터 정상적으로 인증을 시도하게 되면 0(HUB_E_SUCCESS)을 반환한다. 이후 server로부터의 응답을 callbackFunction에서 받을 수 있다.
 */
int CS_unity_hubAuth(
	const char *callbackObject,
	const char *callbackFunction
);

/**
 @abstract
 Hub(Hive)로 사용자 인증을 요청한다. (param 추가)
 @param param 추가 옵션 param
 @param callbackObject callbackFunction
 @param callbackFunction
 @return Hub error code.
 @discussion
 server로부터 정상적으로 인증을 시도하게 되면 0(HUB_E_SUCCESS)을 반환한다. 이후 server로부터의 응답을 callbackFunction에서 받을 수 있다.
 */
int CS_unity_hubAuthWithParam(
    const char *param,
    const char *callbackObject,
    const char *callbackFunction
);
	
	
/**
 @abstract
 인증되어 있는 사용자를 logout을 요청한다.
 @param callbackObject
 @param callbackFunction
 @return Hub error code.
 @discussion
 server로부터 정상적으로 logout을 시도하게 되면 0(HUB_E_SUCCESS)을 반환한다. 이후 server로부터의 응답을 callbackFunction에서 받을 수 있다.
 */
int CS_unity_hubLogout(
	const char *callbackObject,
	const char *callbackFunction
);

/**
 @abstract
 인증되어 있는 사용자계정의 탈퇴를 요청한다.
 @param callbackObject
 @param callbackFunction
 @return Hub error code.
 @discussion
 server로부터 정상적으로 delete을 시도하게 되면 0(HUB_E_SUCCESS)을 반환한다. 이후 server로부터의 응답을 callbackFunction에서 받을 수 있다.
 */
int CS_unity_hubUserDelete(
    const char *callbackObject,
    const char *callbackFunction
);


/**
 @abstract
 특정 경로(ex. home, feed, ...)의 dialog를 표시한다.
 @param relativePath 접근하고자 하는 hive page 경로.
 @param callbackObject
 @param callbackFunction
 @return Hub error code.
 */
int CS_unity_hubShowDialog(
	const char *relativePath,
	const char *callbackObject,
	const char *callbackFunction
);

/**
 @abstract
 server로 비동기 요청을 한다.
 @param requestName 비동기 요청을 위한 요청 이름.
 @param params 비동기 요청시 필요한 parameter. The default is nil.
 @param callbackObject
 @param callbackFunction
 @return Hub error code.
 */
int CS_unity_hubAsyncRequest(
	const char *requestName,
	const char *param,
	const char *callbackObject,
	const char *callbackFunction
);
	
/**
 @abstract
 게스트 계정을 정식전환으로 요청시 사용된다.
 게스트 상태에서 함수를 호출하면, 정식 사용자 인증을 시도한다.
 인증에 성공하게 되면 해당 정식 사용자의 uid정보를 가져온다.
 @param callbackObject
 @param callbackFunction
 @return Hub error code.
 */
int CS_unity_hubGuestAcquireUid(
	const char *callbackObject,
	const char *callbackFunction
);

/**
 @abstract
 게스트 계정을 정식 사용자로 전환을 시도한다.
 @param guestUid 현재 로그인 되어 있는 게스트 계정의 uid.
 @param candidateUid 전환하고자 하는 정식 사용자의 uid.
 @param callbackObject
 @param callbackFunction
 @return Hub error code.
 @discussion
 이 함수를 호출하기 이전에 CS_hubGuestAcquireUid()을 호출하여 전환하고자 하는 정식 사용자의 uid정보를 server로부터 얻어와야 한다.
 */
int CS_unity_hubGuestBind(
	const char *guestUid,
	const char *candidateUid,
	const char *callbackObject,
	const char *callbackFunction
);
	

//// Social Extension
#define CSHUB_SOCIAL_SERVICE_FACEBOOK	"facebook"
#define CSHUB_SOCIAL_SERVICE_GOOGLEPLUS	"googleplus"

#define CSHUB_SOCIAL_USER_PROFILE		"social/request_user_profile"
#define CSHUB_SOCIAL_FRIENDS			"social/request_friends"
#define CSHUB_SOCIAL_SHARE				"social/share_app_activity"
#define CSHUB_SOCIAL_APP_MESSAGE		"social/send_app_message"
#define CSHUB_SOCIAL_APP_INVITATION		"social/send_app_invitation"
#define CSHUB_SOCIAL_INVITATION_LIST	"social/request_invitation_list"
#define CSHUB_SOCIAL_QUERY				"social/query"
#define CSHUB_SOCIAL_BUSINESS_MODEL		"social/business_model"
#define CSHUB_SOCIAL_INVITE_DIALOG		"social/invite_dialog"
#define CSHUB_SOCIAL_RECEIVED_INVITE	"social/received_invite"
#define CSHUB_SOCIAL_USER_PROFILE_IMAGE "social/user_profile_image"
#define CSHUB_SOCIAL_IS_AUTHORIZED		"social/is_authorized"
#define CSHUB_SOCIAL_LOGOUT				"social/logout"
#define CSHUB_SOCIAL_CONNECT			"social/connect"

/**
 @abstract
 Hub에서 지원되는 외부 소셜 서비스 기능을 사용한다.
 @param socialService 사용하고자 하는 외부 소셜 서비스의 이름. ex) facebook, googleplus, ...
 @param requestName 사용하고자 하는 외부 소셜 서비스의 기능 이름. ex)social/share_app_activity, ...
 @param params 서비스 사용시 필요한 parameter. JSON은 string format. The default is NULL.
 @param callbackObject
 @param callbackFunction
 @return Hub error code.
 @discussion
 요청시 hub server로의 요청이 아닌 외부 소셜 서비스의 서버로 요청한다.
 서비스에 따라 지원되지 않는 기능이 존재한다.
 */
int CS_unity_hubSocialRequest(
	const char *socialService,
	const char *actionName,
	const char *params,
	const char *callbackObject,
	const char *callbackFunction
);
	
/**
 @abstract
 Hive에서의 설정값이 변경할 때 사용가능하다.
 @param key allowed_key 값
 @param value 설정하고자 하는 key의 값
 @param Hive error code.
 @discussion
 hive module내 일부 설정 값을 변경하고자 할 때 사용한다.
 */
// int CS_unity_hubSetOption (
// 	const char *key,
// 	const char *value
// ) __deprecated_msg("Not Support");

/**
 @abstract
 @param
 */
int CS_unity_hubShowDialogForChatbot (
    const char *chatbotParam,
    const char *callbackObject,
    const char *callbackFunction
);

/**
 @abstract
 Hub(Hive)모듈 로그인 Dialog를 투명하게 노출한다.
 @return Hub error code.
 */
int CS_unity_hubSetAuthDialogClear(void);

#ifdef __OBJC__
//// Application Delegate
// void CS_unity_hubApplicationDidBecomeActive() __deprecated_msg("Replace C2SAppDelegate");

// bool CS_unity_hubApplicationOpenURL(
// 	NSURL *url,
// 	NSString *sourceApplication,
// 	id annotation
// ) __deprecated_msg("Replace C2SAppDelegate");
#endif // __OBJC__


#ifdef __cplusplus
}
#endif
