#ifndef _C2SINAPP_V4_H_
#define _C2SINAPP_V4_H_

// Last Update : C2S.21.0.2

#ifdef __cplusplus
extern "C" {
#endif

// Market Id
const static int MARKET_LIST_NOT_SELECTED = 0;
const static int MARKET_LIST_APPLE_APPSTORE = 1;
const static int MARKET_LIST_GOOGLE_PLAYSTORE = 2;
const static int MARKET_LIST_HIVE_LEBI = 3;
const static int MARKET_LIST_ONESTORE = 4;
const static int MARKET_LIST_AMAZON_APPSTORE = 5;
const static int MARKET_LIST_SAMSUNG_GALAXYSTORE = 6;
const static int MARKET_LIST_HUAWEI_APPGALLERY = 7;
const static int MARKET_LIST_FUNTAP = 8;
const static int MARKET_LIST_OPPO_APPMARKET = 9;
const static int MARKET_LIST_VIVO_APPSTORE = 10;
const static int MARKET_LIST_TENCENT_MYAPP = 11;
const static int MARKET_LIST_XIAOMI_APPSTORE = 12;
const static int MARKET_LIST_HUAWEI_APPGALLERY_CHINA = 13;
const static int MARKET_LIST_FACEBOOK_CLOUD_GAME = 14;
const static int MARKET_LIST_HIVESTORE = 15;
const static int MARKET_LIST_STEAM = 16;
const static int MARKET_LIST_NOWGG = 17;

// IapV4Lifecycle Type
const static int IAPV4_LIFE_CYCLE_ON_SETUP_FINISHED = 0;
const static int IAPV4_LIFE_CYCLE_ON_INITIALIZE_FINISHED = 1;
const static int IAPV4_LIFE_CYCLE_ON_SIGN_IN = 2;
const static int IAPV4_LIFE_CYCLE_ON_SIGN_OUT = 3;
const static int IAPV4_LIFE_CYCLE_ON_PROCESS_URI = 4;
const static int IAPV4_LIFE_CYCLE_ON_RESET = 5;


// [ResultAPI]
typedef struct CS_IapV4Result {
    int result;
    int errorCode;
    const char* errorMessage;
} CS_IapV4Result;

// [ProductInfo]
// iOS : Apple
// Android : Google, OneStore, Lebi, Huawei, Amazon
typedef struct CS_IapV4ProductInfo {
    const char* productType;
    const char* marketPid;
    const char* currency;
    double price;
    const char* displayPrice;           //
    const char* title;
    const char* description;
    const char* originalJson;
    const char* originalMarketJson;
    const char* displayOriginalPrice;
    double originalPrice;            //
    const char* iconUrl;
    int coinsReward;
} CS_IapV4ProductInfo;

// [ReceiptInfo]
typedef struct CS_IapV4ReceiptInfo {
    int type;
    const char* iapPayload;
    const char* bypassInfo;
    const char* hiveiapReceipt;
    int quantity;
    // ProductInfo
    const char* productType;
    const char* marketPid;
    const char* currency;
    double price;
    const char* displayPrice;           //
    const char* title;
    const char* description;
    const char* originalJson;
    const char* originalMarketJson;
    const char* displayOriginalPrice;
    double originalPrice;            //
    const char* iconUrl;
    int coinsReward;
} CS_IapV4ReceiptInfo;

// [PurchaseParam]
typedef struct CS_IapV4PurchaseParam {
    const char* marketPid;
    const char* oldMarketPid;
    const char* iapPayload;
    const char* offerToken;
} CS_IapV4PurchaseParam;

// [Callback]
typedef void (*MarketListCB)(CS_IapV4Result iapResult, int numOfMarketList, int* marketList);
typedef void (*ProductInfoCB)(CS_IapV4Result iapResult, int numOfProduct, CS_IapV4ProductInfo* productInfo);
typedef void (*ReceiptInfoCB)(CS_IapV4Result iapResult, CS_IapV4ReceiptInfo receiptInfo);
typedef void (*ReceiptListInfoCB)(CS_IapV4Result iapResult, int numOfReceipt, CS_IapV4ReceiptInfo* receiptInfo);
typedef void (*TransactionCB)(CS_IapV4Result iapResult, const char* marketPid);
typedef void (*BalanceCB)(CS_IapV4Result iapResult, int balance);
typedef void (*CheckPromotePurchaseCB)(CS_IapV4Result iapResult, const char* marketPid);
typedef void (*IapV4LifecycleCB)(int type, const char* data);
typedef void (*InAppOffersCB)(int responseCode);

// [API]
void CS_IapV4MarketConnect(MarketListCB cb);
void CS_IapV4GetMarketProductInfo(const char** pidList, ProductInfoCB cb);
void CS_IapV4GetProductInfo(ProductInfoCB cb);
void CS_IapV4GetSubscriptionProductInfo(ProductInfoCB cb);
void CS_IapV4Purchase(const char* marketPid, const char* iapPayload, ReceiptInfoCB cb);
void CS_IapV4PurchaseSubscription(const char* marketPid, const char* oldMarketPid, const char* iapPayload, ReceiptInfoCB cb);
void CS_IapV4PurchaseSubscriptionWithParam(CS_IapV4PurchaseParam param, ReceiptInfoCB cb);
void CS_IapV4Restore(ReceiptListInfoCB cb);
void CS_IapV4RestoreSubscription(ReceiptListInfoCB cb);
void CS_IapV4TransactionFinish(const char* marketPid, TransactionCB cb);
//void CS_IapV4TransactionMultiFinish();        // not support
void CS_IapV4CheckPromotePurchase(CheckPromotePurchaseCB cb);
int CS_IapV4GetSelectedMarket(void);
const char* CS_IapV4GetVersion(void);
const char* CS_IapV4GetAccountUuid(void);

// [API] Android Only
void CS_IapV4ShowMarketSelection(MarketListCB cb);
void CS_IapV4GetBalanceInfo(BalanceCB cb);
void CS_IapV4ShowCharge(BalanceCB cb);
void CS_IapV4SetLifecycleCB(IapV4LifecycleCB cb);

/**
 * Google Early Access Program In-App Offers. (Android Only API)
 * @param categoryId 3(PROMOTIONAL)
 * @param listener Int(responseCode) 가 2(NAVIGATE_TO_DEV_APP_STORE) 일 경우 유저가 제안(Offer)를 확인하고 사용하기를 원한 상황입니다.
 * 개발자는 유저가 구매할 인앱 항목을 선택할 수 있는 게임/앱 내 상점으로 (유저가 해당 화면에 없는 경우) 유저를 이동해야 합니다.
 * 0 은 아무런 액션을 하지 않아도 되고, 음수는 에러상황 입니다.
 */
void CS_IapV4ShowInAppMessages(int categoryId, InAppOffersCB cb);

// +++++ +++++ +++++ +++++ +++++
// Deprecated (removed)
// +++++ +++++ +++++ +++++ +++++
// void CS_IapV4SetUid(const char* uid) __deprecated_msg("Not support");

#ifdef __cplusplus
};
#endif

#endif    // _C2SINAPP_V4_H_
