/**
 * @file    C2SModuleErrorCode.h
 
 * @brief   통합 모듈의 에러 코드를 정의해놓은 파일
 
 * @author  Byeong Yeol Jeon
 * @ingroup iOS Android Cocos2d-x
 * @date    2014/04/30
 */
#ifndef _C2SMODULEERRORCODE_H_
#define _C2SMODULEERRORCODE_H_

/**
 * @brief 에러코드의 타입
 
 * @author Byeong Yeol Jeon
 * @ingroup iOS Android Cocos2d-x
 */
typedef enum _C2SModuleErrorCode {
    /** 성공 */
	C2SModuleErrorCode_Success = 0,
    /** 실패 */
	C2SModuleErrorCode_Fail = -1,
    /** 인자가 잘못됨 */
	C2SModuleErrorCode_InvalidArg = -2,
    /** 요청이 실패함 */
    C2SModuleErrorCode_FailOperation = -3,
    /** 요청이 취소됨 */
    C2SModuleErrorCode_CancelOperation = -4,
    /** 초기화가 실패함 */
    C2SModuleErrorCode_FailInitialization = -5,
    /** 지원되지 않음 */
    C2SModuleErrorCode_NotSupported = -6,
    /** 이미 요청중이거나 요청할 수 없는 상태 */
    C2SModuleErrorCode_OperationPending = -29,
    /** 안드로이드 권한 에러 **/
    C2SModuleErrorCode_PermissionDenied = -99,
    /** 정식 로그인이 필요함 */
    C2SModuleErrorCode_NeedLogin = -200,
    /** 게스트 로그인이 필요함 */
    C2SModuleErrorCode_NeedGeustLogin = -202,
    /** Restore가 필요 */
    C2SModuleErrorCode_NeedRestore = -300,
	/** 네트워크 통신 장애 */
    C2SModuleErrorCode_NetworkError = -301,
	/** 통신 성공후 서버 결과가 fail일 경우 */
    C2SModuleErrorCode_ServerResponseFail = -302,
	/** purchase/check 에서 서버응답이 리젝(유저 블락 등)일 경우, 통신 실패 아님 */
    C2SModuleErrorCode_PurchaseStateUnavilable = -303,
	/** 마켓구매 성공 후, 모듈서버와 통신에서 실패시, restore 필요 */
    C2SModuleErrorCode_PurchaseTransactionError = -304,
	/** 이니셜라이즈가 필요함 */
    C2SModuleErrorCode_NeedInitialize = -305,
    /** restore 아이템이 없음 */
    C2SModuleErrorCode_RestoreNotOwnedItem = -306,
    /** 요청 조건에 맞는 아이템이 없음 */
    C2SModuleErrorCode_NotOwned = -306,
    /** 잘못된 json 문자열 형태 */
    C2SModuleErrorCode_InvalidJsonFormat = -4000,
    /** json문자열 형태로 변환할 수 없음 */
    C2SModuleErrorCode_FailConvertToJsonFormat = -4001,
    /** InApp 구입 차단 설정 */
    C2SModuleErrorCode_FailCanMakePayments = -5000,
    /** 데이터의 무결성이 깨짐 */
    C2SModuleErrorCode_Broken = -8001,
    /** 메모리를 할당받을 수 없음 */
    C2SModuleErrorCode_MemoryExceed = -8002,
    /** 알 수 없는 에러 */
	C2SModuleErrorCode_Unknown = -1444445,
    
} C2SModuleErrorCode;

#endif
