﻿/**
 * @file    cscrc32.h
 
 * @brief   cscrc32를 계산하는 클래스의 헤더
 
 * @author  Byeong Yeol Jeon
 * @ingroup iOS Android Cocos2d-x Protection
 * @date    2014/04/24
 */
/**
 * @defgroup Protection
 */
#ifndef _CSCRC32_H_
#define _CSCRC32_H_

#include <stdint.h>
/**
 * @class CSCRC32
 
 * @brief  CRC를 계산한다.
 * @ingroup iOS Android Cocos2d-x Protection
 * @author Byeong Yeol Jeon
 */
class CSCRC32
{
public:
    /**
     * @brief default constructor
     
     * @author Byeong Yeol Jeon
     
     */
    CSCRC32();
    
    /**
     * @brief crc를 업데이트한다.
     * @param[out] outCrc
     * @param[in] crc_accum
     * @param[in] data
     * @param[in] length
     * @return true일 경우 성공. false일 경우 실패.
     * @author Byeong Yeol Jeon
     */
    bool update(uint32_t& outCrc, uint32_t crc_accum, uint8_t* data, uint32_t length);
	
private:
    /**
     * @brief crc 테이블을 생성한다.
     
     * @author Byeong Yeol Jeon
     
     */
	void gen_table();
    
    /** 테이블 정보 변수 */
    uint32_t m_secureCRCtable[256];
    
    /**  copy prevent */
    CSCRC32(const CSCRC32&);
    /**  copy prevent */
    CSCRC32& operator=(const CSCRC32&);
};

#endif
